Package: orisma
Type: Package
Title: Occupational Risk Integrated Systematic Mapping and Analysis
Version: 0.1.0
Authors@R: c(
    person(
        given = "Raúl",
        family = "Aguilar Elena",
        role = c("aut", "cre"),
        email = "raguilar@universidadviu.com",
        comment = c(ORCID = "0000-0001-8656-8155")
    ),
    person(
        given = "Ana",
        family = "Delgado-Garcia",
        role = "aut",
        email = "a.delgado@usal.es"
    )
    )
Description: A complete pipeline for systematic bibliometric mapping of
    occupational health and safety (OHS) evidence. Starting from reference
    files exported from major bibliographic databases such as Web of Science,
    Scopus, PubMed, Dimensions, EBSCO, and others, 'orisma' automates
    ingestion, deduplication, relevance filtering, occupational risk category
    extraction, bibliometric analysis, and report generation. The package is
    related to bibliometric science mapping and evidence synthesis workflows
    described by Aria and Cuccurullo (2017) <doi:10.1016/j.joi.2017.08.007>,
    Westgate (2019) <doi:10.1002/jrsm.1374>, and Lajeunesse (2016)
    <doi:10.1111/2041-210X.12472>, but adds a domain-specific occupational
    safety and health layer.
    The package implements three original bibliometric indicators:
    (1) the Worker-Risk Disconnection Index (WRDI), measuring the
    proportion of studies that characterise an occupational risk without
    including direct worker exposure data; (2) the Risk Category Saturation
    Index (RCS), measuring the relative over- or under-representation of
    each risk category relative to a uniform baseline; and (3) the
    Material-Gap Profile (MGP), measuring the ratio between a material's
    known hazard potential and its coverage in the occupational health
    literature.
    Two additional preventive intelligence indicators are provided:
    (4) the Abstract Sufficiency Score (ASS, 0-5), a cumulative hierarchical
    index of the preventively useful information contained in an abstract;
    and (5) the Bridge Article Score (0-5), identifying studies that
    simultaneously address technology, hazardous agent, worker population,
    exposure measurement, and preventive recommendations.
    Risk categories are extracted using a built-in occupational risk
    dictionary of 58 categories anchored in ISO 45001:2018, INSST, NIOSH,
    and EU-OSHA frameworks, organised in six blocks: Safety, Industrial
    Hygiene, Ergonomics, Psychosociology, Biological Hazards, and Emerging
    Technologies. The dictionary is user-extensible.
    Outputs include bilingual HTML reports, occupational risk sheets,
    priority reading rankings, guided extraction matrices for systematic
    review, and reproducibility certificates with MD5 hashes.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.3
URL: https://github.com/Aguilar-Elena/orisma
BugReports: https://github.com/Aguilar-Elena/orisma/issues
Depends: R (>= 4.1.0)
Imports: cli (>= 3.4.0), digest (>= 0.6.29), dplyr (>= 1.1.0), ggplot2
        (>= 3.4.0), ggrepel (>= 0.9.0), glue (>= 1.6.0), jsonlite (>=
        1.8.0), magrittr (>= 2.0.0), pheatmap (>= 1.0.12), readr (>=
        2.1.0), stringdist (>= 0.9.8), stringr (>= 1.5.0), synthesisr
        (>= 0.3.0), tidyr (>= 1.3.0), tools, utils
Suggests: knitr, rmarkdown, rsvg, testthat (>= 3.0.0)
VignetteBuilder: knitr
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-12 20:14:07 UTC; okashi
Author: Raúl Aguilar Elena [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8656-8155>),
  Ana Delgado-Garcia [aut]
Maintainer: Raúl Aguilar Elena <raguilar@universidadviu.com>
Repository: CRAN
Date/Publication: 2026-05-18 18:10:09 UTC
Built: R 4.5.2; ; 2026-05-18 19:44:56 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,orisma_dict)
S3method(print,orisma_dims)
S3method(print,orisma_kappa)
S3method(print,orisma_matrix)
S3method(print,orisma_normativa)
S3method(print,orisma_priority)
S3method(print,orisma_ranking)
S3method(print,orisma_result)
export(orm_analyse)
export(orm_ass)
export(orm_ass_plot)
export(orm_autodim)
export(orm_bridge)
export(orm_dedup)
export(orm_dict)
export(orm_dict_add_category)
export(orm_dict_add_terms)
export(orm_dict_categories)
export(orm_dict_list)
export(orm_dim_matrix)
export(orm_extract)
export(orm_extraction_matrix)
export(orm_load)
export(orm_normativa)
export(orm_priority)
export(orm_ranking)
export(orm_relevance_guard)
export(orm_report)
export(orm_risk_sheet)
export(orm_run)
export(orm_run_guarded)
export(orm_validate)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_alert_warning)
importFrom(cli,cli_h1)
importFrom(cli,cli_h2)
importFrom(cli,cli_progress_bar)
importFrom(cli,cli_progress_done)
importFrom(cli,cli_progress_update)
importFrom(cli,cli_rule)
importFrom(digest,digest)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_colour_gradient)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggrepel,geom_text_repel)
importFrom(glue,glue)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(jsonlite,read_json)
importFrom(jsonlite,write_json)
importFrom(magrittr,"%>%")
importFrom(readr,read_csv)
importFrom(readr,write_csv)
importFrom(stats,na.omit)
importFrom(stats,reorder)
importFrom(stats,setNames)
importFrom(stringr,str_escape)
importFrom(stringr,str_extract)
importFrom(tidyr,pivot_longer)
importFrom(tools,file_ext)
importFrom(utils,head)
importFrom(utils,packageVersion)

# orisma <img src="man/figures/logo.png" align="right" height="120" alt="orisma logo"/>

**Occupational Risk Integrated Systematic Mapping and Analysis**

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/orisma)](https://CRAN.R-project.org/package=orisma)
[![R-CMD-check](https://github.com/Aguilar-Elena/orisma/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Aguilar-Elena/orisma/actions)
[![License: MIT](https://img.shields.io/badge/License-MIT-green.svg)](https://opensource.org/licenses/MIT)
<!-- badges: end -->

`orisma` is an R package for **systematic bibliometric mapping of occupational risk evidence**.

It is designed for researchers, occupational safety and health professionals, industrial hygienists, ergonomists, psychosocial risk specialists and prevention practitioners who need to understand whether the scientific literature on a given topic is actually connected to **workers, workplaces, exposure conditions and preventive decision-making**.

Unlike general bibliometric tools, `orisma` focuses on the **preventive usefulness** of scientific evidence. It does not only count publications or keywords. It helps identify whether a research field is technically abundant but weakly connected to real occupational exposure, workplace tasks or preventive action.

<hr>

## Why ORISMA?

Emerging technologies, new work processes and complex occupational hazards often generate a large scientific literature before their real workplace risks are fully understood.

This creates a practical and methodological problem:

> A topic may appear well studied, but the available evidence may still lack data on workers, real exposure conditions, tasks, sectors, controls or preventive recommendations.

`orisma` was created to detect this gap.

It helps answer questions such as:

- Is the literature technically abundant but weakly connected to real workers?
- Which occupational risk categories are over-represented or under-represented?
- Which articles are most useful for occupational risk assessment?
- Which studies connect technical science with applied prevention?
- Which risks require on-site assessment because the literature lacks worker-level evidence?
- Which records are probably off-topic, biomedical, clinical or weakly occupational and should be reviewed manually?

<hr>

## What does ORISMA do?

Starting from reference files exported from major bibliographic databases such as Web of Science, Scopus, PubMed, Dimensions, EBSCO and others, `orisma` runs a complete workflow.

Processing time depends on corpus size, file format, deduplication complexity and the number of risk categories analysed.

1. **Ingestion** — reads RIS, BibTeX and CSV files from multiple databases.
2. **Deduplication** — applies a three-step pipeline: exact DOI, normalised title and fuzzy matching.
3. **Relevance guard** — flags or excludes records with weak topic or occupational relevance.
4. **Risk extraction** — scans titles, abstracts and keywords against a 58-category occupational risk dictionary.
5. **Bibliometric analysis** — generates matrices, temporal trends, co-occurrence structures and risk distributions.
6. **Preventive indicators** — computes WRDI, RCS, MGP, ASS and Bridge Article Score.
7. **Priority ranking** — identifies articles with higher preventive usefulness.
8. **Reports** — generates bilingual HTML reports, practitioner risk sheets, extraction matrices and validation samples.

<hr>

## Main preventive bibliometric indicators

| Indicator | Full name | What it measures |
|---|---|---|
| **WRDI** | Worker-Risk Disconnection Index | Proportion of studies characterising a risk without direct worker exposure data |
| **RCS** | Risk Category Saturation Index | Relative dominance of each risk category compared with a uniform baseline |
| **MGP** | Material-Gap Profile | Ratio between a material's hazard potential and its coverage in the occupational health literature |
| **ASS** | Abstract Sufficiency Score | Amount of preventively useful information contained in each abstract, scored from 0 to 5 |
| **Bridge score** | Bridge Article Score | Degree to which a study connects technical science with applied occupational prevention |

<hr>

## Worker-Risk Disconnection Index (WRDI)

The **Worker-Risk Disconnection Index** measures the proportion of studies that characterise a risk without reporting direct data on workers or workplace exposure.

A high WRDI suggests that the literature is technically developed but weakly connected to real working conditions.

| WRDI value | Interpretation |
|---|---|
| 0.00-0.30 | Reasonable connection with worker-level evidence |
| 0.30-0.70 | Partial disconnection; manual review recommended |
| 0.70-1.00 | High technical-worker disconnection; on-site assessment is especially important |

WRDI is not a substitute for expert judgement. It is a signal that helps prioritise deeper review.

<hr>

## Risk Category Saturation Index (RCS)

The **Risk Category Saturation Index** measures whether a risk category is over-represented or under-represented compared with a uniform distribution across the dictionary.

It helps identify:

- dominant risk categories in the corpus;
- under-studied risk categories;
- risk areas where literature volume may not match preventive relevance;
- potential evidence gaps.

<hr>
## Material-Gap Profile (MGP)

The **Material-Gap Profile** is designed for corpora where records can be stratified by material, substance or agent.

It helps identify materials or agents that appear hazardous but remain poorly covered in the occupational health literature.

This is especially useful for topics such as:

- metal additive manufacturing;
- nanomaterials;
- battery manufacturing;
- advanced materials;
- chemical agents;
- biological agents;
- emerging technologies.

<hr>

## Abstract Sufficiency Score (ASS)

The **ASS** is a cumulative 0-5 score measuring how much preventively useful information an abstract contains.

| Score | Meaning |
|---|---|
| 0 | Non-informative for OHS purposes |
| 1 | Mentions a hazard but no occupational context |
| 2 | Mentions occupational or workplace context |
| 3 | Mentions exposure measurement or quantification |
| 4 | Mentions worker exposure with a result |
| 5 | Complete preventive abstract: worker population, exposure measurement, method and prevention |

The ASS is not a measure of study quality. It is a measure of how informative the abstract is for occupational prevention.

<hr>

## Bridge articles

A **bridge article** connects technical science with applied occupational prevention.

It usually combines:

1. A technology, process or work task.
2. A hazardous agent or risk factor.
3. A real worker population or workplace setting.
4. Exposure measurement or workplace assessment.
5. Preventive recommendations or control measures.

Bridge articles are useful because they help practitioners move from general scientific evidence to concrete preventive action.

<hr>

## Installation

```r
# From CRAN (once published)
install.packages("orisma")

# Development version from GitHub
# install.packages("remotes")
remotes::install_github("Aguilar-Elena/orisma")
```

<hr>

## Minimal usage — 3 lines

```r
library(orisma)

refs   <- orm_load("my_references/")   # load RIS/BibTeX/CSV files
result <- orm_run(refs)                 # full pipeline (2-3 sec)
orm_report(result, lang = "en")         # generate all outputs
```

For **Spanish output**:

```r
options(orisma.lang = "es")
refs   <- orm_load("mis_referencias/")
result <- orm_run(refs)
orm_report(result, lang = "es", out_dir = "resultados/")
```

<hr>

## Complete function reference

| Function | For whom | What it does |
|---|---|---|
| `orm_load()` | Everyone | Multi-source ingestion with format auto-detection |
| `orm_dedup()` | Everyone | Three-step deduplication: DOI, title and fuzzy matching |
| `orm_relevance_guard()` | Both | Flags or excludes records with weak topic or occupational relevance |
| `orm_extract()` | Researcher | Risk category extraction via occupational risk dictionary |
| `orm_analyse()` | Researcher | Computes WRDI, RCS and MGP |
| `orm_autodim()` | Researcher | Automatic dimension discovery |
| `orm_dim_matrix()` | Researcher | Risk x dimension heatmap |
| `orm_ass()` | Both | Abstract Sufficiency Score per record |
| `orm_ass_plot()` | Both | ASS distribution plot |
| `orm_bridge()` | Both | Bridge article detection and classification |
| `orm_ranking()` | Both | Priority reading list |
| `orm_priority()` | Both | RED/AMBER/GREEN/GREY priority classification |
| `orm_run()` | Everyone | Complete ORISMA pipeline in one call |
| `orm_run_guarded()` | Everyone | Complete pipeline with relevance-control layer |
| `orm_report()` | Researcher | Full HTML report with visualisations and tables |
| `orm_risk_sheet()` | OHS practitioner | Actionable risk sheet |
| `orm_extraction_matrix()` | Both | Guided extraction template for PDF review |
| `orm_validate()` | Researcher | Manual validation sample |
| `orm_dict()` | Everyone | Load or customise the risk dictionary |

<hr>

## Outputs generated automatically

After running `orm_report()` and `orm_risk_sheet()`:

### For researchers
| File | Description |
|---|---|
| `orisma_report.html` | Interactive bilingual executive report with 7 plots |
| `orisma_corpus.csv` | All records after deduplication |
| `orisma_matrix.csv` | Binary risk category matrix (records x categories) |
| `orisma_indicators.csv` | WRDI, RCS, MGP per category |
| `prisma_log.csv` | PRISMA-compatible selection flow |
| `analysis.orisma` | Reproducibility certificate (JSON with MD5 hashes) |
| `plots/` | 7 publication-ready PNG plots |

### For OHS practitioners
| File | Description |
|---|---|
| `orisma_risk_sheet.html` | Actionable risk sheet with RED/AMBER/GREEN traffic light |
| `orisma_extraction_matrix.csv` | Pre-filled extraction template for PDF review |
| `orisma_priority_ranking.csv` | Top-20 priority articles by bridge + ASS score |
| `orisma_validation_sample.csv`| Manual validation sample |
<hr>

## Risk dictionary

The built-in dictionary covers **58 occupational risk categories** in **6 blocks**.

| Block | Area | Examples |
|---|---|---|
| A | Safety at work | Falls, collision, fire, explosion, work equipment |
| B | Industrial hygiene | Chemical agents, dust, noise, vibration, radiation |
| C | Ergonomics | Postures, manual handling, repetitive movements, workload |
| D | Psychosociology | Mental workload, autonomy, social support, violence, harassment |
| E | Biological hazards | Bacteria, viruses, fungi, parasites, biological agents |
| F | Emerging technologies | Robotics, AI, nanotechnology, additive manufacturing, wearables |

The dictionary can be extended for any domain:

```r
dict <- orm_dict()

# Add terms to an existing category
dict <- orm_dict_add_terms(dict, "nanomaterials", c("nano-aerosol", "NOAA"))

# Add a completely new category
dict <- orm_dict_add_category(dict,
  key      = "exoskeleton_risk",
  label_en = "Exoskeleton-related musculoskeletal risk",
  label_es = "Riesgo musculoesqueletico por exoesqueleto",
  terms    = c("exoskeleton", "powered exosuit", "wearable robot")
)
```

<hr>

## Supported databases

| Database | Recommended format | Batch limit |
|---|---|---|
| Web of Science | RIS (Plain text) | 1 000 |
| Scopus | RIS or CSV | 2 000 |
| PubMed | RIS | No limit |
| Dimensions | CSV or RIS | 2 500 |
| EBSCO (CINAHL, BSC) | RIS | 25 000 |
| ProQuest | RIS or BibTeX | 100 |
| Cochrane Library | RIS | No limit |
| Ovid / MEDLINE | RIS | 1 000 |
| ScienceDirect | RIS | No limit |
| The Lens (free) | RIS or CSV | No limit |

Export all databases in **RIS format**, place files in a folder, and run `orm_load("folder/")`. ORISMA detects the source database automatically from the filename.

<hr>

## Abstract Sufficiency Score (ASS)

The ASS is a cumulative 0-5 index measuring how much preventively useful information an abstract contains:

| Score | Meaning |
|---|---|
| 0 | Non-informative for OHS purposes |
| 1 | Mentions a hazard but no occupational context |
| 2 | Mentions occupational/workplace context |
| 3 | Mentions exposure measurement or quantification |
| 4 | Mentions exposure in workers with a result |
| 5 | Complete: exposure + worker population + method + prevention |

<hr>

## Bridge articles

A **bridge article** connects technical science with applied OHS prevention. It simultaneously addresses:

1. Technology or process
2. Hazardous agent
3. Workers (real workplace population)
4. Exposure measurement
5. Preventive recommendation

Articles meeting 4-5 criteria = **Strong bridge** (highest priority for reading).
Articles meeting 3 criteria (must include workers + measurement) = **Partial bridge**.

<hr>

## Methodological note

ORISMA uses dictionary-based automatic classification. This may produce false positives. Manual validation of a representative sample is recommended using `orm_validate()`, which computes Cohen's Kappa between automatic and manual classification. A Kappa >= 0.7 is acceptable for high-impact journal publication.

ORISMA does not include country-specific regulations or limit values, as these vary by jurisdiction. The practitioner applies the relevant national/regional regulation based on the risk categories identified.

<hr>

## Limitations

ORISMA relies primarily on bibliographic metadata, titles, abstracts and keywords. It may miss information that appears only in the full text.

Automatic classification may produce false positives or false negatives, especially when terms are used differently across disciplines. This is why ORISMA includes a relevance guard and a validation workflow.

WRDI, ASS and Bridge Score should be interpreted as prioritisation and mapping indicators, not as definitive quality assessment tools.

Country-specific legal requirements, occupational exposure limits and regulatory thresholds are not embedded in ORISMA because they vary by jurisdiction. Practitioners should apply the relevant national or regional legislation after identifying the risk categories.



<hr>

## Citation

If you use `orisma` in your research, please cite:

```
Aguilar-Elena, R. & Delgado-Garcia, A. (2025). orisma: Occupational Risk
Integrated Systematic Mapping and Analysis. R package version 0.1.0.
Universidad Internacional de Valencia (VIU) & Universidad de Salamanca (USAL).
https://github.com/Aguilar-Elena/orisma
```

<hr>

## Authors

**PhD. Raul Aguilar-Elena** &middot; raguilar@universidadviu.com  
Occupational Risk Prevention and Occupational Health Research Group (GPRL)  
Universidad Internacional de Valencia (VIU), Valencia, Spain

**Ana Delgado-Garcia** &middot; a.delgado@usal.es  
Universidad de Salamanca (USAL), Salamanca, Spain

<hr>

## License

MIT © 2025 Raul Aguilar-Elena & Ana Delgado-Garcia

#' Load bibliographic references from one or multiple files / folders
#'
#' @description
#' `orm_load()` is the entry point of every ORISMA analysis. It reads
#' bibliographic files in **RIS**, **BibTeX**, or **CSV** format from a folder
#' (or a vector of individual file paths), detects the format of each file
#' automatically, combines all records into a single tidy data frame, and
#' records the source database for each record.
#'
#' All major bibliographic databases export to at least one supported format:
#'
#' | Database | Recommended format | Notes |
#' |---|---|---|
#' | Web of Science | RIS / Plain text | Max 1 000 records per batch |
#' | Scopus | RIS or CSV | Max 2 000 records per batch |
#' | PubMed | RIS | No limit |
#' | Dimensions | CSV or RIS | Max 2 500 per batch |
#' | EBSCO (CINAHL, BSC) | RIS | Up to 25 000 |
#' | ProQuest | RIS or BibTeX | Max 100 per batch |
#' | Cochrane Library | RIS | No limit |
#' | Ovid / MEDLINE | RIS | Max 1 000 per batch |
#' | ScienceDirect | RIS | No limit |
#' | The Lens (free) | RIS or CSV | No limit |
#'
#' @param path Character. Path to a **folder** containing reference files, or a
#'   **character vector** of individual file paths.
#' @param lang Character. Language for console messages: `"en"` (default) or
#'   `"es"`. Overrides `getOption("orisma.lang")`.
#' @param verbose Logical. Print progress messages? Default `TRUE`.
#'
#' @return A tibble (class `orisma_refs`) with standardised columns:
#'   \describe{
#'     \item{`record_id`}{Internal unique identifier assigned by ORISMA}
#'     \item{`source_file`}{Name of the original file}
#'     \item{`source_db`}{Database inferred from file name or format}
#'     \item{`title`}{Article title}
#'     \item{`authors`}{Authors (semicolon-separated)}
#'     \item{`year`}{Publication year}
#'     \item{`doi`}{Digital Object Identifier (if available)}
#'     \item{`abstract`}{Abstract text}
#'     \item{`keywords`}{Author keywords}
#'     \item{`journal`}{Journal name}
#'     \item{`volume`, `issue`, `pages`}{Bibliographic location}
#'     \item{`document_type`}{Article, review, conference paper, etc.}
#'   }
#'
#' @examples
#' \dontrun{
#' # Load all .ris and .bib files from a folder
#' refs <- orm_load("my_references/")
#'
#' # Load specific files
#' refs <- orm_load(c("wos_results.ris", "scopus_results.csv"))
#'
#' # Spanish messages
#' refs <- orm_load("mis_referencias/", lang = "es")
#' }
#'
#' @export
orm_load <- function(path, lang = getOption("orisma.lang", "en"),
                     verbose = getOption("orisma.verbose", TRUE)) {

  .check_lang(lang)

  if (verbose) cli::cli_h1(orm_msg("phase_load", lang))

  # ── 1. Resolve file list ────────────────────────────────────────────────────
  files <- .resolve_files(path, verbose, lang)
  if (length(files) == 0) {
    cli::cli_alert_danger(orm_msg("load_no_files", lang, path = path))
    cli::cli_alert_info(orm_msg("load_fmt_hint", lang))
    return(invisible(NULL))
  }

  formats <- unique(tools::file_ext(files))
  if (verbose) {
    cli::cli_alert_info(orm_msg("load_files", lang,
                             n = length(files),
                             formats = paste(formats, collapse = ", ")))
  }

  # ── 2. Read each file ───────────────────────────────────────────────────────
  records_list <- lapply(seq_along(files), function(i) {
    f   <- files[[i]]
    ext <- tolower(tools::file_ext(f))
    db  <- .infer_database(f)

    raw <- tryCatch(
      switch(ext,
        ris = synthesisr::read_refs(f),
        bib = synthesisr::read_refs(f),
        csv = .read_csv_refs(f),
        {
          cli::cli_alert_warning(paste0("Skipping unsupported file: ", basename(f)))
          return(NULL)
        }
      ),
      error = function(e) {
        cli::cli_alert_warning(paste0("Could not read: ", basename(f), " - ", e$message))
        NULL
      }
    )

    if (is.null(raw) || nrow(raw) == 0) return(NULL)

    # Standardise column names
    raw <- .standardise_columns(raw)
    raw$source_file <- basename(f)
    raw$source_db   <- db
    raw
  })

  # ── 3. Combine ──────────────────────────────────────────────────────────────
  records_list <- Filter(Negate(is.null), records_list)
  if (length(records_list) == 0) {
    cli::cli_alert_danger("No records could be parsed from the provided files.")
    return(invisible(NULL))
  }

  records_list <- lapply(records_list, function(x) dplyr::mutate(x, dplyr::across(everything(), as.character)))
  combined <- dplyr::bind_rows(records_list)

  # Assign internal IDs
  combined$record_id <- paste0("ORM", sprintf("%05d", seq_len(nrow(combined))))

  # Reorder columns
  combined <- .reorder_columns(combined)

  # Set class
  class(combined) <- c("orisma_refs", "tbl_df", "tbl", "data.frame")
  attr(combined, "orisma_stage")   <- "loaded"
  attr(combined, "orisma_lang")    <- lang
  attr(combined, "orisma_created") <- Sys.time()
  attr(combined, "n_sources")      <- length(files)

  if (verbose) {
    cli::cli_alert_success(orm_msg("load_done", lang,
                               n_total   = nrow(combined),
                               n_sources = length(files)))
  }

  combined
}


# ── Internal helpers ──────────────────────────────────────────────────────────

#' @noRd
.resolve_files <- function(path, verbose, lang) {
  supported_ext <- c("ris", "bib", "csv")

  if (length(path) == 1 && dir.exists(path)) {
    all_files <- list.files(path, full.names = TRUE, recursive = FALSE)
    files <- all_files[tolower(tools::file_ext(all_files)) %in% supported_ext]
  } else {
    files <- path[file.exists(path) &
                    tolower(tools::file_ext(path)) %in% supported_ext]
    missing <- path[!file.exists(path)]
    if (length(missing) > 0) {
      cli::cli_alert_warning(paste0("Files not found: ",
                                    paste(basename(missing), collapse = ", ")))
    }
  }
  files
}

#' @noRd
.infer_database <- function(filepath) {
  fname <- tolower(basename(filepath))
  dplyr::case_when(
    grepl("wos|web.of.science|savedrecs", fname) ~ "Web of Science",
    grepl("scopus",                        fname) ~ "Scopus",
    grepl("pubmed|medline",                fname) ~ "PubMed",
    grepl("dimensions",                    fname) ~ "Dimensions",
    grepl("cochrane",                      fname) ~ "Cochrane",
    grepl("proquest",                      fname) ~ "ProQuest",
    grepl("ebsco|cinahl",                  fname) ~ "EBSCO",
    grepl("lens",                          fname) ~ "The Lens",
    grepl("ovid",                          fname) ~ "Ovid",
    TRUE                                          ~ "Unknown"
  )
}

#' @noRd
.read_csv_refs <- function(filepath) {
  raw <- readr::read_csv(filepath, show_col_types = FALSE, progress = FALSE)
  # Scopus CSV uses specific column names; normalise to synthesisr-like names
  names(raw) <- tolower(gsub("\\s+", "_", names(raw)))
  raw
}

#' @noRd
.standardise_columns <- function(df) {
  # Map common column name variants to ORISMA standard names
  col_map <- c(
    title        = "title|TI|article.title|Article.Title",
    authors      = "author|AU|authors|Authors",
    year         = "year|PY|publication.year|Year",
    doi          = "doi|DO|DOI",
    abstract     = "abstract|AB|Abstract",
    keywords     = "keywords|DE|author.keywords|Author.Keywords",
    journal      = "journal|SO|source.title|Source.Title",
    volume       = "volume|VL|Volume",
    issue        = "issue|IS|Issue",
    pages        = "pages|BP|EP|Page.Start",
    document_type = "document.type|DT|Document.Type"
  )

  df_names <- names(df)
  for (std_name in names(col_map)) {
    pattern  <- col_map[[std_name]]
    matched  <- grep(paste0("^(", pattern, ")$"), df_names,
                     ignore.case = TRUE, value = TRUE)
    if (length(matched) > 0 && !std_name %in% df_names) {
      names(df)[names(df) == matched[[1]]] <- std_name
    }
  }

  # Ensure all standard columns exist (fill with NA if missing)
  std_cols <- names(col_map)
  for (col in std_cols) {
    if (!col %in% names(df)) df[[col]] <- NA_character_
  }

  dplyr::as_tibble(df)
}

#' @noRd
.reorder_columns <- function(df) {
  priority <- c("record_id", "source_db", "source_file",
                "title", "authors", "year", "doi",
                "abstract", "keywords", "journal",
                "volume", "issue", "pages", "document_type")
  rest     <- setdiff(names(df), priority)
  present  <- intersect(priority, names(df))
  df[, c(present, rest)]
}

#' @noRd
.check_lang <- function(lang) {
  if (!lang %in% c("en", "es")) {
    stop(orm_msg("err_lang", "en", lang = lang), call. = FALSE)
  }
}

#' Cross-reference detected risks with applicable European regulation
#'
#' @description
#' `orm_normativa()` crosses the risk categories detected by ORISMA with
#' the main applicable European directives and ISO standards, providing
#' the occupational health practitioner with a direct regulatory anchor
#' for each identified risk.
#'
#' The regulatory database is built into ORISMA and covers EU directives,
#' Spanish INSST technical notes (NTP), and key ISO standards. It is
#' updated with each major package release.
#'
#' @param result An `orisma_result` object.
#' @param min_records Integer. Min records for a category to be included.
#'   Default `1`.
#' @param lang Character. `"en"` or `"es"`.
#'
#' @return A data frame with detected categories and their applicable
#'   regulations.
#' @export
orm_normativa <- function(result,
                           min_records = 1L,
                           lang        = getOption("orisma.lang", "en")) {

  .check_lang(lang)
  if (!inherits(result, "orisma_result"))
    stop("'result' must be an orisma_result object.", call. = FALSE)

  # Built-in regulatory database
  reg_db <- .build_reg_db(lang)

  ind <- result$indicators %>%
    dplyr::filter(.data$n_records >= min_records)

  # Join with regulatory database
  reg_df <- dplyr::left_join(ind, reg_db, by = "category") %>%
    dplyr::filter(!is.na(.data$directive)) %>%
    dplyr::select(
      .data$label, .data$n_records, .data$WRDI,
      .data$directive, .data$directive_name,
      .data$limit_value, .data$key_requirement
    ) %>%
    dplyr::arrange(dplyr::desc(.data$n_records))

  class(reg_df) <- c("orisma_normativa", "data.frame")
  reg_df
}


#' @noRd
.build_reg_db <- function(lang) {
  is_es <- lang == "es"

  data.frame(
    category = c(
      "chemical_hazardous",
      "chemical_hazardous",
      "carcinogens",
      "noise",
      "vibration",
      "non_ionising_radiation",
      "ionising_radiation",
      "manual_handling",
      "display_screen",
      "biological",
      "bacteria",
      "virus",
      "fungi",
      "asbestos",
      "explosion",
      "fire",
      "electrical_direct",
      "electrical_indirect",
      "thermal_stress",
      "nanomaterials",
      "additive_manufacturing",
      "psychosocial",
      "workload_pace",
      "harassment",
      "work_posture",
      "repetitive_movements"
    ),
    directive = c(
      "Dir. 98/24/CE",
      "Reg. REACH (CE) 1907/2006",
      "Dir. 2004/37/CE",
      "Dir. 2003/10/CE",
      "Dir. 2002/44/CE",
      "Dir. 2006/25/CE",
      "Dir. 2013/59/EURATOM",
      "Dir. 90/269/CEE",
      "Dir. 90/270/CEE",
      "Dir. 2000/54/CE",
      "Dir. 2000/54/CE",
      "Dir. 2000/54/CE",
      "Dir. 2000/54/CE",
      "Dir. 2009/148/CE",
      "Dir. 1999/92/CE (ATEX)",
      "Dir. 89/654/CEE",
      "Dir. 89/391/CEE",
      "Dir. 89/391/CEE",
      "ISO 7933 / ISO 11079",
      "Rec. CE 2022/C 229/01",
      "EN ISO 11228 / NTP 1120",
      "Acuerdo Marco Europeo 2004",
      "Acuerdo Marco Europeo 2004",
      "Dir. 2002/73/CE",
      "ISO 11228-3 / NTP 629",
      "ISO 11228-3 / NTP 629"
    ),
    directive_name = c(
      if(is_es) "Agentes quimicos" else "Chemical agents",
      if(is_es) "Registro sustancias quimicas" else "Chemical substances registration",
      if(is_es) "Agentes cancerigenos y mutagenos" else "Carcinogens and mutagens",
      if(is_es) "Ruido" else "Noise at work",
      if(is_es) "Vibraciones mecanicas" else "Mechanical vibration",
      if(is_es) "Radiacion optica artificial" else "Artificial optical radiation",
      if(is_es) "Radiaciones ionizantes" else "Ionising radiation",
      if(is_es) "Manipulacion manual de cargas" else "Manual handling of loads",
      if(is_es) "Pantallas de visualizacion" else "Display screen equipment",
      if(is_es) "Agentes biologicos" else "Biological agents",
      if(is_es) "Agentes biologicos" else "Biological agents",
      if(is_es) "Agentes biologicos" else "Biological agents",
      if(is_es) "Agentes biologicos" else "Biological agents",
      if(is_es) "Exposicion al amianto" else "Asbestos exposure",
      if(is_es) "Atmosteras explosivas" else "Explosive atmospheres",
      if(is_es) "Lugares de trabajo" else "Workplace requirements",
      if(is_es) "Marco general de seguridad" else "General safety framework",
      if(is_es) "Marco general de seguridad" else "General safety framework",
      if(is_es) "Estres termico (norma tecnica)" else "Thermal stress (technical standard)",
      if(is_es) "Recomendacion nanomateriales" else "Nanomaterials recommendation",
      if(is_es) "Fabricacion aditiva (NTP)" else "Additive manufacturing (NTP)",
      if(is_es) "Estres laboral (acuerdo)" else "Work-related stress (framework)",
      if(is_es) "Estres laboral (acuerdo)" else "Work-related stress (framework)",
      if(is_es) "Igualdad de trato" else "Equal treatment",
      if(is_es) "Movimientos repetitivos" else "Repetitive movements",
      if(is_es) "Movimientos repetitivos" else "Repetitive movements"
    ),
    limit_value = c(
      if(is_es) "VLA-ED / VLA-EC segun sustancia" else "OEL/STEL per substance",
      if(is_es) "Ver ficha de datos de seguridad" else "See safety data sheet",
      if(is_es) "VLA especifico por cancerigeno" else "Specific OEL per carcinogen",
      if(is_es) "LEX,8h <= 80 dB(A); Lpico <= 135 dB(C)" else "LEX,8h <= 80 dB(A); Lpeak <= 135 dB(C)",
      if(is_es) "HAV: A(8) <= 2.5 m/s2; WBV: A(8) <= 0.5 m/s2" else "HAV: A(8) <= 2.5 m/s2; WBV: A(8) <= 0.5 m/s2",
      if(is_es) "Segun longitud de onda y clase laser" else "Per wavelength and laser class",
      if(is_es) "Dosis efectiva: 20 mSv/ano" else "Effective dose: 20 mSv/year",
      if(is_es) "Ecuacion NIOSH / peso maximo recomendado" else "NIOSH equation / recommended weight limit",
      if(is_es) "Evaluacion ergonomica obligatoria" else "Mandatory ergonomic assessment",
      if(is_es) "Clasificacion grupos 1-4" else "Groups 1-4 classification",
      if(is_es) "Clasificacion grupos 1-4" else "Groups 1-4 classification",
      if(is_es) "Clasificacion grupos 1-4" else "Groups 1-4 classification",
      if(is_es) "Clasificacion grupos 1-4" else "Groups 1-4 classification",
      if(is_es) "0.1 fibras/cm3 (8h)" else "0.1 fibres/cm3 (8h)",
      if(is_es) "Zona ATEX clasificada" else "ATEX zone classification",
      if(is_es) "Plan de emergencia obligatorio" else "Mandatory emergency plan",
      if(is_es) "Evaluacion de riesgos obligatoria" else "Mandatory risk assessment",
      if(is_es) "Evaluacion de riesgos obligatoria" else "Mandatory risk assessment",
      if(is_es) "WBGT segun metabolismo" else "WBGT per metabolic rate",
      if(is_es) "Gestion del riesgo por analogia" else "Risk management by analogy",
      if(is_es) "Evaluacion higienica especifica" else "Specific hygiene assessment",
      if(is_es) "Evaluacion psicosocial obligatoria" else "Mandatory psychosocial assessment",
      if(is_es) "Evaluacion psicosocial obligatoria" else "Mandatory psychosocial assessment",
      if(is_es) "Protocolo de acoso obligatorio" else "Mandatory harassment protocol",
      if(is_es) "OCRA / indice de repetitividad" else "OCRA / repetitivity index",
      if(is_es) "OCRA / indice de repetitividad" else "OCRA / repetitivity index"
    ),
    key_requirement = c(
      if(is_es) "Medicion ambiental y biologica; VLA como referencia" else "Environmental and biological monitoring; OEL as reference",
      if(is_es) "Registro, evaluacion y autorizacion de sustancias" else "Registration, evaluation, authorisation of substances",
      if(is_es) "Sustitucion preferente; control de exposicion" else "Substitution preferred; exposure control",
      if(is_es) "Medicion del nivel de ruido; audiometria periodica" else "Noise level measurement; periodic audiometry",
      if(is_es) "Medicion vibraciones; vigilancia de la salud" else "Vibration measurement; health surveillance",
      if(is_es) "Evaluacion exposicion; gafas protectoras laser" else "Exposure assessment; laser protective eyewear",
      if(is_es) "Dosimetria individual; vigilancia medica" else "Individual dosimetry; medical surveillance",
      if(is_es) "Evaluacion tarea; ayudas mecanicas si procede" else "Task assessment; mechanical aids if applicable",
      if(is_es) "Evaluacion puesto; pausas; ajuste ergonomico" else "Workstation assessment; breaks; ergonomic adjustment",
      if(is_es) "Clasificacion agente; medidas de contencion" else "Agent classification; containment measures",
      if(is_es) "Clasificacion agente; medidas de contencion" else "Agent classification; containment measures",
      if(is_es) "Clasificacion agente; medidas de contencion" else "Agent classification; containment measures",
      if(is_es) "Clasificacion agente; medidas de contencion" else "Agent classification; containment measures",
      if(is_es) "Inventario; plan de desamiantado" else "Inventory; asbestos management plan",
      if(is_es) "Clasificacion zona; equipos certificados ATEX" else "Zone classification; ATEX certified equipment",
      if(is_es) "Vias de evacuacion; extincion; simulacros" else "Evacuation routes; extinguishing; drills",
      if(is_es) "Evaluacion y medidas preventivas documentadas" else "Assessment and documented preventive measures",
      if(is_es) "Evaluacion y medidas preventivas documentadas" else "Assessment and documented preventive measures",
      if(is_es) "Medicion WBGT; acceso agua; pausas termicas" else "WBGT measurement; water access; thermal breaks",
      if(is_es) "Principio de precaucion; gestion por analogia" else "Precautionary principle; analogy-based management",
      if(is_es) "Evaluacion higienica; EPIs especificos AM" else "Hygiene assessment; AM-specific PPE",
      if(is_es) "Evaluacion metodo FPSICO; plan de intervencion" else "FPSICO method assessment; intervention plan",
      if(is_es) "Evaluacion metodo FPSICO; plan de intervencion" else "FPSICO method assessment; intervention plan",
      if(is_es) "Protocolo anti-acoso; canal de denuncia" else "Anti-harassment protocol; reporting channel",
      if(is_es) "Evaluacion OCRA; rotacion de tareas" else "OCRA assessment; task rotation",
      if(is_es) "Evaluacion OCRA; rotacion de tareas" else "OCRA assessment; task rotation"
    ),
    stringsAsFactors = FALSE
  )
}


#' Print method for orisma_normativa
#' @param x An `orisma_normativa` object.
#' @param ... Further arguments (ignored).
#' @return Invisibly returns `x`.
#' @export
print.orisma_normativa <- function(x, ...) {
  cat("\n-- ORISMA Regulatory Cross-reference --\n")
  cat(" Risk categories with applicable regulation:", nrow(x), "\n\n")
  for (i in seq_len(nrow(x))) {
    cat(sprintf(" [%d] %s\n", i, x$label[i]))
    cat(sprintf("     Directive: %s - %s\n", x$directive[i], x$directive_name[i]))
    cat(sprintf("     Limit/Reference: %s\n", x$limit_value[i]))
    cat(sprintf("     Key requirement: %s\n\n", x$key_requirement[i]))
  }
  invisible(x)
}

#' orisma: Occupational Risk Integrated Systematic Mapping and Analysis
#'
#' @description
#' `orisma` is an R package for systematic bibliometric mapping of
#' occupational risk evidence. It is designed to help researchers and
#' occupational safety and health practitioners analyse whether the scientific
#' literature on a given topic is connected to workers, workplaces, exposure
#' conditions and preventive decision-making.
#'
#' @details
#' ORISMA provides a complete workflow for occupational risk evidence mapping:
#'
#' \itemize{
#'   \item multi-source bibliographic ingestion;
#'   \item deduplication;
#'   \item relevance filtering through `orm_relevance_guard()` and `orm_run_guarded()`;
#'   \item risk category extraction using a 58-category occupational risk dictionary;
#'   \item preventive bibliometric indicators such as WRDI, RCS and MGP;
#'   \item Abstract Sufficiency Score (ASS);
#'   \item Bridge Article Score;
#'   \item bilingual academic reports and practitioner-oriented risk sheets.
#' }
#'
#' Typical workflow:
#'
#' \preformatted{
#' library(orisma)
#'
#' refs <- orm_load("my_references/")
#'
#' result <- orm_run_guarded(
#'   refs,
#'   topic = "Collaborative robotics and occupational health and safety",
#'   mode = "conservative"
#' )
#'
#' orm_report(result)
#' orm_risk_sheet(result)
#' }
#'
#' @section Original indicators:
#' \itemize{
#'   \item WRDI: Worker-Risk Disconnection Index.
#'   \item RCS: Risk Category Saturation Index.
#'   \item MGP: Material-Gap Profile.
#'   \item ASS: Abstract Sufficiency Score.
#'   \item Bridge Article Score.
#' }
#'
#' @section Citation:
#' Aguilar-Elena, R., & Delgado-Garcia, A. (2026). `orisma`:
#' Occupational Risk Integrated Systematic Mapping and Analysis.
#' R package version 0.1.0. Universidad Internacional de Valencia (VIU)
#' & Universidad de Salamanca (USAL).
#' https://github.com/Aguilar-Elena/orisma
#'
#' @section Author(s):
#' Raul Aguilar-Elena <raguilar@universidadviu.com> \cr
#' Occupational Risk Prevention and Occupational Health Research Group (GPRL),
#' Universidad Internacional de Valencia (VIU), Valencia, Spain. \cr
#'
#' Ana Delgado-Garcia <a.delgado@usal.es> \cr
#' Universidad de Salamanca (USAL), Salamanca, Spain.
#'
#' @seealso
#' Useful links:
#' \itemize{
#'   \item \url{https://github.com/Aguilar-Elena/orisma}
#'   \item Report bugs at \url{https://github.com/Aguilar-Elena/orisma/issues}
#' }
#'
#' @docType package
#' @name orisma-package
#' @aliases orisma
"_PACKAGE"

#' Automatic dimension extraction and risk cross-matrix
#'
#' @description
#' `orm_autodim()` automatically discovers the most relevant contextual
#' dimensions of a corpus using two complementary modes:
#'
#' **Mode 1: Dictionary blocks** (default, `method = "blocks"`)
#' Uses the normative blocks of the ORISMA dictionary (A-Safety, B-Hygiene,
#' C-Ergonomics, D-Psychosociology, E-Biological, F-Emerging) as dimensions.
#' Computes a block x block co-occurrence matrix showing how many studies
#' address combinations of risk blocks simultaneously. Works for any corpus
#' without any configuration.
#'
#' **Mode 2: Free text** (`method = "text"`)
#' Extracts discriminant terms from abstracts using TF-IDF-like filtering.
#' Useful for discovering domain-specific dimensions not covered by the
#' dictionary (e.g. specific materials, sectors, tasks).
#'
#' @param mx An `orisma_matrix` object from [orm_extract()].
#' @param method Character. `"blocks"` (default) or `"text"`.
#' @param text_col Character. Text field for `method = "text"`. Default `"abstract"`.
#' @param n_dims Integer. Max dimensions for `method = "text"`. Default `12`.
#' @param min_freq Integer. Min document frequency for `method = "text"`. Default `3`.
#' @param max_doc_pct Numeric (0-1). Max document proportion for `method = "text"`.
#'   Terms above this are too generic. Default `0.35`.
#' @param min_cooccur Numeric (0-1). Min co-occurrence with a risk. Default `0.5`.
#' @param fuzzy_sim Numeric (0-1). Fuzzy grouping threshold. Default `0.85`.
#' @param stopwords Character vector. Extra stopwords for `method = "text"`.
#' @param lang Character. `"en"` or `"es"`.
#' @param verbose Logical.
#'
#' @return A list (class `orisma_dims`) ready for [orm_dim_matrix()].
#' @seealso [orm_dim_matrix()]
#' @export
orm_autodim <- function(mx,
                        method      = "blocks",
                        text_col    = "abstract",
                        n_dims      = 12L,
                        min_freq    = 3L,
                        max_doc_pct = 0.35,
                        min_cooccur = 0.5,
                        fuzzy_sim   = 0.85,
                        stopwords   = NULL,
                        lang        = getOption("orisma.lang", "en"),
                        verbose     = getOption("orisma.verbose", TRUE)) {

  .check_lang(lang)
  if (!inherits(mx, "orisma_matrix")) {
    stop("'mx' must be an orisma_matrix object from orm_extract().", call. = FALSE)
  }
  if (!method %in% c("blocks", "text")) {
    stop("'method' must be 'blocks' or 'text'.", call. = FALSE)
  }

  if (verbose) cli::cli_h2(
    if (lang == "es") "Extraccion automatica de dimensiones"
    else "Automatic dimension extraction"
  )

  if (method == "blocks") {
    .autodim_blocks(mx, lang, verbose)
  } else {
    .autodim_text(mx, text_col, n_dims, min_freq, max_doc_pct,
                  min_cooccur, fuzzy_sim, stopwords, lang, verbose)
  }
}


# =============================================================================
# MODE 1: DICTIONARY BLOCKS
# =============================================================================

#' @noRd
.autodim_blocks <- function(mx, lang, verbose) {

  refs    <- mx$refs
  bin_mat <- mx$matrix
  dict    <- mx$dict

  # Extract block for each category
  cat_blocks <- vapply(names(dict), function(k) {
    b <- dict[[k]]$block
    if (is.null(b)) "Unknown" else b
  }, character(1))

  unique_blocks <- unique(cat_blocks)
  unique_blocks <- sort(unique_blocks[!is.na(unique_blocks)])

  if (verbose) cli::cli_alert_info(paste0(
    if (lang == "es") "Modo: bloques normativos . " else "Mode: normative blocks . ",
    length(unique_blocks), " bloques detectados"
  ))

  # For each block, which records have at least one category from that block?
  block_presence <- matrix(0L,
    nrow = nrow(bin_mat),
    ncol = length(unique_blocks),
    dimnames = list(rownames(bin_mat), unique_blocks)
  )

  for (b in unique_blocks) {
    cats_in_block <- names(cat_blocks)[cat_blocks == b]
    cats_present  <- intersect(cats_in_block, colnames(bin_mat))
    if (length(cats_present) > 0) {
      block_presence[, b] <- as.integer(rowSums(bin_mat[, cats_present, drop = FALSE]) > 0)
    }
  }

  # Build dim_list: each block is a dimension
  # Terms = category keys belonging to that block
  dim_list <- lapply(unique_blocks, function(b) {
    cats_in_block <- names(cat_blocks)[cat_blocks == b]
    intersect(cats_in_block, colnames(bin_mat))
  })
  names(dim_list) <- unique_blocks

  # Remove empty blocks
  non_empty <- vapply(dim_list, length, integer(1)) > 0
  dim_list  <- dim_list[non_empty]

  # Frequency: how many records have at least one cat from this block
  group_freqs <- vapply(names(dim_list), function(b) {
    sum(block_presence[, b])
  }, integer(1))

  dim_list <- dim_list[order(-group_freqs)]

  if (verbose) {
    cli::cli_alert_success(paste0(
      length(dim_list),
      if (lang == "es") " bloques como dimensiones"
      else " blocks as dimensions"
    ))
    for (nm in names(dim_list)) {
      cli::cli_alert_info(paste0(
        "  [", nm, "]  n=", group_freqs[nm],
        "  cats=", length(dim_list[[nm]])
      ))
    }
  }

  tf_df <- data.frame(
    term       = names(dim_list),
    group_freq = as.integer(group_freqs[names(dim_list)]),
    n_cats     = vapply(dim_list, length, integer(1)),
    stringsAsFactors = FALSE
  )

  result <- list(
    dims           = dim_list,
    term_freq      = tf_df,
    n_dims         = length(dim_list),
    text_col       = NA_character_,
    method         = "blocks",
    block_presence = block_presence,
    params         = list(method = "blocks")
  )
  class(result) <- c("orisma_dims", "list")
  attr(result, "orisma_lang") <- lang
  result
}


# =============================================================================
# MODE 2: FREE TEXT
# =============================================================================

#' @noRd
.autodim_text <- function(mx, text_col, n_dims, min_freq, max_doc_pct,
                           min_cooccur, fuzzy_sim, stopwords, lang, verbose) {

  refs         <- mx$refs
  bin_mat      <- mx$matrix
  n_total_docs <- nrow(refs)

  # Select text column
  if (!text_col %in% names(refs) ||
      mean(is.na(refs[[text_col]]) | refs[[text_col]] == "") > 0.5) {
    text_col <- "title"
    if (verbose) cli::cli_alert_warning(
      if (lang == "es") "Abstract no disponible, usando titulo"
      else "Abstract not available, falling back to title"
    )
  }

  has_risk  <- rowSums(bin_mat) > 0
  refs_risk <- refs[has_risk, ]
  text_risk <- tolower(as.character(refs_risk[[text_col]]))
  text_risk[is.na(text_risk)] <- ""

  if (verbose) cli::cli_alert_info(paste0(
    if (lang == "es") "Modo: texto libre . analizando "
    else "Mode: free text . analysing ",
    sum(has_risk), " records with risk detected"
  ))

  # Stopwords
  sw_base <- c(
    "the","a","an","and","or","but","in","on","at","to","for","of","with",
    "by","from","as","is","was","are","were","be","been","being","have",
    "has","had","do","does","did","will","would","could","should","may",
    "might","shall","can","not","no","nor","so","yet","both","either",
    "neither","this","that","these","those","it","its","we","our","they",
    "their","he","she","his","her","us","them","which","who","what","when",
    "where","why","how","all","each","every","any","some","such","than",
    "more","most","also","into","over","after","between","through","during",
    "before","under","about","against","while","within","without","among",
    "across","along","above","below","up","down","out","off","then","there",
    "here","only","just","even","well","back","still","way","however","thus",
    "study","studies","results","data","analysis","method","methods",
    "conclusion","background","objective","aim","paper","article","research",
    "review","literature","based","using","used","use","shown","found","show",
    "shows","including","included","include","significantly","significant",
    "associated","compared","comparison","measured","measurement","assessed",
    "assessment","evaluated","evaluation","investigated","performed",
    "conducted","reported","report","presented","present","provide",
    "provided","provides","suggest","suggests","indicate","indicates",
    "demonstrate","demonstrates","high","low","higher","lower","large",
    "small","different","similar","various","several","many","two","three",
    "four","five","first","second","respectively","therefore","furthermore",
    "moreover","although","despite","due","given","known","well","new","one",
    "since","whether","other","further","exposure","risk","risks","hazard",
    "hazards","health","safety","occupational","workers","worker","operator",
    "operators","concentration","level","levels","particle","particles",
    "emission","emissions","sample","samples","control","controls","effect",
    "effects","potential","current","process","processes","surface",
    "material","materials","type","types","size","number","time","work",
    "working","workplace","environment","environmental","author","authors",
    "technique","techniques","application","applications","printed","printer",
    "printers","release","released","increase","increased","increases",
    "technology","technologies","industrial","printing","powder","powders",
    "laser","metals","chemical","chemicals","showed","showed","range",
    "related","system","systems","conditions","condition","properties",
    "impact","impacts","human","energy","fusion","melting","metallic",
    "filament","filaments","challenges","challenge","concentrations",
    "additive","additives","manufacturing","metal","process","processes"
  )
  sw_all <- unique(c(sw_base, stopwords))

  # Tokenise
  tokens_list <- lapply(seq_along(text_risk), function(i) {
    txt   <- text_risk[[i]]
    words <- unlist(regmatches(txt, gregexpr("[a-z]{4,}", txt)))
    words <- words[!words %in% sw_all]
    unique(words)
  })

  all_terms <- unlist(tokens_list)
  term_freq <- sort(table(all_terms), decreasing = TRUE)
  term_freq <- term_freq[term_freq >= min_freq]

  if (length(term_freq) == 0) return(.empty_dims())

  # Full corpus for doc-frequency
  all_text_full <- tolower(as.character(refs[[text_col]]))
  all_text_full[is.na(all_text_full)] <- ""

  candidate_terms <- names(term_freq)

  # Filter: max_doc_pct
  term_doc_pct <- vapply(candidate_terms, function(term) {
    n_docs <- sum(grepl(paste0("\\b", term, "\\b"), all_text_full, perl = TRUE))
    n_docs / n_total_docs
  }, numeric(1))
  candidate_terms <- candidate_terms[term_doc_pct <= max_doc_pct]
  term_freq       <- term_freq[candidate_terms]

  if (length(candidate_terms) == 0) return(.empty_dims())

  # Co-occurrence filter
  cooccur_scores <- vapply(candidate_terms, function(term) {
    in_risk   <- vapply(tokens_list, function(toks) term %in% toks, logical(1))
    n_in_risk <- sum(in_risk)
    if (n_in_risk == 0) return(0)
    n_total <- sum(grepl(paste0("\\b", term, "\\b"), all_text_full, perl = TRUE))
    if (n_total == 0) return(0)
    n_in_risk / n_total
  }, numeric(1))

  valid_terms <- candidate_terms[cooccur_scores >= min_cooccur]
  if (length(valid_terms) == 0) return(.empty_dims())

  # Fuzzy grouping
  n_terms <- length(valid_terms)
  groups  <- seq_len(n_terms)

  if (n_terms > 1) {
    dist_mat  <- stringdist::stringdistmatrix(valid_terms, valid_terms, method = "lv")
    max_chars <- outer(nchar(valid_terms), nchar(valid_terms), pmax)
    sim_mat   <- 1 - dist_mat / pmax(max_chars, 1)
    for (i in seq_len(n_terms - 1)) {
      for (j in seq(i + 1, n_terms)) {
        if (sim_mat[i, j] >= fuzzy_sim && groups[j] == j) groups[j] <- groups[i]
      }
    }
  }

  unique_groups <- unique(groups)
  dim_list <- lapply(unique_groups, function(g) {
    members <- valid_terms[groups == g]
    freqs   <- as.integer(term_freq[members])
    members[order(-freqs)]
  })
  names(dim_list) <- vapply(dim_list, `[[`, character(1), 1)

  group_freqs <- vapply(dim_list, function(g) {
    sum(as.integer(term_freq[g[g %in% names(term_freq)]]), na.rm = TRUE)
  }, numeric(1))
  dim_list <- dim_list[order(-group_freqs)]
  if (length(dim_list) > n_dims) dim_list <- dim_list[seq_len(n_dims)]

  if (verbose) {
    cli::cli_alert_success(paste0(length(dim_list), " dimensions detected"))
    for (nm in names(dim_list)) {
      syns <- dim_list[[nm]]
      cli::cli_alert_info(paste0(
        "  [", nm, "]",
        if (length(syns) > 1) paste0(" + ", paste(syns[-1], collapse = ", ")) else ""
      ))
    }
  }

  tf_df <- data.frame(
    term        = names(dim_list),
    group_freq  = as.integer(group_freqs[seq_along(dim_list)]),
    cooccur_pct = round(cooccur_scores[names(dim_list)] * 100, 1),
    doc_pct     = round(term_doc_pct[names(dim_list)] * 100, 1),
    n_synonyms  = vapply(dim_list, length, integer(1)),
    synonyms    = vapply(dim_list,
                         function(x) paste(x[-1], collapse = "; "),
                         character(1)),
    stringsAsFactors = FALSE
  )

  result <- list(
    dims      = dim_list,
    term_freq = tf_df,
    n_dims    = length(dim_list),
    text_col  = text_col,
    method    = "text",
    params    = list(min_freq = min_freq, min_cooccur = min_cooccur,
                     max_doc_pct = max_doc_pct, fuzzy_sim = fuzzy_sim)
  )
  class(result) <- c("orisma_dims", "list")
  attr(result, "orisma_lang") <- lang
  result
}


# =============================================================================
# orm_dim_matrix
# =============================================================================

#' Build a risk category x dimension cross-matrix
#'
#' @description
#' Builds a risk category x dimension cross-matrix and saves a hierarchical
#' clustered heatmap with dendrograms and numeric values in each cell.
#'
#' When `dims` was built with `method = "blocks"`, the matrix shows
#' risk categories x normative blocks (A-Safety, B-Hygiene, etc.).
#' When `dims` was built with `method = "text"`, the matrix shows
#' risk categories x discovered text dimensions.
#'
#' @param result An `orisma_result` object from [orm_analyse()] or [orm_run()].
#' @param dims An `orisma_dims` object from [orm_autodim()].
#' @param min_records Integer. Min records for a risk category row. Default `2`.
#' @param out_dir Character or NULL. Directory to save the heatmap PNG.
#' @param filename Character. Output filename. Default `"risk_dimension_heatmap.png"`.
#' @param lang Character. `"en"` or `"es"`.
#' @param verbose Logical.
#'
#' @return Invisibly returns the cross-matrix (risk categories x dimensions).
#' @export
orm_dim_matrix <- function(result,
                            dims,
                            min_records = 2L,
                            out_dir     = NULL,
                            filename    = "risk_dimension_heatmap.png",
                            lang        = getOption("orisma.lang", "en"),
                            verbose     = getOption("orisma.verbose", TRUE)) {

  if (!inherits(result, "orisma_result"))
    stop("'result' must be an orisma_result object.", call. = FALSE)
  if (!inherits(dims, "orisma_dims"))
    stop("'dims' must be an orisma_dims object from orm_autodim().", call. = FALSE)

  refs     <- result$refs
  bin_mat  <- result$matrix
  dim_list <- dims$dims
  method   <- dims$method

  # Active risk categories (rows)
  active_cats <- result$indicators %>%
    dplyr::filter(.data$n_records >= min_records) %>%
    dplyr::arrange(dplyr::desc(.data$n_records)) %>%
    dplyr::pull(.data$category)

  active_labels <- result$indicators %>%
    dplyr::filter(.data$category %in% active_cats) %>%
    dplyr::arrange(dplyr::desc(.data$n_records)) %>%
    dplyr::pull(.data$label)

  bin_active <- bin_mat[, active_cats, drop = FALSE]
  dim_names  <- names(dim_list)

  cross_mat <- matrix(0L,
    nrow = length(active_cats),
    ncol = length(dim_names),
    dimnames = list(
      stringr::str_wrap(active_labels, 28),
      dim_names
    )
  )

  if (method == "blocks" && !is.null(dims$block_presence)) {
    # Mode 1: use pre-computed block presence matrix
    bp <- dims$block_presence
    for (j in seq_along(dim_names)) {
      blk     <- dim_names[[j]]
      has_dim <- if (blk %in% colnames(bp)) bp[, blk] == 1L
                 else rep(FALSE, nrow(bp))
      for (i in seq_along(active_cats)) {
        has_risk        <- bin_active[, i] == 1L
        cross_mat[i, j] <- sum(has_risk & has_dim)
      }
    }
  } else {
    # Mode 2: search terms in text
    text_col  <- dims$text_col
    text_all  <- tolower(as.character(refs[[text_col]]))
    text_all[is.na(text_all)] <- ""

    for (j in seq_along(dim_names)) {
      dim_terms <- dim_list[[j]]
      pattern   <- paste0("\\b(", paste(dim_terms, collapse = "|"), ")\\b")
      has_dim   <- grepl(pattern, text_all, ignore.case = TRUE, perl = TRUE)
      for (i in seq_along(active_cats)) {
        has_risk        <- bin_active[, i] == 1L
        cross_mat[i, j] <- sum(has_risk & has_dim)
      }
    }
  }

  # Remove zero rows/cols
  row_active <- rowSums(cross_mat) > 0
  col_active <- colSums(cross_mat) > 0
  mat_plot   <- cross_mat[row_active, col_active, drop = FALSE]

  if (verbose) {
    cli::cli_alert_success(paste0(
      if (lang == "es") "Matriz: " else "Matrix: ",
      nrow(mat_plot), " x ", ncol(mat_plot)
    ))
  }

  # Heatmap
  if (nrow(mat_plot) >= 2 && ncol(mat_plot) >= 2 && !is.null(out_dir)) {
    if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)
    out_path <- file.path(out_dir, filename)

    title_txt <- if (lang == "es") "Focos de Riesgo PRL por Bloque Normativo"
                 else "OHS Risk Focus by Normative Block"

    col_palette <- grDevices::colorRampPalette(
      c("white", "#FFF3E0", "#FFB74D", "#F44336")
    )(50)

    grDevices::png(out_path,
      width  = max(2800, ncol(mat_plot) * 400),
      height = max(2400, nrow(mat_plot) * 220),
      res    = 300)

    pheatmap::pheatmap(
      mat_plot,
      cluster_rows    = nrow(mat_plot) > 2,
      cluster_cols    = ncol(mat_plot) > 2,
      color           = col_palette,
      main            = title_txt,
      fontsize        = 11,
      fontsize_row    = 9,
      fontsize_col    = 10,
      border_color    = "white",
      angle_col       = 45,
      display_numbers = TRUE,
      number_format   = "%d",
      number_color    = "grey20"
    )
    grDevices::dev.off()

    if (verbose) cli::cli_alert_success(paste0(
      if (lang == "es") "Heatmap guardado en: " else "Heatmap saved to: ",
      out_path
    ))
  }

  invisible(cross_mat)
}


# =============================================================================
# PRINT METHODS
# =============================================================================

#' Print method for orisma_dims
#' @param x An `orisma_dims` object.
#' @param ... Further arguments (ignored).
#' @return Invisibly returns `x`.
#' @export
print.orisma_dims <- function(x, ...) {
  cat("\n-- ORISMA Dimensions --\n")
  cat(" Method:          ", x$method, "\n")
  cat(" Dimensions found:", x$n_dims, "\n\n")

  if (x$method == "blocks") {
    cat(" Normative blocks (dimensions):\n")
    for (i in seq_along(x$dims)) {
      nm <- names(x$dims)[i]
      tf <- x$term_freq[x$term_freq$term == nm, ]
      cat(sprintf("  %2d. %-40s  n_records=%-4d  n_cats=%d\n",
                  i, nm,
                  if (nrow(tf) > 0) tf$group_freq[1] else 0,
                  if (nrow(tf) > 0) tf$n_cats[1] else 0))
    }
  } else {
    cat(" Text dimensions (by frequency):\n")
    for (i in seq_along(x$dims)) {
      nm   <- names(x$dims)[i]
      tf   <- x$term_freq[x$term_freq$term == nm, ]
      syns <- x$dims[[nm]]
      cat(sprintf("  %2d. %-22s  freq=%-4d  doc_pct=%s%%",
                  i, nm,
                  if (nrow(tf) > 0) tf$group_freq[1] else 0,
                  if (nrow(tf) > 0) tf$doc_pct[1] else "?"))
      if (length(syns) > 1) cat("  [+", paste(syns[-1], collapse = ", "), "]")
      cat("\n")
    }
  }
  cat("\nUse orm_dim_matrix(result, dims) to build the risk x dimension heatmap.\n")
  invisible(x)
}


#' @noRd
.empty_dims <- function() {
  result <- list(dims = list(), term_freq = data.frame(), n_dims = 0L,
                 text_col = NA_character_, method = "text",
                 params = list())
  class(result) <- c("orisma_dims", "list")
  result
}

# NOTE: The orm_dim_matrix function above is replaced by this improved version
# which strips block prefixes from column labels and uses log scale for color

#' @noRd
.plot_dim_heatmap <- function(mat_plot, out_path, lang) {
  title_txt <- if (lang == "es") "Focos de Riesgo PRL por Bloque Normativo"
               else "OHS Risk Focus by Normative Block"

  # Clean column names: remove "X - " prefix
  colnames(mat_plot) <- gsub("^[A-Z] - ", "", colnames(mat_plot))

  # Log scale for color (avoids one dominant row flattening the rest)
  mat_log <- log1p(mat_plot)

  col_palette <- grDevices::colorRampPalette(
    c("white", "#FFF3E0", "#FFB74D", "#F44336")
  )(50)

  grDevices::png(out_path,
    width  = max(2800, ncol(mat_plot) * 450),
    height = max(2400, nrow(mat_plot) * 230),
    res    = 300)

  pheatmap::pheatmap(
    mat_log,
    cluster_rows    = nrow(mat_log) > 2,
    cluster_cols    = ncol(mat_log) > 2,
    color           = col_palette,
    main            = title_txt,
    fontsize        = 11,
    fontsize_row    = 9,
    fontsize_col    = 10,
    border_color    = NA,
    angle_col       = 45,
    display_numbers = mat_plot,   # show original counts, color by log
    number_format   = "%d",
    number_color    = "grey20",
    legend          = FALSE
  )
  grDevices::dev.off()
}

#' Relevance guard for occupational risk evidence mapping
#'
#' @description
#' Adds a relevance-control layer before ORISMA analysis. The function identifies
#' whether each record is relevant to the target topic, whether it contains an
#' occupational context, whether it is likely to be biomedical or clinical noise,
#' and whether it should be excluded from the main occupational analysis.
#'
#' @param data A data frame of bibliographic records.
#' @param topic Optional topic label used to derive a topic-specific regular expression.
#' @param topic_regex Optional regular expression defining the target technology/topic.
#' @param occupational_regex Optional regular expression defining occupational relevance.
#' @param noise_regex Optional regular expression defining likely off-topic biomedical/clinical noise.
#' @param title_col Optional title column name. If `NULL`, it is detected automatically.
#' @param abstract_col Optional abstract column name. If `NULL`, it is detected automatically.
#' @param keywords_col Optional keywords column name. If `NULL`, it is detected automatically.
#' @param mode Relevance filtering mode. `"flag"` excludes only records outside
#' the target topic and marks uncertain records for review. `"conservative"`
#' excludes off-topic and likely non-occupational biomedical/clinical records.
#' `"strict"` also excludes records with weak occupational context.
#'
#' @return The input data frame with additional relevance-control columns.
#' @export
orm_relevance_guard <- function(data,
                                topic = NULL,
                                topic_regex = NULL,
                                occupational_regex = NULL,
                                noise_regex = NULL,
                                title_col = NULL,
                                abstract_col = NULL,
                                keywords_col = NULL,
                                mode = c("conservative", "flag", "strict")) {

  mode <- match.arg(mode)

  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }

  nms <- names(data)
  nms_low <- tolower(nms)

  detect_col <- function(candidates) {
    idx <- match(tolower(candidates), nms_low)
    idx <- idx[!is.na(idx)]
    if (length(idx) == 0) return(NULL)
    nms[idx[1]]
  }

  if (is.null(title_col)) {
    title_col <- detect_col(c("title", "ti", "article_title", "document_title"))
  }

  if (is.null(abstract_col)) {
    abstract_col <- detect_col(c("abstract", "ab", "summary", "description"))
  }

  if (is.null(keywords_col)) {
    keywords_col <- detect_col(c("keywords", "author_keywords", "de", "id", "kw", "keyword"))
  }

  safe_text <- function(col) {
    if (is.null(col) || !col %in% names(data)) {
      return(rep("", nrow(data)))
    }
    x <- data[[col]]
    x[is.na(x)] <- ""
    as.character(x)
  }

  txt <- paste(
    safe_text(title_col),
    safe_text(abstract_col),
    safe_text(keywords_col),
    sep = " "
  )

  txt_low <- tolower(txt)

  if (is.null(topic_regex)) {
    topic_text <- tolower(ifelse(is.null(topic), "", topic))

    if (grepl("robot|cobot|automation|human.robot|collaborative", topic_text)) {
      topic_regex <- paste(
        c(
          "cobot[s]?",
          "collaborative robot[s]?",
          "collaborative robotics",
          "human[- ]?robot",
          "human robot collaboration",
          "human.robot collaboration",
          "human.robot interaction",
          "\\bhrc\\b",
          "industrial robot[s]?",
          "robotic automation",
          "advanced robotics",
          "advanced robotic[s]?",
          "robotic manipulation",
          "robotic inspection",
          "robotic intervention[s]?",
          "robotic system[s]?",
          "robotic platform[s]?",
          "robotic technolog[y|ies]",
          "robotic[s]?",
          "autonomous mobile robot[s]?",
          "\\bamr\\b",
          "human[- ]?machine collaboration",
          "human.machine collaboration",
          "human[- ]?machine interaction",
          "human.machine interaction",
          "industry 5\\.0",
          "tele[- ]?operation",
          "teleoperation",
          "teleoperated",
          "hazard detection",
          "hazardous environment[s]?",
          "inspection robot[s]?",
          "automation"
        ),
        collapse = "|"
      )
    } else if (grepl("additive|3d|powder bed|metal", topic_text)) {
      topic_regex <- paste(
        c(
          "additive manufacturing",
          "3d printing",
          "three-dimensional printing",
          "metal printing",
          "laser powder bed fusion",
          "powder bed fusion",
          "\\bl-pbf\\b",
          "\\blpbf\\b",
          "selective laser melting",
          "\\bslm\\b",
          "directed energy deposition",
          "\\bded\\b"
        ),
        collapse = "|"
      )
    } else {
      words <- unlist(strsplit(topic_text, "[^a-z0-9]+"))
      words <- words[nchar(words) >= 4]
      if (length(words) == 0) {
        topic_regex <- "."
      } else {
        topic_regex <- paste(unique(words), collapse = "|")
      }
    }
  }

  if (is.null(occupational_regex)) {
    occupational_regex <- paste(
      c(
        "occupational",
        "workplace",
        "work place",
        "worker[s]?",
        "employee[s]?",
        "operator[s]?",
        "personnel",
        "job",
        "task",
        "industrial",
        "factory",
        "manufacturing",
        "assembly",
        "production line",
        "safety",
        "health and safety",
        "risk assessment",
        "hazard",
        "exposure",
        "ergonomic",
        "ergonomics",
        "musculoskeletal",
        "psychosocial",
        "mental workload",
        "workload",
        "fatigue",
        "human factors",
        "personal sampling",
        "breathing zone",
        "field study",
        "on-site",
        "onsite",
        "work environment",
        "hazard detection",
        "hazardous environment",
        "inspection",
        "maintenance",
        "decommissioning",
        "pipeline",
        "nuclear",
        "safety and efficiency",
        "product safety",
        "compliance engineering",
        "injury prevention",
        "risk reduction",
        "human-machine collaboration",
        "industry 5.0"
      ),
      collapse = "|"
    )
  }

  if (is.null(noise_regex)) {
    noise_regex <- paste(
      c(
        "patient[s]?",
        "clinical",
        "surgery",
        "surgical",
        "hospital infection",
        "infection control",
        "disease treatment",
        "antimicrobial resistance",
        "antibiotic",
        "vaccine[s]?",
        "renal transplant",
        "transplant",
        "aquaculture",
        "aquamedicine",
        "manure",
        "livestock",
        "animal vaccine",
        "genomic surveillance",
        "pathogenic bacteria",
        "public health surveillance"
      ),
      collapse = "|"
    )
  }

  topic_relevant <- grepl(topic_regex, txt_low, perl = TRUE)
  occupational_relevant <- grepl(occupational_regex, txt_low, perl = TRUE)
  biomedical_noise <- grepl(noise_regex, txt_low, perl = TRUE)

  # Strong occupational override: if title/abstract clearly contains workplace terms,
  # do not exclude only because of biomedical vocabulary.
  strong_occupational_regex <- paste(
    c(
      "occupational exposure",
      "worker exposure",
      "workplace exposure",
      "personal sampling",
      "breathing zone",
      "operators?",
      "workers?",
      "industrial workplace",
      "manufacturing worker",
      "assembly worker",
      "occupational safety",
      "occupational health"
    ),
    collapse = "|"
  )

  strong_occupational <- grepl(strong_occupational_regex, txt_low, perl = TRUE)

  # Conservative exclusion logic:
  # - Records outside the target topic are excluded.
  # - Topic-related records with no occupational signal are excluded.
  # - Biomedical/clinical records are excluded only when they lack occupational relevance.
  # - Records with both topic and occupational relevance are retained, even if they contain
  #   biomedical terms, because robotics, rehabilitation, healthcare and wearable-sensor
  #   studies may still be relevant for occupational ergonomics or prevention.
  weak_occupational_context <- topic_relevant & !occupational_relevant
  biomedical_review <- biomedical_noise & occupational_relevant & topic_relevant
  weak_context_review <- weak_occupational_context & !biomedical_noise

  if (mode == "flag") {

    # Exploratory mode:
    # Exclude only records outside the target topic. Keep uncertain records and
    # label them for review.
    exclusion_flag <- !topic_relevant

  } else if (mode == "conservative") {

    # Balanced default:
    # Exclude records outside the target topic and biomedical/clinical records
    # without occupational or strong occupational signal. Keep biomedical overlap
    # when there is an occupational signal, but flag it for review.
    exclusion_flag <- (!topic_relevant) |
      (biomedical_noise & !occupational_relevant & !strong_occupational)

  } else if (mode == "strict") {

    # Technical screening mode:
    # Exclude records outside the target topic, records with weak occupational
    # context, and biomedical/clinical records without strong occupational signal.
    exclusion_flag <- (!topic_relevant) |
      weak_occupational_context |
      (biomedical_noise & !strong_occupational)

  }

  review_flag <- (biomedical_review | weak_context_review) & !exclusion_flag

  exclusion_reason <- rep("included", length(txt_low))
  exclusion_reason[!topic_relevant] <- "not related to target topic"
  exclusion_reason[weak_context_review & !exclusion_flag] <- "included but flagged for weak occupational context"
  exclusion_reason[biomedical_noise & !occupational_relevant & !strong_occupational & exclusion_flag] <- "likely biomedical/clinical/non-occupational noise"
  exclusion_reason[biomedical_review & !exclusion_flag] <- "included but flagged for biomedical/clinical review"
  exclusion_reason[weak_occupational_context & exclusion_flag] <- "topic-related but no clear occupational context"
  exclusion_reason[topic_relevant & occupational_relevant & !biomedical_noise] <- "included"

  data$relevance_guard_mode <- mode
  data$topic_relevant <- topic_relevant
  data$occupational_relevant <- occupational_relevant
  data$biomedical_noise <- biomedical_noise
  data$strong_occupational_signal <- strong_occupational
  data$review_flag <- review_flag
  data$exclusion_flag <- exclusion_flag
  data$exclusion_reason <- exclusion_reason

  data
}

#' Generate all ORISMA outputs and reports
#'
#' @description
#' `orm_report()` takes a completed `orisma_result` object and generates
#' the full set of outputs including improved visualisations and a rich
#' bilingual HTML executive report.
#'
#' @param result An `orisma_result` object from [orm_analyse()] or [orm_run()].
#' @param lang Character. `"en"` or `"es"`. Report language.
#' @param out_dir Character. Output directory. Created if it does not exist.
#' @param formats Character vector. Which outputs to generate.
#'   Options: `"html"`, `"csv"`, `"plots"`, `"certificate"`. Default: all.
#' @param min_records Integer. Minimum records for a category to appear in
#'   plots. Default `1`.
#' @param top_n Integer. Number of top categories to show in temporal plot.
#'   Default `8`.
#' @param topic Character. Domain or technology being analysed. Used in plot
#'   subtitles and report headers. If NULL, neutral generic text is used.
#' @param verbose Logical. Print progress?
#'
#' @return Invisibly returns the output directory path.
#' @export
orm_report <- function(result,
                       topic       = NULL,
                       lang        = getOption("orisma.lang", "en"),
                       out_dir     = getOption("orisma.out_dir", "orisma_output"),
                       formats     = c("html", "csv", "plots", "certificate"),
                       min_records = 1L,
                       top_n       = 8L,
                       verbose     = getOption("orisma.verbose", TRUE)) {

  .check_lang(lang)
  if (!inherits(result, "orisma_result")) {
    stop("'result' must be an orisma_result object from orm_analyse() or orm_run().",
         call. = FALSE)
  }

  # Use topic from result if not explicitly provided
  if (is.null(topic) && !is.null(result$topic)) topic <- result$topic

  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)

  if (verbose) {
    cli::cli_h1(orm_msg("phase_report", lang))
    cli::cli_alert_info(orm_msg("report_start", lang, out_dir = out_dir))
  }

  # ── 1. CSV files ─────────────────────────────────────────────────────────────
  if ("csv" %in% formats) {
    readr::write_csv(result$refs,       file.path(out_dir, "orisma_corpus.csv"))
    readr::write_csv(result$indicators, file.path(out_dir, "orisma_indicators.csv"))

    mat_df <- as.data.frame(result$matrix)
    mat_df <- cbind(record_id = rownames(result$matrix), mat_df)
    readr::write_csv(mat_df, file.path(out_dir, "orisma_matrix.csv"))

    if (!is.null(result$MGP)) {
      readr::write_csv(result$MGP, file.path(out_dir, "orisma_mgp.csv"))
    }

    prisma_log <- .build_prisma_log(result, lang)
    readr::write_csv(prisma_log, file.path(out_dir, "prisma_log.csv"))

    if (verbose) cli::cli_alert_success(paste0("CSV files saved to: ", out_dir))
  }

  # ── 2. Plots ──────────────────────────────────────────────────────────────────
  if ("plots" %in% formats) {
    plots_dir <- file.path(out_dir, "plots")
    if (!dir.exists(plots_dir)) dir.create(plots_dir)

    tryCatch({
      .plot_wrdi_v2(result, plots_dir, lang, min_records)
      .plot_rcs_v2(result, plots_dir, lang, min_records)
      .plot_gap_map_v2(result, plots_dir, lang, min_records)
      .plot_temporal_v2(result, plots_dir, lang, top_n, topic)
      .plot_cooccur_v2(result, plots_dir, lang, min_records)

      # ASS distribution plot
      if (!is.null(result$mx) && "ass_score" %in% names(result$mx$refs)) {
        tryCatch({
          orm_ass_plot(result$mx, out_dir = plots_dir, lang = lang)
        }, error = function(e) {
          cli::cli_alert_warning(paste0("ASS plot: ", e$message))
        })
      }

      # Risk x dimension heatmap
      if (!is.null(result$dims) && result$dims$n_dims > 0) {
        tryCatch({
          orm_dim_matrix(result, result$dims,
                         min_records = min_records,
                         out_dir     = plots_dir,
                         lang        = lang,
                         verbose     = FALSE)
        }, error = function(e) {
          cli::cli_alert_warning(paste0("Dim heatmap: ", e$message))
        })
      }

      .plot_distribution_v2(result, plots_dir, lang, min_records)
      if (verbose) cli::cli_alert_success(paste0("Plots saved to: ", plots_dir))
    }, error = function(e) {
      cli::cli_alert_warning(paste0("Some plots could not be generated: ", e$message))
    })
  }

  # ── 3. Certificate ────────────────────────────────────────────────────────────
  if ("certificate" %in% formats) {
    cert      <- .build_certificate(result)
    cert_path <- file.path(out_dir, "analysis.orisma")
    jsonlite::write_json(cert, cert_path, pretty = TRUE, auto_unbox = TRUE)
    if (verbose) {
      cli::cli_alert_success(orm_msg("report_cert", lang, file = basename(cert_path)))
    }
  }

  # ── 4. HTML report ────────────────────────────────────────────────────────────
  if ("html" %in% formats) {
    html_path <- file.path(out_dir, "orisma_report.html")
    tryCatch({
      .write_html_report_v2(result, html_path, lang, out_dir, min_records, topic)
      if (verbose) {
        cli::cli_alert_success(orm_msg("report_html", lang, file = basename(html_path)))
      }
    }, error = function(e) {
      cli::cli_alert_warning(paste0("HTML report error: ", e$message))
    })
  }

  if (verbose) {
    cli::cli_rule()
    cli::cli_alert_success(orm_msg("report_done", lang, out_dir = out_dir))
  }

  invisible(out_dir)
}


# =============================================================================
# IMPROVED PLOT FUNCTIONS
# =============================================================================

#' @noRd
.plot_wrdi_v2 <- function(result, dir, lang, min_records = 1) {
  df <- result$indicators %>%
    dplyr::filter(.data$n_records >= min_records, !is.na(.data$WRDI)) %>%
    dplyr::arrange(.data$WRDI) %>%
    dplyr::mutate(
      label_w = stringr::str_wrap(.data$label, 35),
      fill_col = dplyr::case_when(
        .data$WRDI >= 0.8 ~ "high",
        .data$WRDI >= 0.5 ~ "mid",
        TRUE              ~ "low"
      )
    )

  title_txt    <- if (lang == "es") "Indice de Desconexion Tecnico-Laboral (WRDI)"
                  else "Worker-Risk Disconnection Index (WRDI)"
  subtitle_txt <- if (lang == "es")
    paste0("WRDI global: ", result$WRDI_global,
           " * linea punteada = valor global\n0 = todos los estudios incluyen trabajadores * 1 = ningun estudio incluye trabajadores")
  else
    paste0("Global WRDI: ", result$WRDI_global,
           " * dashed line = global value\n0 = all studies include worker data * 1 = no study includes worker data")

  p <- ggplot2::ggplot(df, ggplot2::aes(
    x    = .data$WRDI,
    y    = reorder(.data$label_w, .data$WRDI),
    fill = .data$fill_col
  )) +
    ggplot2::geom_col(show.legend = FALSE) +
    ggplot2::geom_text(ggplot2::aes(label = round(.data$WRDI, 2)),
                       hjust = -0.15, size = 3, colour = "grey30") +
    ggplot2::geom_vline(xintercept = result$WRDI_global,
                        linetype = "dashed", colour = "grey40", linewidth = 0.8) +
    ggplot2::scale_fill_manual(
      values = c("high" = "#0F6E56", "mid" = "#4DAF8D", "low" = "#9FE1CB")
    ) +
    ggplot2::scale_x_continuous(limits = c(0, 1.15), breaks = seq(0, 1, 0.25)) +
    ggplot2::labs(title = title_txt, subtitle = subtitle_txt,
                  x = "WRDI", y = NULL) +
    ggplot2::theme_minimal(base_size = 11) +
    ggplot2::theme(
      panel.grid.major.y = ggplot2::element_blank(),
      plot.subtitle      = ggplot2::element_text(size = 9, colour = "grey40")
    )

  ggplot2::ggsave(file.path(dir, "wrdi_chart.png"), p,
                  width = 12, height = max(6, nrow(df) * 0.45), dpi = 300,
                  limitsize = FALSE)
}


#' @noRd
.plot_rcs_v2 <- function(result, dir, lang, min_records = 1) {
  df <- result$indicators %>%
    dplyr::filter(.data$n_records >= min_records) %>%
    dplyr::mutate(
      label_w   = stringr::str_wrap(.data$label, 25),
      saturated = .data$RCS > 1
    )

  title_txt    <- if (lang == "es") "Indice de Saturacion por Categoria (RCS)"
                  else "Risk Category Saturation Index (RCS)"
  subtitle_txt <- if (lang == "es") "RCS > 1 = sobrerepresentado * < 1 = infrarepresentado"
                  else "RCS > 1 = over-represented * RCS < 1 = under-represented"

  p <- ggplot2::ggplot(df, ggplot2::aes(
    x      = .data$RCS,
    y      = .data$pct_records,
    size   = .data$n_records,
    colour = .data$saturated
  )) +
    ggplot2::geom_point(alpha = 0.85) +
    ggrepel::geom_text_repel(ggplot2::aes(label = .data$label_w),
                              size = 3, max.overlaps = 20,
                              box.padding = 0.4) +
    ggplot2::geom_vline(xintercept = 1, linetype = "dashed",
                        colour = "grey50", linewidth = 0.7) +
    ggplot2::scale_colour_manual(
      values = c("TRUE" = "#0F6E56", "FALSE" = "#D85A30"),
      labels = if (lang == "es") c("TRUE" = "Sobrerepresentado",
                                    "FALSE" = "Infrarepresentado")
               else c("TRUE" = "Over-represented",
                      "FALSE" = "Under-represented"),
      name = NULL
    ) +
    ggplot2::scale_size_continuous(
      name   = if (lang == "es") "N estudios" else "N studies",
      range  = c(3, 14)
    ) +
    ggplot2::labs(
      title    = title_txt,
      subtitle = subtitle_txt,
      x        = "RCS",
      y        = if (lang == "es") "% de estudios" else "% of studies"
    ) +
    ggplot2::theme_minimal(base_size = 11) +
    ggplot2::theme(
      legend.position = "right",
      plot.subtitle   = ggplot2::element_text(size = 9, colour = "grey40")
    )

  ggplot2::ggsave(file.path(dir, "rcs_chart.png"), p,
                  width = 12, height = 8, dpi = 300)
}


#' @noRd
.plot_gap_map_v2 <- function(result, dir, lang, min_records = 1) {
  df <- result$indicators %>%
    dplyr::filter(.data$n_records >= min_records, !is.na(.data$WRDI)) %>%
    dplyr::mutate(label_w = stringr::str_wrap(.data$label, 22))

  title_txt    <- if (lang == "es") "Mapa de lagunas * ORISMA Gap Map"
                  else "ORISMA Gap Map * Evidence Landscape"
  subtitle_txt <- if (lang == "es")
    "Cuadrante superior derecho = sobreestudiado pero sin datos de trabajadores"
  else
    "Upper-right quadrant = over-studied but lacking worker exposure data"

  wrdi_global <- result$WRDI_global

  p <- ggplot2::ggplot(df, ggplot2::aes(
    x      = .data$RCS,
    y      = .data$WRDI,
    size   = .data$n_records,
    colour = .data$RCS
  )) +
    ggplot2::annotate("rect",
                      xmin = 1, xmax = Inf, ymin = wrdi_global, ymax = 1,
                      fill = "#FFE5E0", alpha = 0.3) +
    ggplot2::annotate("text",
                      x = max(df$RCS, na.rm = TRUE) * 0.85,
                      y = 0.97,
                      label = if (lang == "es") "Alta saturacion\nAlta desconexion"
                              else "High saturation\nHigh disconnection",
                      size = 3, colour = "#D85A30", hjust = 1) +
    ggplot2::geom_point(alpha = 0.85) +
    ggrepel::geom_text_repel(ggplot2::aes(label = .data$label_w),
                              size = 2.8, max.overlaps = 15,
                              box.padding = 0.5) +
    ggplot2::geom_hline(yintercept = wrdi_global,
                        linetype = "dashed", colour = "grey50", linewidth = 0.7) +
    ggplot2::geom_vline(xintercept = 1,
                        linetype = "dashed", colour = "grey50", linewidth = 0.7) +
    ggplot2::scale_colour_gradient(low = "#9FE1CB", high = "#0F6E56",
                                   guide = "none") +
    ggplot2::scale_size_continuous(
      name  = if (lang == "es") "N estudios" else "N studies",
      range = c(3, 14)
    ) +
    ggplot2::labs(
      title    = title_txt,
      subtitle = subtitle_txt,
      x        = "RCS (Risk Category Saturation)",
      y        = "WRDI (Worker-Risk Disconnection)"
    ) +
    ggplot2::theme_minimal(base_size = 11) +
    ggplot2::theme(
      plot.subtitle = ggplot2::element_text(size = 9, colour = "grey40")
    )

  ggplot2::ggsave(file.path(dir, "gap_map.png"), p,
                  width = 12, height = 9, dpi = 300)
}


#' @noRd
.plot_temporal_v2 <- function(result, dir, lang, top_n = 8, topic = NULL) {
  if (is.null(result$temporal)) return(invisible(NULL))

  # Top N categories by total records
  top_cats <- result$indicators %>%
    dplyr::arrange(dplyr::desc(.data$n_records)) %>%
    dplyr::slice_head(n = top_n) %>%
    dplyr::pull(.data$category)

  cat_cols <- paste0("cat_", top_cats)
  available <- intersect(cat_cols, names(result$temporal))
  if (length(available) == 0) return(invisible(NULL))

  # Labels for top cats
  cat_labels <- result$indicators %>%
    dplyr::filter(.data$category %in% top_cats) %>%
    dplyr::select(.data$category, .data$label) %>%
    dplyr::mutate(label_w = stringr::str_wrap(.data$label, 28))

  temporal_long <- result$temporal %>%
    tidyr::pivot_longer(
      cols      = dplyr::all_of(available),
      names_to  = "cat_key",
      values_to = "n"
    ) %>%
    dplyr::mutate(category = gsub("^cat_", "", .data$cat_key)) %>%
    dplyr::left_join(cat_labels, by = "category")

  title_txt    <- if (lang == "es") "Evolucion temporal por categoria de riesgo"
                  else "Risk category evolution over time"
  subtitle_txt <- if (lang == "es")
    paste0("Top ", top_n, if (!is.null(topic) && nchar(topic) > 0) paste0(" . ", topic) else "")
  else
    paste0("Top ", top_n, if (!is.null(topic) && nchar(topic) > 0) paste0(" . ", topic) else "")

  p <- ggplot2::ggplot(temporal_long,
                       ggplot2::aes(x     = .data$year,
                                    y     = .data$n,
                                    colour = .data$label_w,
                                    group  = .data$label_w)) +
    ggplot2::geom_line(linewidth = 1.1) +
    ggplot2::geom_point(size = 2.5) +
    ggplot2::scale_x_continuous(breaks = seq(
      min(result$temporal$year, na.rm = TRUE),
      max(result$temporal$year, na.rm = TRUE), 1)
    ) +
    ggplot2::labs(
      title    = title_txt,
      subtitle = subtitle_txt,
      x        = if (lang == "es") "Ano" else "Year",
      y        = if (lang == "es") "Numero de estudios" else "Number of studies",
      colour   = if (lang == "es") "Categoria" else "Category"
    ) +
    ggplot2::theme_minimal(base_size = 11) +
    ggplot2::theme(
      axis.text.x     = ggplot2::element_text(angle = 45, hjust = 1),
      legend.position = "right",
      legend.text     = ggplot2::element_text(size = 9),
      plot.subtitle   = ggplot2::element_text(size = 9, colour = "grey40")
    )

  ggplot2::ggsave(file.path(dir, "temporal_trend.png"), p,
                  width = 13, height = 7, dpi = 300)
}


#' @noRd
.plot_cooccur_v2 <- function(result, dir, lang, min_records = 2) {
  # Filter to categories with at least min_records
  active_cats <- result$indicators %>%
    dplyr::filter(.data$n_records >= min_records) %>%
    dplyr::pull(.data$category)

  mat <- result$cooccur_mat
  idx <- rownames(mat) %in% active_cats
  mat <- mat[idx, idx]

  if (nrow(mat) < 2) return(invisible(NULL))

  # Use labels instead of keys
  label_map <- result$indicators %>%
    dplyr::filter(.data$category %in% rownames(mat)) %>%
    dplyr::select(.data$category, .data$label)

  new_names <- label_map$label[match(rownames(mat), label_map$category)]
  new_names <- stringr::str_wrap(new_names, 25)
  rownames(mat) <- new_names
  colnames(mat) <- new_names

  title_txt <- if (lang == "es") "Matriz de co-ocurrencia de riesgos"
               else "Risk co-occurrence matrix"

  png(file.path(dir, "cooccurrence_heatmap.png"),
      width = 3200, height = 2800, res = 300)
  pheatmap::pheatmap(
    mat,
    cluster_rows  = TRUE,
    cluster_cols  = TRUE,
    color         = grDevices::colorRampPalette(c("white", "#9FE1CB", "#0F6E56"))(50),
    main          = title_txt,
    fontsize      = 8,
    fontsize_row  = 7,
    fontsize_col  = 7,
    border_color  = "white",
    angle_col     = 45
  )
  grDevices::dev.off()
}


#' @noRd
.plot_distribution_v2 <- function(result, dir, lang, min_records = 1) {
  df <- result$indicators %>%
    dplyr::filter(.data$n_records >= min_records) %>%
    dplyr::arrange(dplyr::desc(.data$n_records)) %>%
    dplyr::mutate(
      label_w  = stringr::str_wrap(.data$label, 38),
      block    = dplyr::case_when(
        grepl("Fall|Collision|Entrap|Overturn|Vehicle|Electrical|Thermal contact|Explosion|Fire|Irritant|Toxic",
              .data$label, ignore.case = TRUE) ~ "A - Safety",
        grepl("Chemical|Carcinogen|Asbestos|Ionising|Non-ionising|Noise|Vibration|Thermal stress",
              .data$label, ignore.case = TRUE) ~ "B - Hygiene",
        grepl("Manual|Posture|Repetitive|Mental|Thermal comfort|Lighting|Indoor|Display",
              .data$label, ignore.case = TRUE) ~ "C - Ergonomics",
        grepl("Work content|Workload|Working time|Participation|Role|Professional|Interpersonal|Team|Harassment|Violence|Emotional",
              .data$label, ignore.case = TRUE) ~ "D - Psychosociology",
        grepl("Virus|Bacteria|Fungi|Parasit|Prion",
              .data$label, ignore.case = TRUE) ~ "E - Biological",
        TRUE ~ "F - Emerging"
      )
    )

  block_colors <- c(
    "A - Safety"         = "#D85A30",
    "B - Hygiene"        = "#E8A838",
    "C - Ergonomics"     = "#4DAF8D",
    "D - Psychosociology" = "#7B68EE",
    "E - Biological"     = "#E05C8A",
    "F - Emerging"       = "#0F6E56"
  )

  title_txt    <- if (lang == "es") "Distribucion de categorias de riesgo"
                  else "Risk category distribution"
  subtitle_txt <- if (lang == "es")
    paste0("N = ", result$n_records, " registros * Superdiccionario ORISMA v2.0")
  else
    paste0("N = ", result$n_records, " records * ORISMA Superdictionary v2.0")

  p <- ggplot2::ggplot(df, ggplot2::aes(
    x    = .data$n_records,
    y    = reorder(.data$label_w, .data$n_records),
    fill = .data$block
  )) +
    ggplot2::geom_col(alpha = 0.9) +
    ggplot2::geom_text(
      ggplot2::aes(label = paste0(.data$n_records, " (", .data$pct_records, "%)")),
      hjust = -0.1, size = 3, colour = "grey30"
    ) +
    ggplot2::scale_fill_manual(values = block_colors, name = "Block") +
    ggplot2::scale_x_continuous(expand = ggplot2::expansion(mult = c(0, 0.18))) +
    ggplot2::labs(
      title    = title_txt,
      subtitle = subtitle_txt,
      x        = if (lang == "es") "Numero de registros" else "Number of records",
      y        = NULL
    ) +
    ggplot2::theme_minimal(base_size = 11) +
    ggplot2::theme(
      panel.grid.major.y = ggplot2::element_blank(),
      legend.position    = "right",
      plot.subtitle      = ggplot2::element_text(size = 9, colour = "grey40")
    )

  ggplot2::ggsave(file.path(dir, "risk_distribution.png"), p,
                  width  = 14,
                  height = max(6, nrow(df) * 0.45),
                  dpi    = 300, limitsize = FALSE)
}


# =============================================================================
# IMPROVED HTML REPORT
# =============================================================================

#' @noRd
.write_html_report_v2 <- function(result, path, lang, out_dir, min_records = 1, topic = NULL) {

  is_es   <- lang == "es"
  n_rec   <- result$n_records
  wrdi    <- result$WRDI_global
  wrdi_pct <- round(wrdi * 100, 1)
  n_cats  <- result$n_categories
  ps      <- attr(result, "pipeline_summary")
  n_load  <- if (!is.null(ps)) ps$n_loaded  else n_rec
  n_dedup <- if (!is.null(ps)) ps$n_removed else 0L
  date_str <- format(Sys.time(), "%Y-%m-%d %H:%M")

  # Active categories only
  ind_active <- result$indicators %>%
    dplyr::filter(.data$n_records >= min_records) %>%
    dplyr::arrange(dplyr::desc(.data$n_records))

  # Top gaps: high WRDI + present in literature
  top_gaps <- ind_active %>%
    dplyr::filter(.data$WRDI >= 0.7, .data$n_records >= 2) %>%
    dplyr::arrange(dplyr::desc(.data$WRDI), dplyr::desc(.data$n_records))

  # Translate labels
  lbl <- function(en, es) if (is_es) es else en

  # Build indicator rows
  ind_rows <- paste0(apply(ind_active, 1, function(r) {
    wrdi_val <- as.numeric(r["WRDI"])
    rcs_val  <- as.numeric(r["RCS"])
    wrdi_col <- if (is.na(wrdi_val)) "#888"
                else if (wrdi_val >= 0.8) "#D85A30"
                else if (wrdi_val >= 0.5) "#E8A838"
                else "#0F6E56"
    rcs_badge <- if (is.na(rcs_val)) ""
                 else if (rcs_val > 1)
                   paste0('<span style="background:#E8F5F0;color:#0F6E56;padding:2px 6px;border-radius:4px;font-size:11px">',
                          if(is_es) "sobrerepresentado" else "over-represented", '</span>')
                 else
                   paste0('<span style="background:#FFF3F0;color:#D85A30;padding:2px 6px;border-radius:4px;font-size:11px">',
                          if(is_es) "infrarepresentado" else "under-represented", '</span>')

    paste0(
      "<tr>",
      "<td>", r["label"], "</td>",
      "<td style='text-align:center;font-weight:bold'>", r["n_records"], "</td>",
      "<td style='text-align:center'>", r["pct_records"], "%</td>",
      "<td style='text-align:center;color:", wrdi_col, ";font-weight:bold'>",
        if (is.na(wrdi_val)) "N/A" else round(wrdi_val, 3), "</td>",
      "<td style='text-align:center'>", round(as.numeric(r["RCS"]), 2), "</td>",
      "<td>", rcs_badge, "</td>",
      "</tr>"
    )
  }), collapse = "\n")

  # Gap rows
  gap_rows <- if (nrow(top_gaps) > 0) {
    paste0(apply(top_gaps, 1, function(r) {
      paste0(
        "<tr>",
        "<td>", r["label"], "</td>",
        "<td style='text-align:center;color:#D85A30;font-weight:bold'>",
          round(as.numeric(r["WRDI"]), 3), "</td>",
        "<td style='text-align:center'>", r["n_records"], "</td>",
        "<td>",
          if (is_es)
            "Riesgo presente en literatura pero sin datos directos de trabajadores"
          else
            "Risk present in literature but lacking direct worker exposure data",
        "</td>",
        "</tr>"
      )
    }), collapse = "\n")
  } else {
    paste0("<tr><td colspan='4'>",
           if(is_es) "No se detectaron lagunas criticas" else "No critical gaps detected",
           "</td></tr>")
  }

  # WRDI color for gauge
  wrdi_color <- if (wrdi >= 0.8) "#D85A30"
                else if (wrdi >= 0.5) "#E8A838"
                else "#0F6E56"

  # Plots paths (relative)
  plots_exist <- dir.exists(file.path(out_dir, "plots"))

  html <- paste0('<!DOCTYPE html>
<html lang="', if(is_es) "es" else "en", '">
<head>
<meta charset="UTF-8">
<meta name="viewport" content="width=device-width, initial-scale=1.0">
<title>', if(is_es) "Informe ORISMA" else "ORISMA Report", '</title>
<style>
  :root {
    --green-dark: #0F6E56;
    --green-mid:  #4DAF8D;
    --green-light:#9FE1CB;
    --orange:     #D85A30;
    --amber:      #E8A838;
    --grey-bg:    #F8F9FA;
    --grey-border:#E0E0E0;
  }
  * { box-sizing: border-box; margin: 0; padding: 0; }
  body {
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", sans-serif;
    background: #fff;
    color: #2c2c2c;
    line-height: 1.6;
  }
  .header {
    background: var(--green-dark);
    color: white;
    padding: 2rem 3rem;
  }
  .header h1 { font-size: 2rem; font-weight: 700; letter-spacing: -0.5px; }
  .header p  { opacity: 0.85; margin-top: 0.25rem; font-size: 0.95rem; }
  .header .meta { margin-top: 1rem; font-size: 0.85rem; opacity: 0.7; }
  .container { max-width: 1100px; margin: 0 auto; padding: 2rem 2rem; }
  .cards { display: grid; grid-template-columns: repeat(3, 1fr); gap: 1rem; margin: 2rem 0; }
  .card {
    background: var(--grey-bg);
    border: 1px solid var(--grey-border);
    border-radius: 10px;
    padding: 1.5rem;
    text-align: center;
  }
  .card .value { font-size: 2.5rem; font-weight: 700; }
  .card .label { font-size: 0.85rem; color: #666; margin-top: 0.25rem; }
  .card .sublabel { font-size: 0.75rem; color: #999; margin-top: 0.1rem; }
  .section { margin: 2.5rem 0; }
  .section h2 {
    font-size: 1.2rem;
    font-weight: 600;
    color: var(--green-dark);
    border-bottom: 2px solid var(--green-light);
    padding-bottom: 0.5rem;
    margin-bottom: 1rem;
  }
  .section h3 { font-size: 1rem; font-weight: 600; margin: 1.5rem 0 0.5rem; }
  table { border-collapse: collapse; width: 100%; font-size: 0.88rem; }
  th {
    background: var(--green-dark);
    color: white;
    padding: 10px 12px;
    text-align: left;
    font-weight: 500;
  }
  td { border: 1px solid var(--grey-border); padding: 8px 12px; }
  tr:nth-child(even) td { background: var(--grey-bg); }
  .plot-grid { display: grid; grid-template-columns: 1fr 1fr; gap: 1rem; margin: 1rem 0; }
  .plot-grid img { width: 100%; border-radius: 8px; border: 1px solid var(--grey-border); }
  .plot-full img  { width: 100%; border-radius: 8px; border: 1px solid var(--grey-border); margin: 0.5rem 0; }
  .alert-box {
    background: #FFF8E1;
    border-left: 4px solid var(--amber);
    padding: 1rem 1.25rem;
    border-radius: 0 6px 6px 0;
    margin: 1rem 0;
    font-size: 0.9rem;
  }
  .gap-box {
    background: #FFF3F0;
    border-left: 4px solid var(--orange);
    padding: 1rem 1.25rem;
    border-radius: 0 6px 6px 0;
    margin: 1rem 0;
    font-size: 0.9rem;
  }
  .prisma {
    display: flex;
    gap: 0.5rem;
    align-items: center;
    flex-wrap: wrap;
    margin: 1rem 0;
  }
  .prisma-step {
    background: var(--green-dark);
    color: white;
    padding: 0.4rem 0.8rem;
    border-radius: 20px;
    font-size: 0.82rem;
  }
  .prisma-arrow { color: #999; font-size: 1.2rem; }
  .footer {
    background: var(--grey-bg);
    border-top: 1px solid var(--grey-border);
    padding: 1.5rem 3rem;
    font-size: 0.8rem;
    color: #888;
    margin-top: 3rem;
  }
  @media (max-width: 700px) {
    .cards { grid-template-columns: 1fr; }
    .plot-grid { grid-template-columns: 1fr; }
  }
</style>
</head>
<body>

<div class="header">
  <h1>ORISMA</h1>
  <p>',
    if(!is.null(topic) && nchar(topic) > 0) paste0('<strong>', topic, '</strong> &nbsp;&mdash;&nbsp;') else "",
    if(is_es) "Informe de mapeo sistematico de evidencia sobre riesgos laborales"
    else "Occupational Risk Integrated Systematic Mapping and Analysis Report", '</p>
  <div class="meta">
    ', if(is_es) "Generado el" else "Generated on", ': ', date_str, ' &nbsp;|&nbsp;
    ', if(is_es) "Diccionario" else "Dictionary", ': iso45001_insst v2.0.0 (', length(result$dict), ' ', if(is_es) "categorias" else "categories", ') &nbsp;|&nbsp;
    orisma v0.1.0
  </div>
</div>

<div class="container">

  <!-- PRISMA flow -->
  <div class="section">
    <h2>', if(is_es) "Flujo de seleccion (PRISMA)" else "Selection flow (PRISMA)", '</h2>
    <div class="prisma">
      <div class="prisma-step">',
        if(is_es) paste0("Registros identificados: ", n_load)
        else      paste0("Records identified: ", n_load),
      '</div>
      <div class="prisma-arrow">&#8594;</div>
      <div class="prisma-step">',
        if(is_es) paste0("Duplicados eliminados: ", n_dedup)
        else      paste0("Duplicates removed: ", n_dedup),
      '</div>
      <div class="prisma-arrow">&#8594;</div>
      <div class="prisma-step">',
        if(is_es) paste0("Registros unicos: ", n_rec)
        else      paste0("Unique records: ", n_rec),
      '</div>
      <div class="prisma-arrow">&#8594;</div>
      <div class="prisma-step">',
        if(is_es) paste0("Analizados: ", n_rec)
        else      paste0("Analysed: ", n_rec),
      '</div>
    </div>
  </div>

  <!-- KPI cards -->
  <div class="cards">
    <div class="card">
      <div class="value" style="color:', wrdi_color, '">', wrdi, '</div>
      <div class="label">WRDI ', if(is_es) "(global)" else "(global)", '</div>
      <div class="sublabel">',
        if(is_es) paste0(wrdi_pct, "% sin datos de trabajadores")
        else      paste0(wrdi_pct, "% lack worker exposure data"),
      '</div>
    </div>
    <div class="card">
      <div class="value" style="color:var(--green-dark)">', n_rec, '</div>
      <div class="label">', if(is_es) "Registros analizados" else "Records analysed", '</div>
      <div class="sublabel">WoS + Scopus, ', if(is_es) "deduplicados" else "deduplicated", '</div>
    </div>
    <div class="card">
      <div class="value" style="color:var(--green-dark)">', nrow(ind_active), '</div>
      <div class="label">', if(is_es) "Categorias detectadas" else "Categories detected", '</div>
      <div class="sublabel">', if(is_es) paste0("de ", n_cats, " en el diccionario")
                               else      paste0("of ", n_cats, " in dictionary"), '</div>
    </div>
  </div>

  <!-- Interpretacion automatica -->
  <div class="section">
    <h2>', if(is_es) "Interpretacion automatica de resultados"
           else "Automatic results interpretation", '</h2>
    <div class="alert-box">
      <strong>', if(is_es) "WRDI global: " else "Global WRDI: ", wrdi, '</strong> &mdash; ',
      if (wrdi >= 0.8)
        if(is_es) "ALERTA CRITICA: la gran mayoria de estudios caracteriza riesgos sin medir exposicion real de trabajadores. La evidencia disponible tiene muy baja aplicabilidad preventiva directa."
        else "CRITICAL ALERT: the vast majority of studies characterise risks without measuring real worker exposure. Available evidence has very low direct preventive applicability."
      else if (wrdi >= 0.5)
        if(is_es) "ATENCION: mas de la mitad de los estudios no incluyen datos de exposicion directa de trabajadores. El campo muestra una desconexion tecnico-laboral significativa."
        else "ATTENTION: more than half of studies do not include direct worker exposure data. The field shows significant technical-labour disconnection."
      else
        if(is_es) "El campo muestra una conexion razonable entre investigacion tecnica y exposicion de trabajadores."
        else "The field shows a reasonable connection between technical research and worker exposure.",
    '</div>
  </div>

  <!-- Plots: distribucion y temporal -->
  <div class="section">
    <h2>', if(is_es) "Distribucion de riesgos y evolucion temporal"
           else "Risk distribution and temporal evolution", '</h2>',
  if (plots_exist) paste0('
    <div class="plot-full"><img src="plots/risk_distribution.png"
      alt="', if(is_es) "Distribucion de riesgos" else "Risk distribution", '"></div>
    <div class="plot-full"><img src="plots/temporal_trend.png"
      alt="', if(is_es) "Evolucion temporal" else "Temporal trend", '"></div>')
  else "",
  '</div>

  <!-- Plots: WRDI y RCS -->
  <div class="section">
    <h2>', if(is_es) "Indicadores de desconexion y saturacion"
           else "Disconnection and saturation indicators", '</h2>',
  if (plots_exist) paste0('
    <div class="plot-grid">
      <div><img src="plots/wrdi_chart.png"
        alt="WRDI"></div>
      <div><img src="plots/rcs_chart.png"
        alt="RCS"></div>
    </div>')
  else "",
  '</div>

  <!-- Gap map -->
  <div class="section">
    <h2>', if(is_es) "Mapa de lagunas de conocimiento" else "Knowledge gap map", '</h2>',
  if (plots_exist) paste0('
    <div class="plot-full"><img src="plots/gap_map.png" alt="Gap Map"></div>')
  else "",
  '</div>

  <!-- Co-occurrence -->
  <div class="section">
    <h2>', if(is_es) "Matriz de co-ocurrencia de riesgos"
           else "Risk co-occurrence matrix", '</h2>',
  if (plots_exist) paste0('
    <div class="plot-full"><img src="plots/cooccurrence_heatmap.png"
      alt="', if(is_es) "Co-ocurrencia" else "Co-occurrence", '"></div>')
  else "",
  '</div>

  <!-- ASS distribution -->
  <div class="section">
    <h2>', if(is_es) "Suficiencia de los abstracts (ASS)"
           else "Abstract Sufficiency Score (ASS)", '</h2>
    <div class="alert-box">',
      if(is_es)
        "El ASS mide cuanta informacion preventivamente util contiene cada abstract (0 = no informativo . 5 = exposicion + poblacion + metodo + prevencion)."
      else
        "The ASS measures how much preventively useful information each abstract contains (0 = non-informative . 5 = exposure + population + method + prevention).",
    '</div>',
  if (plots_exist && file.exists(file.path(out_dir, "plots", "ass_distribution.png"))) paste0('
    <div class="plot-full"><img src="plots/ass_distribution.png"
      alt="ASS distribution"></div>')
  else "",
  '</div>

  <!-- Risk x Dimension heatmap -->
  <div class="section">
    <h2>', if(is_es) "Focos de riesgo por bloque normativo"
           else "OHS Risk Focus by Normative Block", '</h2>
    <div class="alert-box">',
      if(is_es)
        "Matriz de co-ocurrencia entre categorias de riesgo y bloques normativos (A-Seguridad, B-Higiene, C-Ergonomia, D-Psicosociologia, E-Biologico, F-Tecnologias emergentes)."
      else
        "Co-occurrence matrix between risk categories and normative blocks (A-Safety, B-Hygiene, C-Ergonomics, D-Psychosociology, E-Biological, F-Emerging technologies).",
    '</div>',
  if (plots_exist && file.exists(file.path(out_dir, "plots", "risk_dimension_heatmap.png"))) paste0('
    <div class="plot-full"><img src="plots/risk_dimension_heatmap.png"
      alt="', if(is_es) "Focos de riesgo" else "Risk focus by block", '"></div>')
  else "",
  '</div>

  <!-- Gap table -->
  <div class="section">
    <h2>', if(is_es) "Lagunas criticas detectadas (WRDI >= 0.7)"
           else "Critical gaps detected (WRDI >= 0.7)", '</h2>
    <div class="gap-box">',
      if(is_es) "Estas categorias de riesgo aparecen en la literatura pero carecen de datos directos de exposicion de trabajadores. Son prioritarias para investigacion futura y evaluacion de riesgos."
      else "These risk categories appear in the literature but lack direct worker exposure data. They are priority areas for future research and risk assessment.",
    '</div>
    <table>
      <thead><tr>
        <th>', if(is_es) "Categoria" else "Category", '</th>
        <th>WRDI</th>
        <th>N</th>
        <th>', if(is_es) "Interpretacion" else "Interpretation", '</th>
      </tr></thead>
      <tbody>', gap_rows, '</tbody>
    </table>
  </div>

  <!-- Full indicators table -->
  <div class="section">
    <h2>', if(is_es) "Indicadores completos por categoria"
           else "Full indicators by category", '</h2>
    <table>
      <thead><tr>
        <th>', if(is_es) "Categoria de riesgo" else "Risk category", '</th>
        <th>N</th>
        <th>%</th>
        <th>WRDI</th>
        <th>RCS</th>
        <th>', if(is_es) "Saturacion" else "Saturation", '</th>
      </tr></thead>
      <tbody>', ind_rows, '</tbody>
    </table>
  </div>

</div><!-- /container -->

<div class="footer">
  <strong>ORISMA v0.1.0</strong> &mdash;
  Dr. Ra&uacute;l Aguilar-Elena (raguilar@universidadviu.com) &middot; GPRL &middot; VIU &nbsp;|&nbsp;
  Ana Delgado-Garc&iacute;a (a.delgado@usal.es) &middot; USAL<br>
  ', if(is_es) "Diccionario" else "Dictionary", ': INSST + ISO 45001 + NIOSH + EU-OSHA &nbsp;|&nbsp;
  ', if(is_es) "Generado el" else "Generated", ': ', date_str, ' &nbsp;|&nbsp;
  <a href="https://github.com/Aguilar-Elena/orisma">github.com/Aguilar-Elena/orisma</a>
</div>

</body>
</html>')

  writeLines(html, path, useBytes = FALSE)
  invisible(path)
}

.build_prisma_log <- function(result, lang) {
  ps <- attr(result, "pipeline_summary")
  data.frame(
    phase = c("Records identified","Duplicates removed",
              "Records after deduplication","Records screened",
              "Records included in analysis"),
    n = c(
      if (!is.null(ps)) ps$n_loaded  else result$n_records,
      if (!is.null(ps)) ps$n_removed else 0L,
      if (!is.null(ps)) ps$n_deduped else result$n_records,
      if (!is.null(ps)) ps$n_deduped else result$n_records,
      result$n_records
    ),
    stringsAsFactors = FALSE
  )
}

.build_certificate <- function(result) {
  list(
    orisma_version = as.character(utils::packageVersion("orisma")),
    analysis_date  = format(Sys.time(), "%Y-%m-%d %H:%M:%S"),
    n_records      = result$n_records,
    n_categories   = result$n_categories,
    WRDI_global    = result$WRDI_global,
    r_version      = paste(R.Version()$major, R.Version()$minor, sep="."),
    platform       = R.Version()$platform,
    digest_matrix  = digest::digest(result$matrix, algo="md5"),
    digest_refs    = digest::digest(result$refs, algo="md5")
  )
}

#' ORISMA bibliometric indicators
#'
#' @name orisma_indicators
#' @aliases WRDI RCS MGP ASS bridge_score
#'
#' @description
#' ORISMA implements five original bibliometric indicators designed
#' specifically for occupational health and safety (OHS) evidence mapping.
#' Three are corpus-level indicators (WRDI, RCS, MGP) and two are
#' record-level indicators (ASS, Bridge Score).
#'
#'
#'
#' ## 1. Worker-Risk Disconnection Index (WRDI)
#'
#' ### Definition
#' The WRDI measures the proportion of studies in a corpus that characterise
#' an occupational risk **without** including direct worker exposure data.
#' A study is considered to have worker exposure data if its abstract
#' contains terms indicating real measurement of exposure in actual workers
#' under real working conditions (e.g. "worker exposure", "occupational
#' exposure", "breathing zone", "personal sampling", "field study",
#' "workplace measurement").
#'
#' ### Formula
#' For a given risk category \eqn{c}:
#' \deqn{WRDI_c = 1 - \frac{N_{workers,c}}{N_{total,c}}}
#' where \eqn{N_{workers,c}} is the number of studies in category \eqn{c}
#' that include worker exposure data, and \eqn{N_{total,c}} is the total
#' number of studies in that category.
#'
#' The **global WRDI** is computed across all records:
#' \deqn{WRDI_{global} = 1 - \frac{N_{workers}}{N_{total}}}
#'
#' ### Interpretation
#' - **WRDI = 0**: All studies include direct worker exposure data.
#'   The body of evidence is fully connected to real workplace conditions.
#' - **WRDI = 1**: No study includes worker exposure data. The entire
#'   literature characterises the risk technically (e.g. in simulated
#'   environments, chambers, or in vitro) without measuring real exposure
#'   in workers.
#' - **WRDI >= 0.8**: Critical alert. The evidence has very low direct
#'   preventive transferability. On-site risk assessment is essential.
#' - **WRDI 0.5-0.8**: Attention required. More than half the evidence
#'   lacks worker data.
#' - **WRDI < 0.3**: Reasonable coverage. Most studies include worker data.
#'
#' ### Important limitation
#' WRDI detection is based on abstract text, not full text. Studies that
#' measured worker exposure but did not mention it in the abstract may
#' be misclassified. Manual validation via [orm_validate()] is recommended.
#'
#'
#'
#' ## 2. Risk Category Saturation Index (RCS)
#'
#' ### Definition
#' The RCS measures the **relative dominance** of a risk category in the
#' corpus compared to a hypothetical uniform distribution across all
#' categories. It identifies which categories are over-represented
#' (saturated) and which are under-represented (gaps) in the literature.
#'
#' ### Formula
#' \deqn{RCS_c = \frac{pct_c}{pct_{uniform}}}
#' where \eqn{pct_c} is the percentage of records assigned to category
#' \eqn{c}, and \eqn{pct_{uniform} = 100 / K} is the percentage each
#' category would have under a uniform distribution across all \eqn{K}
#' categories.
#'
#' Equivalently:
#' \deqn{RCS_c = \frac{N_c \cdot K}{N_{total}}}
#' where \eqn{N_c} is the number of records in category \eqn{c},
#' \eqn{K} is the total number of categories, and \eqn{N_{total}} is
#' the total number of records.
#'
#' ### Interpretation
#' - **RCS > 1**: The category is over-represented relative to a uniform
#'   baseline. The literature has concentrated disproportionately on this
#'   risk type.
#' - **RCS = 1**: The category has exactly the representation expected
#'   under a uniform distribution.
#' - **RCS < 1**: The category is under-represented. This risk type has
#'   received less attention than a balanced literature would suggest.
#' - **RCS = 0**: No studies address this category. Complete evidence gap.
#'
#' ### Note
#' RCS is a relative measure. A category can have RCS > 1 with very few
#' absolute studies if the corpus is small or highly specialised. Always
#' interpret RCS together with the absolute number of records (N).
#'
#'
#'
#' ## 3. Material-Gap Profile (MGP)
#'
#' ### Definition
#' The MGP is a domain-specific indicator designed for corpora where the
#' corpus can be stratified by material, substance, or agent. It measures
#' the ratio between a material's known hazard potential and its coverage
#' in the occupational health literature, identifying materials that are
#' dangerous but understudied.
#'
#' ### Formula
#' \deqn{MGP_m = \frac{hazard\_proxy_m}{coverage_m}}
#' where \eqn{hazard\_proxy_m} is an estimate of the material's hazard
#' potential (based on the number of distinct risk categories detected in
#' studies involving that material), and \eqn{coverage_m} is the proportion
#' of corpus records that address that material.
#'
#' ### Interpretation
#' - **High MGP**: The material is associated with multiple risk categories
#'   but appears in few studies. Priority material for future research and
#'   on-site risk assessment.
#' - **Low MGP**: The material is well-covered in the literature relative
#'   to its known hazard profile.
#' - **MGP requires a material column**: The `material_col` parameter in
#'   [orm_analyse()] must point to a column classifying each record by
#'   material or agent. If not available, MGP is not computed.
#'
#'
#'
#' ## 4. Abstract Sufficiency Score (ASS)
#'
#' ### Definition
#' The ASS is a **cumulative hierarchical index** (0-5) measuring how much
#' preventively useful information an abstract contains for an occupational
#' health practitioner. It is not a measure of study quality, but of
#' abstract informativeness for preventive purposes.
#'
#' The score is strictly cumulative: a record cannot reach level N without
#' satisfying all previous levels.
#'
#' ### Levels
#' \describe{
#'   \item{0 - Non-informative}{The abstract contains no hazard or risk
#'     terms relevant to OHS. No useful preventive information.}
#'   \item{1 - Hazard without context}{The abstract mentions a hazard or
#'     risk agent (e.g. nanoparticles, noise, vibration) but provides no
#'     occupational or workplace context. Could be an environmental or
#'     laboratory study.}
#'   \item{2 - Occupational context}{The abstract mentions workers,
#'     employees, operators, or workplace/occupational setting. The study
#'     is clearly situated in a work context.}
#'   \item{3 - Exposure measurement}{The abstract reports quantitative
#'     exposure data: concentrations, levels, measurements, or monitoring
#'     results. Implies some form of exposure quantification.}
#'   \item{4 - Worker exposure with result}{The abstract explicitly
#'     reports exposure in workers (not just in the environment) with a
#'     result (e.g. exceeded a limit, found significant association,
#'     detected at breathing zone).}
#'   \item{5 - Complete preventive abstract}{The abstract addresses all
#'     four dimensions: worker population + exposure measurement + study
#'     method/design + preventive recommendation or control measure. This
#'     is the highest OHS informative level.}
#' }
#'
#' ### Computation
#' Each level is detected via regular expression patterns applied to the
#' abstract text. Detection is strictly cumulative: the algorithm tests
#' each level in sequence and stops at the first level not satisfied.
#'
#' ### Interpretation
#' - **Mean ASS < 2**: The corpus is predominantly technical with very
#'   little preventive context. High priority for on-site investigation.
#' - **Mean ASS 2-3**: Mixed corpus. Some workplace context but limited
#'   quantitative exposure data.
#' - **Mean ASS > 3**: Good preventive evidence base. Substantial
#'   proportion of studies report actual worker exposure data.
#' - **ASS = 5 articles**: These are the most valuable abstracts for
#'   practitioners and should be read in full first.
#'
#'
#'
#' ## 5. Bridge Article Score
#'
#' ### Definition
#' A bridge article is a study that **connects technical science with
#' applied OHS prevention**. It simultaneously addresses five dimensions
#' that are rarely all present in a single study:
#'
#' \describe{
#'   \item{Criterion 1 - Technology/process}{The study involves a specific
#'     technology, industrial process, or work task (e.g. additive
#'     manufacturing, welding, construction, healthcare).}
#'   \item{Criterion 2 - Hazardous agent}{The study characterises a
#'     specific hazardous agent (chemical, physical, biological, or
#'     psychosocial).}
#'   \item{Criterion 3 - Workers (MANDATORY)}{The study involves a real
#'     worker population in a real workplace setting. This criterion is
#'     mandatory for bridge classification.}
#'   \item{Criterion 4 - Exposure measurement (MANDATORY)}{The study
#'     quantitatively measures exposure (air sampling, biological
#'     monitoring, dosimetry, etc.). This criterion is mandatory for
#'     bridge classification.}
#'   \item{Criterion 5 - Prevention/recommendation}{The study includes
#'     preventive recommendations, control measures, or intervention
#'     results.}
#' }
#'
#' ### Classification
#' \describe{
#'   \item{Strong bridge (score 4-5)}{Meets criteria 3+4 (mandatory) plus
#'     2 or 3 additional criteria. Highest priority for full-text reading.
#'     These articles have already done the translation from laboratory
#'     science to workplace prevention.}
#'   \item{Partial bridge (score 3)}{Meets criteria 3+4 (mandatory) plus
#'     1 additional criterion. Valuable but incomplete bridge.}
#'   \item{Technical study (score 0-2, or missing C3/C4)}{Does not meet
#'     the mandatory criteria. Contributes technical knowledge but lacks
#'     direct preventive applicability.}
#' }
#'
#' ### Priority score
#' The overall **priority reading score** used in [orm_ranking()] combines
#' all record-level indicators:
#' \deqn{Priority = (Bridge \times 2) + (ASS \times 1.5) + (N_{cats} \times 0.5)}
#' where \eqn{N_{cats}} is the number of risk categories detected in the
#' record. Bridge score is weighted highest because it reflects the most
#' direct preventive relevance.
#'
#'
#'
#' ## References
#'
#' The WRDI, RCS, and MGP indicators were first described in:
#'
#' Aguilar-Elena, R. & Delgado-Garcia, A. (2025). *Mapping the Safety
#' Landscape of Emerging Technologies: A Bibliometric Analysis of
#' Occupational Risks in Metal Additive Manufacturing*. (Under review)
#'
#' The ORISMA methodological framework is described in:
#'
#' Aguilar-Elena, R. & Delgado-Garcia, A. (2025). *orisma: A Framework
#' for Occupational Risk Integrated Systematic Mapping and Analysis*.
#' R package version 0.1.0. Universidad Internacional de Valencia (VIU) &
#' Universidad de Salamanca (USAL).
#'
#' @seealso
#' [orm_analyse()] to compute WRDI, RCS, and MGP.
#' [orm_ass()] to compute the Abstract Sufficiency Score.
#' [orm_bridge()] to detect bridge articles.
#' [orm_ranking()] to generate a priority reading list.
#' [orm_validate()] to validate automatic classification with Cohen's Kappa.
#'
NULL

#' Compute risk priority scores and traffic light classification
#'
#' @description
#' `orm_priority()` assigns a priority level to each detected risk category
#' using three criteria combined into a single priority score:
#'
#' - **Frequency** (RCS): how saturated is this category in the literature
#' - **Disconnection** (WRDI): how far is the research from real worker data
#' - **Evidence volume**: number of records
#'
#' Categories whose RCS exceeds `context_rcs_threshold` are flagged as
#' **context categories** (the dominant topic of the corpus, not a risk per se)
#' and are reported separately rather than mixed with risk categories.
#'
#' Priority levels for non-context categories:
#' - **RED**: WRDI >= wrdi_high AND RCS >= 1. Over-studied technically but
#'   no worker data. Urgent preventive gap.
#' - **AMBER**: Moderate evidence OR partial worker data.
#' - **GREEN**: WRDI < wrdi_low. Good worker data connection.
#' - **GREY**: n_records < min_records. Insufficient evidence.
#'
#' @param result An `orisma_result` object.
#' @param min_records Integer. Min records for evaluation. Default `2`.
#' @param wrdi_high Numeric. WRDI threshold for high disconnection. Default `0.7`.
#' @param wrdi_low Numeric. WRDI threshold for low disconnection. Default `0.3`.
#' @param context_rcs_threshold Numeric. RCS above which a category is
#'   considered a context category (dominant topic) rather than a risk.
#'   Default `15`.
#' @param lang Character. `"en"` or `"es"`.
#'
#' @return A list with two data frames: `$risks` (priority-classified risk
#'   categories) and `$context` (dominant topic categories).
#' @export
orm_priority <- function(result,
                          min_records            = 2L,
                          wrdi_high              = 0.7,
                          wrdi_low               = 0.3,
                          context_rcs_threshold  = 15,
                          lang                   = getOption("orisma.lang", "en")) {

  .check_lang(lang)
  if (!inherits(result, "orisma_result"))
    stop("'result' must be an orisma_result object.", call. = FALSE)

  is_es <- lang == "es"
  ind   <- result$indicators

  # ── Separate context categories from risk categories ─────────────────────────
  context_cats <- ind %>%
    dplyr::filter(!is.na(.data$RCS), .data$RCS >= context_rcs_threshold)

  risk_cats <- ind %>%
    dplyr::filter(is.na(.data$RCS) | .data$RCS < context_rcs_threshold)

  # ── Classify risk categories ─────────────────────────────────────────────────
  risk_classified <- risk_cats %>%
    dplyr::mutate(
      priority = dplyr::case_when(
        .data$n_records < min_records               ~ "GREY",
        is.na(.data$WRDI)                           ~ "GREY",
        .data$WRDI >= wrdi_high & .data$RCS >= 1   ~ "RED",
        .data$WRDI >= wrdi_high & .data$RCS <  1   ~ "AMBER",
        .data$WRDI <  wrdi_low                      ~ "GREEN",
        TRUE                                        ~ "AMBER"
      ),
      priority_label = dplyr::case_when(
        .data$priority == "RED"   ~
          if(is_es) "ROJO: brecha preventiva urgente" else "RED: urgent preventive gap",
        .data$priority == "AMBER" ~
          if(is_es) "AMBAR: requiere atencion"        else "AMBER: requires attention",
        .data$priority == "GREEN" ~
          if(is_es) "VERDE: cobertura razonable"      else "GREEN: reasonable coverage",
        TRUE ~
          if(is_es) "GRIS: evidencia insuficiente"    else "GREY: insufficient evidence"
      ),
      priority_reason = dplyr::case_when(
        .data$priority == "RED" ~
          if(is_es)
            paste0("Sobreestudiado tecnicamente (RCS=", round(.data$RCS,1),
                   ") pero sin datos de trabajadores (WRDI=", round(.data$WRDI,2), ")")
          else
            paste0("Over-studied technically (RCS=", round(.data$RCS,1),
                   ") but no worker data (WRDI=", round(.data$WRDI,2), ")"),
        .data$priority == "AMBER" ~
          if(is_es)
            paste0("Evidencia parcial. WRDI=", round(.data$WRDI,2),
                   ", RCS=", round(.data$RCS,1))
          else
            paste0("Partial evidence. WRDI=", round(.data$WRDI,2),
                   ", RCS=", round(.data$RCS,1)),
        .data$priority == "GREEN" ~
          if(is_es)
            paste0("Buena conexion con datos de trabajadores (WRDI=",
                   round(.data$WRDI,2), ")")
          else
            paste0("Good connection with worker data (WRDI=",
                   round(.data$WRDI,2), ")"),
        TRUE ~
          if(is_es)
            paste0("Solo ", .data$n_records, " estudio(s). Evidencia insuficiente.")
          else
            paste0("Only ", .data$n_records, " study(ies). Insufficient evidence.")
      ),
      confidence = dplyr::case_when(
        .data$n_records >= 10 ~ if(is_es) "Alta"  else "High",
        .data$n_records >= 3  ~ if(is_es) "Media" else "Medium",
        .data$n_records >= 1  ~ if(is_es) "Baja"  else "Low",
        TRUE                  ~ if(is_es) "Nula"  else "None"
      )
    ) %>%
    dplyr::select(
      .data$category, .data$label, .data$n_records, .data$pct_records,
      .data$WRDI, .data$RCS, .data$priority, .data$priority_label,
      .data$priority_reason, .data$confidence
    ) %>%
    dplyr::arrange(
      factor(.data$priority, levels = c("RED","AMBER","GREEN","GREY")),
      dplyr::desc(.data$n_records)
    )

  result_list <- list(
    risks   = risk_classified,
    context = context_cats,
    params  = list(
      min_records           = min_records,
      wrdi_high             = wrdi_high,
      wrdi_low              = wrdi_low,
      context_rcs_threshold = context_rcs_threshold,
      lang                  = lang
    )
  )
  class(result_list) <- c("orisma_priority", "list")
  result_list
}


#' Print method for orisma_priority
#' @param x An `orisma_priority` object.
#' @param ... Further arguments (ignored).
#' @return Invisibly returns `x`.
#' @export
print.orisma_priority <- function(x, ...) {
  lang  <- x$params$lang
  is_es <- lang == "es"

  cat("\n-- ORISMA Risk Priority Classification --\n\n")

  if (nrow(x$context) > 0) {
    cat(if(is_es) "Categorias de CONTEXTO (tema dominante del corpus):\n"
        else "CONTEXT categories (dominant corpus topic):\n")
    for (i in seq_len(nrow(x$context))) {
      cat(sprintf("  [*] %-45s  N=%-4d  RCS=%.1f\n",
                  substr(x$context$label[i], 1, 45),
                  x$context$n_records[i],
                  x$context$RCS[i]))
    }
    cat("\n")
  }

  for (lvl in c("RED","AMBER","GREEN","GREY")) {
    sub <- x$risks[x$risks$priority == lvl, ]
    if (nrow(sub) == 0) next
    sym <- switch(lvl, RED="X", AMBER="!", GREEN="v", "?")
    cat(sprintf("[%s] %s (%d)\n", sym, lvl, nrow(sub)))
    for (i in seq_len(nrow(sub))) {
      cat(sprintf("    %-45s  N=%-3d  WRDI=%.2f  RCS=%.1f  conf=%s\n",
                  substr(sub$label[i], 1, 45),
                  sub$n_records[i],
                  ifelse(is.na(sub$WRDI[i]), 0, sub$WRDI[i]),
                  ifelse(is.na(sub$RCS[i]),  0, sub$RCS[i]),
                  sub$confidence[i]))
    }
    cat("\n")
  }
  invisible(x)
}

#' Run ORISMA with a relevance-control layer
#'
#' @description
#' Runs ORISMA after applying `orm_relevance_guard()`. This is useful for
#' real-world bibliographic searches where broad database queries may retrieve
#' technically related but non-occupational or off-topic records.
#'
#' @param refs A data frame of references, usually produced by `orm_load()`.
#' @param topic Topic label passed to `orm_relevance_guard()` and `orm_run()`.
#' @param exclude_non_relevant Logical. If `TRUE`, records flagged as non-relevant
#' are excluded before running the main ORISMA pipeline.
#' @param min_records Minimum number of records required after filtering. If the
#' filter leaves fewer records, the function stops to avoid accidental over-filtering.
#' @param topic_regex Optional topic regex.
#' @param occupational_regex Optional occupational relevance regex.
#' @param noise_regex Optional noise regex.
#' @param mode Relevance filtering mode. `"flag"` excludes only records outside
#' the target topic and marks uncertain records for review. `"conservative"`
#' excludes off-topic and likely non-occupational biomedical/clinical records.
#' `"strict"` also excludes records with weak occupational context.
#' @param ... Additional arguments passed to `orm_run()`.
#'
#' @return An ORISMA result object with an added `relevance_guard` component.
#' @export
orm_run_guarded <- function(refs,
                            topic = NULL,
                            exclude_non_relevant = TRUE,
                            min_records = 50,
                            topic_regex = NULL,
                            occupational_regex = NULL,
                            noise_regex = NULL,
                            mode = c("conservative", "flag", "strict"),
                            ...) {

  mode <- match.arg(mode)

  if (!is.data.frame(refs)) {
    stop("`refs` must be a data frame. Use orm_load() first.", call. = FALSE)
  }

  guarded <- orm_relevance_guard(
    refs,
    topic = topic,
    topic_regex = topic_regex,
    occupational_regex = occupational_regex,
    noise_regex = noise_regex,
    mode = mode
  )

  relevance_summary <- data.frame(
    metric = c(
      "records_before_guard",
      "topic_relevant",
      "occupational_relevant",
      "biomedical_noise",
      "excluded_by_guard",
      "records_after_guard",
      "guard_mode"
    ),
    value = c(
      nrow(guarded),
      sum(guarded$topic_relevant, na.rm = TRUE),
      sum(guarded$occupational_relevant, na.rm = TRUE),
      sum(guarded$biomedical_noise, na.rm = TRUE),
      sum(guarded$exclusion_flag, na.rm = TRUE),
      sum(!guarded$exclusion_flag, na.rm = TRUE),
      mode
    )
  )

  message("ORISMA relevance guard")
  message("Mode: ", mode)
  message("Records before guard: ", nrow(guarded))
  message("Records excluded: ", sum(guarded$exclusion_flag, na.rm = TRUE))
  message("Records retained: ", sum(!guarded$exclusion_flag, na.rm = TRUE))

  refs_for_run <- guarded

  if (isTRUE(exclude_non_relevant)) {
    refs_for_run <- guarded[!guarded$exclusion_flag, , drop = FALSE]
  }

  if (nrow(refs_for_run) < min_records) {
    stop(
      "The relevance guard retained only ", nrow(refs_for_run),
      " records. This is below `min_records = ", min_records, "`. ",
      "Review the regex rules or set `exclude_non_relevant = FALSE`.",
      call. = FALSE
    )
  }

  result <- orm_run(
    refs_for_run,
    topic = topic,
    ...
  )

  result$relevance_guard <- list(
    summary = relevance_summary,
    all_records = guarded,
    analysed_records = refs_for_run,
    exclude_non_relevant = exclude_non_relevant,
    topic_regex = topic_regex,
    occupational_regex = occupational_regex,
    noise_regex = noise_regex,
    mode = mode
  )

  result
}

#' Extract risk categories from bibliographic records
#'
#' @description
#' `orm_extract()` scans the **title**, **abstract**, and **keywords** of each
#' record against the active risk dictionary and builds a **binary presence
#' matrix** (record x risk category). It also detects whether each study
#' contains direct worker exposure data - the key signal for computing the
#' **WRDI** indicator.
#'
#' Matching is case-insensitive and uses whole-word boundary detection to
#' avoid false positives (e.g. "laser" does not match "eyelaser").
#'
#' @param refs An `orisma_refs` object (output of [orm_load()] or [orm_dedup()]).
#' @param dict An `orisma_dict` object. Default: [orm_dict()] (ISO 45001 /
#'   INSST / NIOSH).
#' @param fields Character vector. Which text fields to search. Default
#'   `c("title", "abstract", "keywords")`.
#' @param lang Character. `"en"` or `"es"`.
#' @param verbose Logical. Print progress?
#'
#' @return A list (class `orisma_matrix`) containing:
#'   \describe{
#'     \item{`refs`}{Original `orisma_refs` tibble with added columns:
#'       one binary column per risk category (`cat_*`), `n_categories` (total
#'       categories matched), and `has_worker_data` (logical).}
#'     \item{`matrix`}{Pure binary matrix (records x categories) for
#'       downstream analysis.}
#'     \item{`dict`}{The dictionary used.}
#'     \item{`categories`}{Category metadata tibble.}
#'   }
#'
#' @examples
#' \dontrun{
#' refs   <- orm_load("my_references/")
#' deduped <- orm_dedup(refs)
#'
#' # Use default dictionary
#' mx <- orm_extract(deduped)
#'
#' # Use a customised dictionary
#' dict <- orm_dict()
#' dict <- orm_dict_add_terms(dict, "nanoparticles", c("nano-dust", "UFP"))
#' mx   <- orm_extract(deduped, dict = dict)
#'
#' # Restrict to title + abstract only
#' mx <- orm_extract(deduped, fields = c("title", "abstract"))
#' }
#'
#' @export
orm_extract <- function(refs,
                        dict    = orm_dict(),
                        fields  = c("title", "abstract", "keywords"),
                        lang    = getOption("orisma.lang", "en"),
                        verbose = getOption("orisma.verbose", TRUE)) {

  .check_lang(lang)

  if (verbose) {
    cli::cli_h1(orm_msg("phase_extract", lang))
    cli::cli_alert_info(
      orm_msg("extract_start", lang,
           dict_name    = attr(dict, "dict_name"),
           dict_version = attr(dict, "dict_version"))
    )
  }

  # -- 1. Build searchable text corpus -----------------------------------------
  available_fields <- intersect(fields, names(refs))
  if (length(available_fields) == 0) {
    stop("None of the specified fields found in the data.", call. = FALSE)
  }

  refs <- refs %>%
    dplyr::mutate(
      .text_corpus = tolower(
        apply(dplyr::select(., dplyr::all_of(available_fields)), 1,
              function(x) paste(x[!is.na(x)], collapse = " "))
      )
    )

  # -- 2. Match each category ---------------------------------------------------
  cat_names   <- names(dict)
  n_cats      <- length(cat_names)
  n_records   <- nrow(refs)

  binary_mat  <- matrix(0L, nrow = n_records, ncol = n_cats,
                        dimnames = list(refs$record_id, cat_names))
  worker_flag <- logical(n_records)

  if (verbose) {
    cli::cli_progress_bar("Scanning records", total = n_records)
  }

  for (i in seq_len(n_records)) {
    text <- refs$.text_corpus[[i]]
    if (is.na(text) || nchar(text) == 0) next

    for (j in seq_len(n_cats)) {
      cat_key <- cat_names[[j]]
      entry   <- dict[[cat_key]]

      # Build regex: whole-word boundary for each term
      pattern <- paste0(
        "\\b(", paste(stringr::str_escape(entry$terms), collapse = "|"), ")\\b"
      )

      if (grepl(pattern, text, ignore.case = TRUE, perl = TRUE)) {
        binary_mat[i, j] <- 1L
      }

      # Worker exposure detection
      if (!worker_flag[[i]] && length(entry$worker_exposure_terms) > 0) {
        wp <- paste0(
          "\\b(",
          paste(stringr::str_escape(entry$worker_exposure_terms),
                collapse = "|"),
          ")\\b"
        )
        if (grepl(wp, text, ignore.case = TRUE, perl = TRUE)) {
          worker_flag[[i]] <- TRUE
        }
      }
    }

    if (verbose) cli::cli_progress_update()
  }

  if (verbose) cli::cli_progress_done()

  # -- 3. Attach results back to refs tibble ------------------------------------
  cat_df <- as.data.frame(binary_mat)
  names(cat_df) <- paste0("cat_", names(cat_df))

  refs <- dplyr::bind_cols(refs, cat_df) %>%
    dplyr::mutate(
      n_categories    = rowSums(binary_mat),
      has_worker_data = worker_flag
    ) %>%
    dplyr::select(-".text_corpus")

  # -- 4. Warn about empty matches ----------------------------------------------
  n_empty <- sum(refs$n_categories == 0)
  if (n_empty > 0 && verbose) {
    cli::cli_alert_warning(orm_msg("extract_empty", lang, n = n_empty))
  }

  if (verbose) {
    cli::cli_alert_success(
      orm_msg("extract_done", lang,
           n_records = n_records,
           n_cats    = n_cats)
    )
  }

  # -- 5. Assemble output object -------------------------------------------------
  result <- list(
    refs       = refs,
    matrix     = binary_mat,
    dict       = dict,
    categories = orm_dict_categories(dict, lang = lang),
    fields_used = available_fields,
    n_records  = n_records,
    n_empty    = n_empty
  )

  class(result) <- c("orisma_matrix", "list")
  attr(result, "orisma_stage")    <- "extracted"
  attr(result, "orisma_lang")     <- lang
  attr(result, "orisma_created")  <- Sys.time()

  result
}


#' Print method for orisma_matrix
#' @param x An object to print.
#' @param ... Further arguments passed to or from other methods.
#' @export
#' @return Invisibly returns the input `orisma_matrix` object. Called primarily for its console-printing side effect.
print.orisma_matrix <- function(x, ...) {
  cat("\n-- ORISMA extraction matrix ------------------------------\n")
  cat(" Records:    ", x$n_records, "\n")
  cat(" Categories: ", ncol(x$matrix), "\n")
  cat(" Fields used:", paste(x$fields_used, collapse = ", "), "\n")
  cat(" Empty records (no category matched):", x$n_empty, "\n")
  cat("\n Category coverage:\n")
  coverage <- colSums(x$matrix)
  pct      <- round(100 * coverage / x$n_records, 1)
  cat_info <- data.frame(
    Category = x$categories$label,
    N        = coverage,
    `%`      = pct,
    check.names = FALSE
  )
  cat_info <- cat_info[order(-cat_info$N), ]
  print(cat_info, row.names = FALSE)
  invisible(x)
}

#' Generate an occupational risk sheet
#'
#' @description
#' `orm_risk_sheet()` generates a structured, actionable risk sheet for
#' occupational health practitioners. It synthesises ORISMA outputs into
#' a single HTML document that can be used as supporting evidence in a
#' workplace risk assessment.
#'
#' The sheet is **regulation-neutral**: it does not include country-specific
#' regulations or limit values, as these vary by jurisdiction. The practitioner
#' applies the relevant national/regional regulation based on the risk
#' categories identified.
#'
#' Content:
#' - Context analysis (dominant topic of the corpus)
#' - Priority traffic light (RED/AMBER/GREEN/GREY) per risk category
#' - Evidence summary with confidence level
#' - Knowledge gap alerts
#' - WRDI interpretation with confidence score
#' - Methodological section (bases, deduplication, WRDI definition, limits)
#'
#' @param result An `orisma_result` object.
#' @param topic Character. Technology or domain being assessed.
#' @param search_strategy Character or NULL. Description of the search strategy
#'   used (databases, keywords, date range). If NULL, a placeholder is used.
#' @param inclusion_criteria Character or NULL. Description of inclusion/
#'   exclusion criteria applied. If NULL, ORISMA defaults are described.
#' @param out_dir Character. Output directory.
#' @param lang Character. `"en"` or `"es"`.
#' @param min_records Integer. Min records for a category to appear. Default `1`.
#' @param context_rcs_threshold Numeric. RCS threshold for context detection.
#'   Default `15`.
#' @param verbose Logical.
#'
#' @return Invisibly returns the path to the generated HTML risk sheet.
#' @export
orm_risk_sheet <- function(result,
                            topic                 = "Occupational risk analysis",
                            search_strategy       = NULL,
                            inclusion_criteria    = NULL,
                            out_dir               = "orisma_output",
                            lang                  = getOption("orisma.lang", "en"),
                            min_records           = 1L,
                            context_rcs_threshold = 15,
                            verbose               = getOption("orisma.verbose", TRUE)) {

  .check_lang(lang)
  if (!inherits(result, "orisma_result"))
    stop("'result' must be an orisma_result object.", call. = FALSE)

  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)

  is_es    <- lang == "es"
  date_str <- format(Sys.time(), "%Y-%m-%d")
  n_cats   <- length(result$dict)

  # Get priority classification
  priority <- orm_priority(result,
                            min_records           = min_records,
                            context_rcs_threshold = context_rcs_threshold,
                            lang                  = lang)

  risks   <- priority$risks
  context <- priority$context

  n_red   <- sum(risks$priority == "RED",   na.rm = TRUE)
  n_amber <- sum(risks$priority == "AMBER", na.rm = TRUE)
  n_green <- sum(risks$priority == "GREEN", na.rm = TRUE)
  n_grey  <- sum(risks$priority == "GREY",  na.rm = TRUE)

  ps      <- attr(result, "pipeline_summary")
  n_load  <- if (!is.null(ps)) ps$n_loaded  else result$n_records
  n_dedup <- if (!is.null(ps)) ps$n_removed else 0L
  n_final <- result$n_records
  wrdi    <- result$WRDI_global
  wrdi_pct <- round(wrdi * 100, 1)

  # WRDI interpretation
  wrdi_interp <- if (wrdi >= 0.8)
    if(is_es) "CRITICO: mas del 80% de los estudios no incluyen datos directos de exposicion de trabajadores"
    else "CRITICAL: more than 80% of studies lack direct worker exposure data"
  else if (wrdi >= 0.5)
    if(is_es) "MODERADO: mas de la mitad de los estudios carecen de datos de exposicion real de trabajadores"
    else "MODERATE: more than half of studies lack real worker exposure data"
  else
    if(is_es) "ACEPTABLE: la mayoria de los estudios incluyen datos de exposicion de trabajadores"
    else "ACCEPTABLE: most studies include worker exposure data"

  wrdi_color <- if (wrdi >= 0.8) "#D85A30" else if (wrdi >= 0.5) "#E8A838" else "#0F6E56"

  # Search strategy placeholder
  search_txt <- if (!is.null(search_strategy)) search_strategy
  else if (is_es)
    "Busqueda sistematica en bases de datos bibliograficas. Consulte el fichero prisma_log.csv para el detalle completo del flujo de seleccion de estudios."
  else
    "Systematic search in bibliographic databases. See prisma_log.csv for the full study selection flow."

  # Inclusion criteria placeholder
  incl_txt <- if (!is.null(inclusion_criteria)) inclusion_criteria
  else if (is_es)
    "Articulos en ingles o espanol, cualquier diseno de estudio, sin restriccion temporal. Excluidos: editoriales, cartas, actas de congresos sin abstract."
  else
    "Articles in English or Spanish, any study design, no date restriction. Excluded: editorials, letters, conference abstracts without full text."

  # Build risk rows
  .risk_rows <- function(df) {
    if (nrow(df) == 0) return("")
    paste0(apply(df, 1, function(r) {
      lvl <- r["priority"]
      color <- switch(lvl, RED="#D85A30", AMBER="#E8A838", GREEN="#0F6E56", "#888")
      bg    <- switch(lvl, RED="#FFF3F0", AMBER="#FFFBF0", GREEN="#F0FBF7", "#F8F8F8")
      wrdi_val <- as.numeric(r["WRDI"])
      wrdi_bar <- if (!is.na(wrdi_val))
        paste0('<div style="background:#E0E0E0;border-radius:3px;height:6px;margin-top:4px">',
               '<div style="background:', color, ';width:', round(wrdi_val*100), '%;height:6px;border-radius:3px"></div></div>')
      else ""

      paste0(
        '<tr style="background:', bg, '">',
        '<td style="border-left:4px solid ', color, ';padding:10px 12px">',
          '<strong>', r["label"], '</strong><br>',
          '<small style="color:', color, ';font-weight:600">',
            r["priority_label"], '</small>',
        '</td>',
        '<td style="text-align:center;padding:10px;font-weight:700">',
          r["n_records"],
        '</td>',
        '<td style="text-align:center;padding:10px">',
          '<span style="font-weight:700;color:', color, '">',
            if(is.na(wrdi_val)) "N/A" else paste0(round(wrdi_val*100,0), "%"),
          '</span>', wrdi_bar,
        '</td>',
        '<td style="text-align:center;padding:10px">',
          r["confidence"],
        '</td>',
        '<td style="padding:10px;color:#555;font-size:12px">',
          r["priority_reason"],
        '</td>',
        '</tr>'
      )
    }), collapse = "\n")
  }

  # Context rows
  context_rows <- if (nrow(context) > 0) {
    paste0(apply(context, 1, function(r) {
      paste0(
        '<tr style="background:#F0F8FF">',
        '<td style="border-left:4px solid #4A90D9;padding:10px 12px">',
          '<strong>', r["label"], '</strong><br>',
          '<small style="color:#4A90D9">',
            if(is_es) "Tema dominante del corpus - contexto tecnologico"
            else "Dominant corpus topic - technological context",
          '</small>',
        '</td>',
        '<td style="text-align:center;padding:10px;font-weight:700">', r["n_records"], '</td>',
        '<td style="text-align:center;padding:10px">',
          round(as.numeric(r["RCS"]), 1), 'x', '</td>',
        '<td colspan="2" style="padding:10px;color:#555;font-size:12px">',
          if(is_es)
            "Esta categoria describe el contexto tecnologico del corpus, no un riesgo laboral especifico. Los riesgos asociados se detallan en las secciones siguientes."
          else
            "This category describes the technological context of the corpus, not a specific occupational risk. Associated risks are detailed in the sections below.",
        '</td>',
        '</tr>'
      )
    }), collapse = "\n")
  } else ""

  html <- paste0('<!DOCTYPE html>
<html lang="', if(is_es) "es" else "en", '">
<head>
<meta charset="UTF-8">
<meta name="viewport" content="width=device-width, initial-scale=1.0">
<title>ORISMA - ', if(is_es) "Ficha de Riesgo Laboral" else "Occupational Risk Sheet", '</title>
<style>
  :root { --green:#0F6E56; --red:#D85A30; --amber:#E8A838; --blue:#4A90D9; }
  * { box-sizing:border-box; margin:0; padding:0; }
  body { font-family:-apple-system,BlinkMacSystemFont,"Segoe UI",sans-serif;
         max-width:1050px; margin:0 auto; padding:2rem; color:#2c2c2c;
         line-height:1.5; }
  .header { background:var(--green); color:white; padding:1.5rem 2rem;
             border-radius:10px; margin-bottom:2rem; }
  .header h1 { font-size:1.6rem; margin-bottom:0.25rem; }
  .header .sub { opacity:0.85; font-size:0.95rem; }
  .header .meta { margin-top:0.75rem; font-size:0.8rem; opacity:0.7; }
  .kpis { display:grid; grid-template-columns:repeat(5,1fr); gap:0.75rem;
           margin:1.5rem 0; }
  .kpi { background:#F8F9FA; border:1px solid #E0E0E0; border-radius:8px;
          padding:1rem; text-align:center; }
  .kpi .val { font-size:1.8rem; font-weight:700; }
  .kpi .lbl { font-size:0.75rem; color:#666; margin-top:0.2rem; }
  .section { margin:2rem 0; }
  .section h2 { font-size:1.05rem; font-weight:600; color:var(--green);
                 border-bottom:2px solid #9FE1CB; padding-bottom:0.4rem;
                 margin-bottom:1rem; }
  .section h3 { font-size:0.95rem; font-weight:600; color:#444;
                 margin:1.5rem 0 0.5rem; }
  table { border-collapse:collapse; width:100%; font-size:0.86rem; }
  th { background:var(--green); color:white; padding:9px 12px; text-align:left;
       font-weight:500; }
  td { border-bottom:1px solid #EEEEEE; vertical-align:top; }
  .alert-red { background:#FFF3F0; border-left:4px solid var(--red);
                padding:0.9rem 1.1rem; border-radius:0 6px 6px 0;
                margin:0.75rem 0; font-size:0.88rem; }
  .alert-blue { background:#F0F8FF; border-left:4px solid var(--blue);
                 padding:0.9rem 1.1rem; border-radius:0 6px 6px 0;
                 margin:0.75rem 0; font-size:0.88rem; }
  .alert-amber { background:#FFFBF0; border-left:4px solid var(--amber);
                  padding:0.9rem 1.1rem; border-radius:0 6px 6px 0;
                  margin:0.75rem 0; font-size:0.88rem; }
  .wrdi-global { background:#F8F9FA; border-radius:8px; padding:1rem 1.5rem;
                  margin:1rem 0; border:1px solid #E0E0E0; }
  .wrdi-bar-big { background:#E0E0E0; border-radius:6px; height:12px; margin:0.5rem 0; }
  .wrdi-fill-big { height:12px; border-radius:6px; }
  .meth-box { background:#F8F9FA; border-radius:8px; padding:1.25rem 1.5rem;
               border:1px solid #E0E0E0; font-size:0.85rem; }
  .meth-box ul { padding-left:1.25rem; }
  .meth-box li { margin:0.4rem 0; color:#555; }
  .footer { background:#F8F9FA; border-top:1px solid #E0E0E0; padding:1rem 1.5rem;
             font-size:0.78rem; color:#888; margin-top:2.5rem;
             border-radius:0 0 8px 8px; }
  .legend { display:flex; gap:1rem; flex-wrap:wrap; margin:0.5rem 0;
             font-size:0.82rem; }
  .legend-item { display:flex; align-items:center; gap:0.4rem; }
  .legend-dot { width:12px; height:12px; border-radius:50%; }
  @media print { body{max-width:100%;padding:1rem} .kpis{grid-template-columns:repeat(5,1fr)} }
</style>
</head>
<body>

<div class="header">
  <h1>ORISMA &mdash; ', if(is_es) "Ficha de Riesgos Laborales" else "Occupational Risk Sheet", '</h1>
  <div class="sub"><strong>', topic, '</strong></div>
  <div class="meta">
    ', if(is_es) "Generado" else "Generated", ': ', date_str,
    ' &nbsp;|&nbsp; orisma v0.1.0 &nbsp;|&nbsp; ',
    if(is_es) "Diccionario" else "Dictionary", ': iso45001_insst v2.0.0 (',
    n_cats, if(is_es) ' categorias)' else ' categories)',
  '</div>
</div>

<!-- KPIs -->
<div class="kpis">
  <div class="kpi">
    <div class="val" style="color:', wrdi_color, '">', wrdi, '</div>
    <div class="lbl">WRDI Global</div>
  </div>
  <div class="kpi">
    <div class="val" style="color:#D85A30">', n_red, '</div>
    <div class="lbl">', if(is_es) "Brechas CRITICAS" else "CRITICAL gaps", '</div>
  </div>
  <div class="kpi">
    <div class="val" style="color:#E8A838">', n_amber, '</div>
    <div class="lbl">', if(is_es) "Atencion" else "Attention", '</div>
  </div>
  <div class="kpi">
    <div class="val" style="color:#0F6E56">', n_green, '</div>
    <div class="lbl">', if(is_es) "Bien cubiertos" else "Well covered", '</div>
  </div>
  <div class="kpi">
    <div class="val" style="color:#888">', n_grey, '</div>
    <div class="lbl">', if(is_es) "Sin evidencia" else "No evidence", '</div>
  </div>
</div>

<!-- WRDI global -->
<div class="section">
  <h2>', if(is_es) "Indice de Desconexion Tecnico-Laboral (WRDI)"
         else "Worker-Risk Disconnection Index (WRDI)", '</h2>
  <div class="wrdi-global">
    <strong>WRDI = ', wrdi, ' (', wrdi_pct, '%)</strong> &mdash; ', wrdi_interp, '
    <div class="wrdi-bar-big">
      <div class="wrdi-fill-big" style="background:', wrdi_color,
           ';width:', wrdi_pct, '%"></div>
    </div>
    <small style="color:#888">',
      if(is_es)
        "El WRDI mide la proporcion de estudios que caracterizan un riesgo sin incluir datos directos de exposicion de trabajadores reales en condiciones de trabajo. Un WRDI alto indica que la investigacion disponible es principalmente tecnica y tiene baja transferibilidad preventiva directa."
      else
        "WRDI measures the proportion of studies that characterise a risk without including direct exposure data from real workers in actual working conditions. A high WRDI indicates that available research is primarily technical and has low direct preventive transferability.",
    '</small>
  </div>
</div>

<!-- Context categories -->',
  if (nrow(context) > 0) paste0('
<div class="section">
  <h2>', if(is_es) "Contexto tecnologico del corpus"
         else "Technological context of the corpus", '</h2>
  <div class="alert-blue">',
    if(is_es)
      "Las siguientes categorias describen el marco tecnologico del corpus (RCS muy alto = dominan el corpus). No son riesgos en si mismos sino el contexto en el que se estudian los riesgos que aparecen en las secciones siguientes."
    else
      "The following categories describe the technological framework of the corpus (very high RCS = they dominate the corpus). They are not risks per se but the context in which the risks shown in the following sections are studied.",
  '</div>
  <table>
    <thead><tr>
      <th>', if(is_es) "Categoria" else "Category", '</th>
      <th>N</th>
      <th>RCS</th>
      <th colspan="2">', if(is_es) "Interpretacion" else "Interpretation", '</th>
    </tr></thead>
    <tbody>', context_rows, '</tbody>
  </table>
</div>') else "",

'<!-- Legend -->
<div class="section">
  <h2>', if(is_es) "Clasificacion de riesgos por prioridad"
         else "Risk classification by priority", '</h2>

  <div class="legend">
    <div class="legend-item">
      <div class="legend-dot" style="background:#D85A30"></div>
      <span><strong>RED</strong>: ',
        if(is_es) "Brecha critica - sobreestudiado sin datos de trabajadores"
        else "Critical gap - over-studied without worker data", '</span>
    </div>
    <div class="legend-item">
      <div class="legend-dot" style="background:#E8A838"></div>
      <span><strong>AMBER</strong>: ',
        if(is_es) "Requiere atencion - evidencia parcial"
        else "Requires attention - partial evidence", '</span>
    </div>
    <div class="legend-item">
      <div class="legend-dot" style="background:#0F6E56"></div>
      <span><strong>GREEN</strong>: ',
        if(is_es) "Cobertura razonable - buena conexion con trabajadores"
        else "Reasonable coverage - good worker data connection", '</span>
    </div>
    <div class="legend-item">
      <div class="legend-dot" style="background:#888"></div>
      <span><strong>GREY</strong>: ',
        if(is_es) "Evidencia insuficiente"
        else "Insufficient evidence", '</span>
    </div>
  </div>

  <!-- RED -->',
  if (n_red > 0) paste0('
  <h3 style="color:#D85A30">&#9888; ',
    if(is_es) "Brechas criticas (ROJO) - Requieren evaluacion presencial inmediata"
    else "Critical gaps (RED) - Require immediate on-site assessment",
  '</h3>
  <div class="alert-red">',
    if(is_es)
      "Estos riesgos estan documentados en la literatura cientifica pero ninguno o casi ningun estudio incluye datos reales de exposicion de trabajadores. Son los candidatos prioritarios para evaluacion de riesgos in situ."
    else
      "These risks are documented in scientific literature but few or no studies include real worker exposure data. They are the top candidates for on-site risk assessment.",
  '</div>
  <table>
    <thead><tr>
      <th>', if(is_es) "Categoria de riesgo" else "Risk category", '</th>
      <th>N</th>
      <th>WRDI</th>
      <th>', if(is_es) "Confianza" else "Confidence", '</th>
      <th>', if(is_es) "Razon" else "Reason", '</th>
    </tr></thead>
    <tbody>', .risk_rows(risks[risks$priority == "RED", ]), '</tbody>
  </table>') else "",

  if (n_amber > 0) paste0('
  <h3 style="color:#E8A838">&#9654; ',
    if(is_es) "Requieren atencion (AMBAR)"
    else "Require attention (AMBER)",
  '</h3>
  <table>
    <thead><tr>
      <th>', if(is_es) "Categoria de riesgo" else "Risk category", '</th>
      <th>N</th>
      <th>WRDI</th>
      <th>', if(is_es) "Confianza" else "Confidence", '</th>
      <th>', if(is_es) "Razon" else "Reason", '</th>
    </tr></thead>
    <tbody>', .risk_rows(risks[risks$priority == "AMBER", ]), '</tbody>
  </table>') else "",

  if (n_green > 0) paste0('
  <h3 style="color:#0F6E56">&#10003; ',
    if(is_es) "Cobertura razonable (VERDE)"
    else "Reasonable coverage (GREEN)",
  '</h3>
  <table>
    <thead><tr>
      <th>', if(is_es) "Categoria de riesgo" else "Risk category", '</th>
      <th>N</th>
      <th>WRDI</th>
      <th>', if(is_es) "Confianza" else "Confidence", '</th>
      <th>', if(is_es) "Razon" else "Reason", '</th>
    </tr></thead>
    <tbody>', .risk_rows(risks[risks$priority == "GREEN", ]), '</tbody>
  </table>') else "",

'</div>

<!-- Methodological section -->
<div class="section">
  <h2>', if(is_es) "Seccion metodologica" else "Methodological section", '</h2>
  <div class="meth-box">

    <h3>', if(is_es) "1. Fuentes y estrategia de busqueda" else "1. Sources and search strategy", '</h3>
    <p style="margin:0.5rem 0;color:#555">', search_txt, '</p>

    <h3>', if(is_es) "2. Criterios de inclusion y exclusion" else "2. Inclusion and exclusion criteria", '</h3>
    <p style="margin:0.5rem 0;color:#555">', incl_txt, '</p>

    <h3>', if(is_es) "3. Proceso de deduplicacion" else "3. Deduplication process", '</h3>
    <ul>
      <li>', if(is_es) paste0("Registros identificados: ", n_load)
             else paste0("Records identified: ", n_load), '</li>
      <li>', if(is_es) paste0("Duplicados eliminados: ", n_dedup,
                               " (matching exacto por DOI + titulo normalizado + fuzzy matching)")
             else paste0("Duplicates removed: ", n_dedup,
                          " (exact DOI match + normalised title + fuzzy matching)"), '</li>
      <li>', if(is_es) paste0("Registros unicos analizados: ", n_final)
             else paste0("Unique records analysed: ", n_final), '</li>
    </ul>

    <h3>', if(is_es) "4. Definicion del WRDI" else "4. WRDI definition", '</h3>
    <ul>
      <li>', if(is_es)
        "Un estudio se considera que tiene datos de exposicion de trabajadores cuando su abstract menciona terminos como: 'worker exposure', 'occupational exposure', 'breathing zone', 'personal sampling', 'field study', 'workplace measurement'."
      else
        "A study is considered to have worker exposure data when its abstract mentions terms such as: 'worker exposure', 'occupational exposure', 'breathing zone', 'personal sampling', 'field study', 'workplace measurement'.", '</li>
      <li>', if(is_es)
        "Limitacion: la deteccion se basa en el abstract, no en el texto completo. Estudios con datos de trabajadores pero sin mencionarlos en el abstract pueden no ser detectados."
      else
        "Limitation: detection is based on the abstract, not the full text. Studies with worker data not mentioned in the abstract may not be detected.", '</li>
    </ul>

    <h3>', if(is_es) "5. Extraccion de categorias de riesgo" else "5. Risk category extraction", '</h3>
    <ul>
      <li>', if(is_es)
        paste0("Diccionario: iso45001_insst v2.0.0 (", n_cats,
               " categorias ancladas en INSST, ISO 45001:2018, NIOSH y EU-OSHA)")
      else
        paste0("Dictionary: iso45001_insst v2.0.0 (", n_cats,
               " categories anchored in INSST, ISO 45001:2018, NIOSH and EU-OSHA)"), '</li>
      <li>', if(is_es)
        "La clasificacion es automatica por busqueda de terminos en titulo, abstract y keywords. Puede producir falsos positivos. Se recomienda validacion manual de una muestra representativa mediante orm_validate()."
      else
        "Classification is automatic via term search in title, abstract and keywords. False positives may occur. Manual validation of a representative sample is recommended using orm_validate().", '</li>
      <li>', if(is_es)
        "El nivel de confianza (Alta/Media/Baja) refleja el numero de estudios que detectaron la categoria, no la calidad metodologica de dichos estudios."
      else
        "The confidence level (High/Medium/Low) reflects the number of studies detecting the category, not the methodological quality of those studies.", '</li>
    </ul>

    <h3>', if(is_es) "6. Limitaciones del analisis" else "6. Analysis limitations", '</h3>
    <ul>
      <li>', if(is_es)
        "Este informe no sustituye la evaluacion de riesgos in situ reglamentaria."
      else
        "This report does not replace the mandatory on-site regulatory risk assessment.", '</li>
      <li>', if(is_es)
        "La normativa aplicable varia por pais y sector. El tecnico debe aplicar la regulacion vigente en su jurisdiccion para cada riesgo identificado."
      else
        "Applicable regulation varies by country and sector. The practitioner must apply current regulation in their jurisdiction for each identified risk.", '</li>
      <li>', if(is_es)
        "Las categorias con confianza Baja (N < 3) tienen evidencia insuficiente para conclusiones solidas."
      else
        "Categories with Low confidence (N < 3) have insufficient evidence for robust conclusions.", '</li>
      <li>', if(is_es)
        "La busqueda bibliografica puede no ser exhaustiva si no se han consultado todas las bases de datos relevantes."
      else
        "The bibliographic search may not be exhaustive if not all relevant databases were consulted.", '</li>
    </ul>
  </div>
</div>

<div class="footer">
  <strong>ORISMA v0.1.0</strong> &mdash;
  Dr. Ra&uacute;l Aguilar-Elena &middot; GPRL &middot; VIU &nbsp;|&nbsp;
  Ana Delgado-Garc&iacute;a &middot; USAL<br>
  ', if(is_es) "Generado" else "Generated", ': ', date_str, ' &nbsp;|&nbsp;
  <a href="https://github.com/Aguilar-Elena/orisma">github.com/Aguilar-Elena/orisma</a><br>
  <em>',
    if(is_es)
      "Herramienta de apoyo bibliometrico. No constituye asesoramiento legal ni sustituye la evaluacion reglamentaria de riesgos."
    else
      "Bibliometric support tool. Does not constitute legal advice and does not replace mandatory regulatory risk assessment.",
  '</em>
</div>

</body>
</html>')

  sheet_path <- file.path(out_dir, "orisma_risk_sheet.html")
  writeLines(html, sheet_path, useBytes = FALSE)

  if (verbose) cli::cli_alert_success(paste0(
    if(is_es) "Ficha de riesgo guardada en: " else "Risk sheet saved to: ",
    sheet_path
  ))

  invisible(sheet_path)
}

#' Compute ORISMA bibliometric indicators and analyses
#'
#' @description
#' `orm_analyse()` takes an extraction matrix and computes:
#'
#' - **WRDI** - Worker-Risk Disconnection Index: the proportion of studies that
#'   characterise a risk without measuring direct worker exposure. A WRDI of 1
#'   means all studies are purely technical (no worker data); 0 means all
#'   studies include direct worker exposure measurement.
#'
#' - **RCS** - Risk Category Saturation Index: relative dominance of each
#'   risk category compared to a uniform-distribution baseline. RCS > 1 means
#'   the category is over-represented; RCS < 1 means it is under-represented.
#'
#' - **MGP** - Material-Gap Profile: ratio of a material's known hazard
#'   potential (from the literature consensus) to its proportional coverage in
#'   the corpus. Detects hazardous materials that are academically under-studied.
#'
#' It also computes co-occurrence matrices, temporal trends, and author networks
#' for visualisation.
#'
#' @param mx An `orisma_matrix` object returned by [orm_extract()].
#' @param material_col Character. Name of the column containing material
#'   information. If `NULL` (default), MGP is skipped with a warning.
#' @param year_col Character. Column name for publication year. Default `"year"`.
#' @param lang Character. `"en"` or `"es"`.
#' @param verbose Logical. Print progress?
#'
#' @return A list (class `orisma_result`) with all indicators and analysis
#'   objects ready for [orm_report()] and visualisation functions.
#'
#' @examples
#' \dontrun{
#' refs    <- orm_load("my_references/")
#' deduped <- orm_dedup(refs)
#' mx      <- orm_extract(deduped)
#' result  <- orm_analyse(mx)
#'
#' # View the three core indicators
#' result$indicators
#'
#' # View WRDI
#' result$WRDI
#' }
#'
#' @export
orm_analyse <- function(mx,
                        material_col = NULL,
                        year_col     = "year",
                        lang         = getOption("orisma.lang", "en"),
                        verbose      = getOption("orisma.verbose", TRUE)) {

  .check_lang(lang)
  if (!inherits(mx, "orisma_matrix")) {
    stop("'mx' must be an orisma_matrix object. Run orm_extract() first.",
         call. = FALSE)
  }

  if (verbose) {
    cli::cli_h1(orm_msg("phase_analyse", lang))
    cli::cli_alert_info(orm_msg("analyse_start", lang))
  }

  refs       <- mx$refs
  bin_mat    <- mx$matrix
  n_records  <- nrow(bin_mat)
  n_cats     <- ncol(bin_mat)
  cat_names  <- colnames(bin_mat)

  # -- 1. WRDI - Worker-Risk Disconnection Index --------------------------------
  # Global WRDI
  n_with_workers    <- sum(refs$has_worker_data, na.rm = TRUE)
  n_without_workers <- n_records - n_with_workers
  WRDI_global       <- round(n_without_workers / n_records, 4)

  # WRDI per risk category
  WRDI_by_cat <- vapply(cat_names, function(cat) {
    col      <- paste0("cat_", cat)
    in_cat   <- refs[[col]] == 1L
    n_cat    <- sum(in_cat, na.rm = TRUE)
    if (n_cat == 0) return(NA_real_)
    n_worker <- sum(refs$has_worker_data[in_cat], na.rm = TRUE)
    round((n_cat - n_worker) / n_cat, 4)
  }, numeric(1))

  if (verbose) {
    cli::cli_alert_success(
      orm_msg("analyse_wrdi", lang,
           value = WRDI_global,
           pct   = round(WRDI_global * 100, 1))
    )
  }

  # -- 2. RCS - Risk Category Saturation Index ----------------------------------
  # Observed frequency per category
  cat_counts  <- colSums(bin_mat)
  total_hits  <- sum(cat_counts)
  uniform_exp <- total_hits / n_cats   # expected count if uniform

  # RCS = observed / expected (values > 1 = over-represented)
  RCS <- round(cat_counts / uniform_exp, 4)

  if (verbose) {
    cli::cli_alert_success(orm_msg("analyse_rcs", lang, n_cats = n_cats))
  }

  # -- 3. MGP - Material-Gap Profile --------------------------------------------
  MGP <- NULL

  if (!is.null(material_col) && material_col %in% names(refs)) {
    materials <- unique(na.omit(refs[[material_col]]))
    n_mats    <- length(materials)

    # Coverage: proportion of records mentioning each material
    coverage <- vapply(materials, function(m) {
      sum(grepl(m, refs[[material_col]], ignore.case = TRUE)) / n_records
    }, numeric(1))

    # Hazard proxy: mean number of risk categories co-occurring with this material
    hazard_proxy <- vapply(materials, function(m) {
      idx <- grepl(m, refs[[material_col]], ignore.case = TRUE)
      if (sum(idx) == 0) return(0)
      mean(refs$n_categories[idx], na.rm = TRUE)
    }, numeric(1))

    # MGP = hazard_proxy / coverage (high = high hazard, low coverage = gap)
    MGP <- data.frame(
      material      = materials,
      coverage      = round(coverage, 4),
      hazard_proxy  = round(hazard_proxy, 3),
      MGP           = round(hazard_proxy / pmax(coverage, 0.001), 3),
      stringsAsFactors = FALSE
    )
    MGP <- MGP[order(-MGP$MGP), ]

    if (verbose) {
      cli::cli_alert_success(orm_msg("analyse_mgp", lang, n_mats = n_mats))
    }
  } else if (!is.null(material_col)) {
    cli::cli_alert_warning(paste0(
      "Column '", material_col, "' not found - MGP skipped."
    ))
  }

  # -- 4. Co-occurrence matrix ---------------------------------------------------
  # How many studies address both category A and category B simultaneously
  cooccur_mat <- t(bin_mat) %*% bin_mat
  diag(cooccur_mat) <- 0L   # remove self-co-occurrences

  # -- 5. Temporal trend --------------------------------------------------------
  temporal <- NULL

  if (year_col %in% names(refs)) {
    temporal <- refs %>%
      dplyr::filter(!is.na(.data[[year_col]])) %>%
      dplyr::mutate(year = as.integer(.data[[year_col]])) %>%
      dplyr::group_by(.data$year) %>%
      dplyr::summarise(
        n_records = dplyr::n(),
        dplyr::across(
          dplyr::starts_with("cat_"),
          ~ sum(.x, na.rm = TRUE),
          .names = "{.col}"
        ),
        .groups = "drop"
      )
  }

  # -- 6. Consolidated indicators table ----------------------------------------
  indicators <- data.frame(
    category    = cat_names,
    label       = mx$categories$label,
    n_records   = as.integer(cat_counts),
    pct_records = round(100 * cat_counts / n_records, 1),
    WRDI        = WRDI_by_cat,
    RCS         = RCS,
    stringsAsFactors = FALSE
  )
  rownames(indicators) <- NULL

  if (verbose) cli::cli_alert_success(orm_msg("analyse_done", lang))

  # -- 7. Assemble result --------------------------------------------------------
  result <- list(
    # Core indicators
    WRDI_global = WRDI_global,
    WRDI_by_cat = WRDI_by_cat,
    RCS         = RCS,
    MGP         = MGP,
    indicators  = indicators,

    # Analysis objects
    cooccur_mat = cooccur_mat,
    temporal    = temporal,

    # Metadata
    n_records          = n_records,
    n_with_workers     = n_with_workers,
    n_without_workers  = n_without_workers,
    n_categories       = n_cats,
    dict               = mx$dict,
    categories         = mx$categories,
    refs               = refs,

    # Pass-through for report
    matrix     = bin_mat
  )

  class(result) <- c("orisma_result", "list")
  attr(result, "orisma_stage")   <- "analysed"
  attr(result, "orisma_lang")    <- lang
  attr(result, "orisma_created") <- Sys.time()
  attr(result, "orisma_version") <- as.character(utils::packageVersion("orisma"))
  attr(result, "dict_name")      <- attr(mx$dict, "dict_name")
  attr(result, "dict_version")   <- attr(mx$dict, "dict_version")

  result
}


#' Print method for orisma_result
#' @param x An object to print.
#' @param ... Further arguments passed to or from other methods.
#' @export
#' @return Invisibly returns the input `orisma_result` object. Called primarily for its console-printing side effect.
print.orisma_result <- function(x, ...) {
  cat("\n-- ORISMA Analysis Result --------------------------------\n")
  cat(" Records analysed:", x$n_records, "\n")
  cat(" Risk categories: ", x$n_categories, "\n\n")

  cat(" WRDI (global):", x$WRDI_global,
      paste0("(", round(x$WRDI_global * 100, 1),
             "% of studies lack direct worker exposure data)\n\n"))

  cat(" Indicators by category:\n")
  df <- x$indicators[order(-x$indicators$n_records), ]
  print(df[, c("label", "n_records", "pct_records", "WRDI", "RCS")],
        row.names = FALSE)

  if (!is.null(x$MGP)) {
    cat("\n MGP - Material-Gap Profile (top 5):\n")
    print(utils::head(x$MGP, 5), row.names = FALSE)
  }

  cat("\nRun orm_report() to generate full reports and visualisations.\n")
  invisible(x)
}

# orm_dict.R
# ORISMA - Superdiccionario completo de riesgos laborales
# 56 categorias - INSST / ISO 45001 / NIOSH / EU-OSHA
# Autor: Dr. Raul Aguilar-Elena - GPRL - VIU
# Todos los terminos en ingles para maxima cobertura bibliografica internacional

#' Built-in risk dictionaries for ORISMA
#'
#' @description
#' ORISMA ships with a comprehensive normative risk dictionary anchored in four
#' internationally recognised taxonomies:
#'
#' - **INSST** Instituto Nacional de Seguridad y Salud en el Trabajo (Spain)
#' - **ISO 45001:2018** International OHS management system standard
#' - **NIOSH** National Institute for Occupational Safety and Health (USA)
#' - **EU-OSHA** European Agency for Safety and Health at Work
#'
#' The dictionary covers 58 risk categories organised in 6 blocks:
#' A) Safety at work (18), B) Industrial hygiene (8), C) Ergonomics (8),
#' D) Psychosociology (11), E) Biological hazards (5),
#' F) Emerging technologies (8).
#'
#' @examples
#' # View available dictionaries
#' orm_dict_list()
#'
#' # Load the default dictionary
#' dict <- orm_dict()
#'
#' # View all categories
#' orm_dict_categories(dict)
#'
#' # Add custom terms to a category
#' dict <- orm_dict_add_terms(dict, "nanomaterials",
#'                            c("metal powder", "powder bed"))
#'
#' @name orisma_dict
NULL


# =============================================================================
# BLOQUE A - SEGURIDAD EN EL TRABAJO (18 categorias - INSST)
# =============================================================================

.dict_a <- list(

  # A01
  fall_height = list(
    label_en  = "Fall from height (fall to different level)",
    label_es  = "Caida de personas a distinto nivel",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "fall from height", "fall from elevation", "elevated work platform",
      "working at height", "roof work", "scaffold fall", "ladder fall",
      "fall arrest", "fall protection", "guardrail", "safety harness",
      "personal fall arrest system", "PFAS", "lifeline", "anchor point",
      "roof access", "elevated surface", "mezzanine fall", "tower fall",
      "climbing fall", "fall prevention", "fall restraint"
    ),
    worker_exposure_terms = c("worker", "operator", "roofer", "climber")
  ),

  # A02
  fall_same_level = list(
    label_en  = "Fall at same level (slip, trip, fall)",
    label_es  = "Caida de personas al mismo nivel",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "slip", "trip", "fall same level", "floor hazard", "wet floor",
      "slippery surface", "uneven floor", "housekeeping slip",
      "walking surface", "floor contamination", "traction loss",
      "stumble", "loss of balance", "flat surface fall",
      "spilled liquid", "floor marking", "anti-slip"
    ),
    worker_exposure_terms = c("worker", "employee", "pedestrian")
  ),

  # A03
  falling_objects_collapse = list(
    label_en  = "Falling objects by collapse or landslide",
    label_es  = "Caida de objetos por desplome o derrumbamiento",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "structural collapse", "wall collapse", "roof collapse",
      "trench collapse", "excavation collapse", "landslide",
      "falling debris", "building collapse", "cave-in",
      "unstable structure", "overhead hazard", "falling material",
      "storage collapse", "racking collapse", "shelf collapse"
    ),
    worker_exposure_terms = c("worker", "construction worker", "miner")
  ),

  # A04
  falling_objects_handling = list(
    label_en  = "Falling objects during handling or manipulation",
    label_es  = "Caida de objetos en manipulacion",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "falling object", "dropped load", "overhead load",
      "material handling fall", "load drop", "rigging failure",
      "crane load", "hoist load", "forklift load",
      "dropped tool", "object projection", "falling part",
      "unsecured load", "hard hat", "head protection"
    ),
    worker_exposure_terms = c("worker", "crane operator", "rigger")
  ),

  # A05
  stepping_objects = list(
    label_en  = "Stepping on objects (puncture wounds)",
    label_es  = "Pisadas sobre objetos",
    block     = "A - Safety",
    taxonomy  = "INSST",
    terms = c(
      "stepping on object", "puncture wound", "nail puncture",
      "sharp object floor", "foot injury", "safety footwear",
      "steel toe boot", "puncture resistant sole",
      "sharps on floor", "debris floor"
    ),
    worker_exposure_terms = c("worker", "construction worker")
  ),

  # A06
  collision_stationary = list(
    label_en  = "Collision with stationary objects",
    label_es  = "Choques contra objetos inmoviles",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "collision stationary object", "impact fixed structure",
      "bumping into", "struck by fixed object", "head bump",
      "low clearance", "protruding object", "blind corner",
      "collision wall", "struck against", "impact injury"
    ),
    worker_exposure_terms = c("worker", "operator")
  ),

  # A07
  collision_moving = list(
    label_en  = "Collision with moving objects",
    label_es  = "Choques contra objetos moviles",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "struck by moving object", "moving machinery contact",
      "impact moving part", "collision moving equipment",
      "swinging load", "pendulum load", "moving conveyor",
      "robot collision", "cobot contact", "moving arm impact",
      "automated guided vehicle", "AGV collision"
    ),
    worker_exposure_terms = c("worker", "operator", "bystander")
  ),

  # A08
  cuts_tools = list(
    label_en  = "Cuts and blows by objects or tools",
    label_es  = "Cortes o golpes por objetos o herramientas",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "laceration", "cut injury", "blade injury", "tool injury",
      "sharp edge", "cutting tool", "hand tool injury",
      "power tool injury", "knife injury", "chisel injury",
      "grinding wheel", "saw injury", "blunt trauma",
      "impact tool", "hammer injury", "hand injury"
    ),
    worker_exposure_terms = c("worker", "machinist", "operator")
  ),

  # A09
  projection_fragments = list(
    label_en  = "Projection of fragments or particles",
    label_es  = "Proyeccion de fragmentos o particulas",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "projectile", "fragment projection", "particle ejection",
      "flying debris", "chip ejection", "metal fragment",
      "grinding particle", "spark projection", "ejected part",
      "pressure release fragment", "bursting fragment",
      "eye injury projection", "face shield", "eye protection"
    ),
    worker_exposure_terms = c("worker", "machinist", "welder")
  ),

  # A10
  entrapment = list(
    label_en  = "Entrapment between objects",
    label_es  = "Atrapamiento por o entre objetos",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "entrapment", "caught between", "pinch point", "nip point",
      "machine guarding", "mechanical hazard", "rotating part",
      "in-running nip", "conveyor entrapment", "press entrapment",
      "machinery entrapment", "power transmission", "gear entrapment",
      "lockout tagout", "LOTO", "energy isolation",
      "machine safeguarding", "safety guard removal"
    ),
    worker_exposure_terms = c("worker", "machine operator", "maintenance")
  ),

  # A11
  overturn = list(
    label_en  = "Entrapment by vehicle overturn",
    label_es  = "Atrapamiento por vuelco de maquinas o vehiculos",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "vehicle overturn", "forklift overturn", "rollover",
      "tractor overturn", "ROPS", "rollover protection",
      "vehicle stability", "load stability", "tipping",
      "crane overturn", "aerial platform overturn"
    ),
    worker_exposure_terms = c("driver", "operator", "worker")
  ),

  # A12
  vehicle_collision = list(
    label_en  = "Run over or collision with vehicles",
    label_es  = "Atropellos o golpes con vehiculos",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "run over", "vehicle collision", "forklift accident",
      "pedestrian vehicle", "traffic accident workplace",
      "road accident", "vehicle pedestrian", "AGV accident",
      "reversing vehicle", "blind spot vehicle",
      "workplace transport", "loading dock accident"
    ),
    worker_exposure_terms = c("worker", "pedestrian", "driver")
  ),

  # A13
  electrical_direct = list(
    label_en  = "Direct electrical contact",
    label_es  = "Contactos electricos directos",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "electrical contact", "electric shock", "electrocution",
      "live conductor", "bare wire", "exposed conductor",
      "high voltage contact", "direct contact electrical",
      "power line contact", "arc flash", "arc blast",
      "electrical burn", "current through body"
    ),
    worker_exposure_terms = c("electrician", "worker", "operator")
  ),

  # A14
  electrical_indirect = list(
    label_en  = "Indirect electrical contact",
    label_es  = "Contactos electricos indirectos",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "indirect electrical contact", "earth fault",
      "residual current", "ground fault", "insulation failure",
      "leakage current", "step potential", "touch potential",
      "RCD", "residual current device", "earth leakage",
      "equipotential bonding", "electrical grounding"
    ),
    worker_exposure_terms = c("worker", "electrician")
  ),

  # A15
  thermal_contact = list(
    label_en  = "Thermal contact (burns and scalds)",
    label_es  = "Contactos termicos",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "thermal burn", "contact burn", "scald", "hot surface",
      "molten metal splash", "steam burn", "hot fluid",
      "cryogenic burn", "cold burn", "frostbite contact",
      "thermal contact injury", "radiant heat burn",
      "welding burn", "laser burn", "fire burn"
    ),
    worker_exposure_terms = c("worker", "welder", "operator")
  ),

  # A16
  explosion = list(
    label_en  = "Explosion",
    label_es  = "Explosiones",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "explosion", "explosive atmosphere", "dust explosion",
      "gas explosion", "ATEX", "deflagration", "detonation",
      "explosive limit", "lower explosive limit", "LEL",
      "upper explosive limit", "UEL", "flammable atmosphere",
      "combustible dust", "metal dust explosion", "titanium dust",
      "aluminium dust", "minimum ignition energy", "MIE",
      "electrostatic ignition", "explosion proof",
      "pressure vessel explosion", "BLEVE", "boilover",
      "reactive chemical", "peroxide explosion"
    ),
    worker_exposure_terms = c("worker", "operator", "plant personnel")
  ),

  # A17
  fire = list(
    label_en  = "Fire",
    label_es  = "Incendios",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "fire", "fire hazard", "fire risk", "flammable material",
      "combustible material", "ignition source", "fire prevention",
      "fire suppression", "fire extinguisher", "sprinkler",
      "fire evacuation", "fire load", "flashpoint",
      "autoignition", "flammability", "fire spread",
      "hot work", "welding fire", "grinding fire"
    ),
    worker_exposure_terms = c("worker", "fire warden", "operator")
  ),

  # A18
  toxic_exposure_safety = list(
    label_en  = "Exposure to irritant or toxic substances (safety context)",
    label_es  = "Exposicion a sustancias irritantes, corrosivas o toxicas",
    block     = "A - Safety",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "chemical burn", "corrosive substance", "acid burn",
      "alkali burn", "skin corrosion", "eye corrosion",
      "irritant skin contact", "acute toxic exposure",
      "hazardous substance spill", "chemical splash",
      "SDS", "safety data sheet", "GHS", "CLP regulation",
      "PPE chemical", "chemical resistant glove",
      "face shield chemical"
    ),
    worker_exposure_terms = c("worker", "laboratory worker", "operator")
  )
)


# =============================================================================
# BLOQUE B - HIGIENE INDUSTRIAL (7 categorias - INSST)
# =============================================================================

.dict_b <- list(

  # B01
  chemical_hazardous = list(
    label_en  = "Exposure to hazardous chemical agents",
    label_es  = "Exposicion a agentes quimicos peligrosos",
    block     = "B - Industrial hygiene",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "chemical exposure", "hazardous chemical", "toxic chemical",
      "occupational exposure limit", "OEL", "threshold limit value", "TLV",
      "time-weighted average", "TWA", "short-term exposure limit", "STEL",
      "permissible exposure limit", "PEL", "biological monitoring",
      "biomonitoring", "urinary metabolite", "blood level",
      "nanoparticle", "nanoparticles", "ultrafine particle", "UFP",
      "nano-aerosol", "airborne particle", "particle emission",
      "aerosol emission", "metal fume", "welding fume",
      "volatile organic compound", "VOC", "organic solvent",
      "isocyanate", "epoxy resin", "flux fume",
      "respirable dust", "inhalable dust", "total dust",
      "PM2.5", "PM10", "PM0.1", "particle concentration",
      "particle number concentration", "particle size distribution",
      "breathing zone", "personal air sampling", "area monitoring",
      "industrial hygiene", "exposure assessment", "air quality",
      "contaminant", "pollutant", "sensitizer", "allergen",
      "chemical contaminant", "workplace air", "ventilation",
      "local exhaust ventilation", "LEV", "general ventilation",
      "fume hood", "enclosure", "emission control",
      "metal powder", "powder handling", "powder exposure",
      "additive manufacturing emission", "3D printing emission",
      "laser sintering emission", "EBM emission", "SLM emission",
      "directed energy deposition fume", "DED emission"
    ),
    worker_exposure_terms = c(
      "worker exposure", "occupational exposure", "personal exposure",
      "breathing zone", "operator exposure", "worker", "employee"
    )
  ),

  # B02
  carcinogens = list(
    label_en  = "Exposure to carcinogens, mutagens and reprotoxic substances (CMR)",
    label_es  = "Exposicion a agentes quimicos cancerigenos o mutagenos",
    block     = "B - Industrial hygiene",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "carcinogen", "carcinogenic", "mutagen", "mutagenic",
      "reprotoxic", "CMR substance", "IARC classification",
      "group 1 carcinogen", "group 2A carcinogen",
      "occupational cancer", "lung cancer occupational",
      "bladder cancer occupational", "mesothelioma",
      "genotoxicity", "genotoxic", "DNA damage",
      "oxidative stress", "cytotoxicity", "cytotoxic",
      "chromium VI", "hexavalent chromium", "nickel carcinogen",
      "cobalt carcinogen", "beryllium carcinogen",
      "cadmium carcinogen", "arsenic carcinogen",
      "formaldehyde carcinogen", "benzene carcinogen",
      "PAH", "polycyclic aromatic hydrocarbon",
      "diesel exhaust", "silica crystalline", "silicosis",
      "wood dust", "hardwood dust", "cancer risk",
      "occupational carcinogen"
    ),
    worker_exposure_terms = c(
      "worker", "occupational exposure", "cancer incidence"
    )
  ),

  # B03
  asbestos = list(
    label_en  = "Exposure to asbestos fibres",
    label_es  = "Exposicion a fibras de amianto",
    block     = "B - Industrial hygiene",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "asbestos", "asbestos fibre", "asbestos fiber",
      "mesothelioma", "asbestosis", "chrysotile",
      "crocidolite", "amosite", "amphibole",
      "asbestos exposure", "asbestos removal",
      "asbestos abatement", "asbestos survey",
      "asbestos containing material", "ACM",
      "fibrous mineral", "man-made mineral fibre", "MMMF",
      "refractory ceramic fibre", "RCF"
    ),
    worker_exposure_terms = c(
      "worker", "asbestos worker", "insulation worker"
    )
  ),

  # B04
  ionising_radiation = list(
    label_en  = "Exposure to ionising radiation",
    label_es  = "Exposicion a radiaciones ionizantes",
    block     = "B - Industrial hygiene",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "ionising radiation", "ionizing radiation",
      "X-ray", "gamma ray", "neutron radiation",
      "alpha particle", "beta particle", "radioactive",
      "radioactivity", "radiological hazard",
      "dose equivalent", "effective dose", "sievert", "Sv",
      "gray", "Gy", "absorbed dose", "dose rate",
      "radiation protection", "radiation shielding",
      "dosimeter", "TLD", "film badge",
      "occupational dose limit", "annual dose limit",
      "NORM", "naturally occurring radioactive material",
      "radon", "radon exposure", "CT scan radiation",
      "fluoroscopy radiation", "radiation worker"
    ),
    worker_exposure_terms = c(
      "radiation worker", "worker", "radiographer"
    )
  ),

  # B05
  non_ionising_radiation = list(
    label_en  = "Exposure to non-ionising radiation",
    label_es  = "Exposicion a radiaciones no ionizantes",
    block     = "B - Industrial hygiene",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "non-ionising radiation", "non-ionizing radiation",
      "laser radiation", "laser safety", "laser hazard",
      "laser exposure", "laser class", "laser protective eyewear",
      "optical radiation", "UV radiation", "ultraviolet radiation",
      "infrared radiation", "visible radiation",
      "electromagnetic field", "EMF", "ELF", "extremely low frequency",
      "radiofrequency", "RF radiation", "microwave radiation",
      "induction heating", "magnetic field exposure",
      "electric field exposure", "ICNIRP guideline",
      "5G radiation", "wireless radiation",
      "arc welding radiation", "UV welding arc",
      "photobiological hazard", "retinal hazard",
      "skin photosensitivity", "photokeratitis"
    ),
    worker_exposure_terms = c(
      "worker", "laser operator", "welder"
    )
  ),

  # B06
  noise = list(
    label_en  = "Noise exposure",
    label_es  = "Exposicion a ruido",
    block     = "B - Industrial hygiene",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "noise", "noise exposure", "occupational noise",
      "noise induced hearing loss", "NIHL",
      "hearing loss", "hearing damage", "audiogram",
      "audiometry", "decibel", "dB", "dBA",
      "sound pressure level", "SPL", "noise level",
      "daily noise exposure", "LEX", "Lex8h",
      "peak sound pressure", "hearing protection",
      "ear plug", "ear muff", "hearing conservation",
      "noise assessment", "noise measurement",
      "noise control", "sound insulation",
      "tinnitus", "hyperacusis", "cochlear damage",
      "noise regulation", "noise standard"
    ),
    worker_exposure_terms = c(
      "worker", "noise-exposed worker", "operator"
    )
  ),

  # B07
  vibration = list(
    label_en  = "Vibration exposure (hand-arm and whole-body)",
    label_es  = "Exposicion a vibraciones",
    block     = "B - Industrial hygiene",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "vibration", "hand-arm vibration", "HAV",
      "whole-body vibration", "WBV",
      "vibration exposure", "vibration measurement",
      "vibration dose value", "VDV",
      "daily vibration exposure", "A8",
      "vibration white finger", "VWF",
      "Raynaud phenomenon occupational",
      "carpal tunnel vibration",
      "hand transmitted vibration",
      "vibrating tool", "pneumatic tool vibration",
      "grinder vibration", "chainsaw vibration",
      "vibration standard", "ISO 5349", "ISO 2631",
      "anti-vibration glove", "vibration damping"
    ),
    worker_exposure_terms = c(
      "worker", "operator", "driver"
    )
  ),

  # B08
  thermal_stress = list(
    label_en  = "Thermal stress (heat and cold)",
    label_es  = "Exposicion a temperaturas ambientales extremas",
    block     = "B - Industrial hygiene",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "heat stress", "heat strain", "thermal stress",
      "heat illness", "heat exhaustion", "heat stroke",
      "hyperthermia", "WBGT", "wet bulb globe temperature",
      "hot environment", "high temperature workplace",
      "radiant heat", "metabolic heat", "heat acclimatisation",
      "cold stress", "cold exposure", "hypothermia",
      "frostbite", "cold environment", "cold work",
      "cryogenic exposure", "freezer work",
      "thermal comfort", "thermal environment",
      "ISO 7933", "ISO 11079", "NIOSH heat criteria",
      "heat related illness", "cold injury"
    ),
    worker_exposure_terms = c(
      "worker", "outdoor worker", "furnace worker"
    )
  )
)


# =============================================================================
# BLOQUE C - ERGONOMIA (8 categorias - INSST)
# =============================================================================

.dict_c <- list(

  # C01
  manual_handling = list(
    label_en  = "Manual handling of loads",
    label_es  = "Carga fisica por manipulacion manual de cargas",
    block     = "C - Ergonomics",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "manual handling", "manual lifting", "load handling",
      "lifting task", "carrying load", "pushing pulling",
      "NIOSH lifting equation", "recommended weight limit", "RWL",
      "cumulative load", "load weight", "awkward posture lifting",
      "musculoskeletal disorder", "MSD", "WMSD",
      "low back pain", "lumbar disorder", "back injury",
      "disc herniation occupational", "manual materials handling",
      "MMH", "patient handling", "patient lifting",
      "mechanical lifting aid", "hoist", "trolley"
    ),
    worker_exposure_terms = c(
      "worker", "nurse", "warehouse worker"
    )
  ),

  # C02
  work_posture = list(
    label_en  = "Physical load by work postures",
    label_es  = "Carga fisica por posturas de trabajo",
    block     = "C - Ergonomics",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "work posture", "awkward posture", "static posture",
      "postural assessment", "RULA", "REBA", "OWAS",
      "neck posture", "shoulder posture", "back posture",
      "trunk flexion", "trunk rotation", "overhead work",
      "prolonged standing", "prolonged sitting",
      "ergonomic assessment", "postural risk",
      "musculoskeletal risk", "upper limb disorder",
      "work-related upper limb disorder", "WRULD",
      "shoulder disorder", "neck disorder",
      "ergonomic intervention"
    ),
    worker_exposure_terms = c(
      "worker", "operator", "assembly worker"
    )
  ),

  # C03
  repetitive_movements = list(
    label_en  = "Repetitive movements (high frequency)",
    label_es  = "Carga fisica por movimientos repetitivos de alta frecuencia",
    block     = "C - Ergonomics",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "repetitive movement", "repetitive motion", "repetitive task",
      "repetitive strain injury", "RSI",
      "cycle time", "repetition rate", "high frequency task",
      "hand repetition level", "HRL",
      "OCRA index", "OCRA checklist", "strain index",
      "carpal tunnel syndrome", "tendinitis", "epicondylitis",
      "De Quervain", "trigger finger", "tenosynovitis",
      "keyboard repetition", "mouse repetition",
      "assembly line repetition"
    ),
    worker_exposure_terms = c(
      "worker", "assembly worker", "keyboard worker"
    )
  ),

  # C04
  mental_load = list(
    label_en  = "Mental workload",
    label_es  = "Carga mental",
    block     = "C - Ergonomics",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "mental workload", "cognitive load", "cognitive demand",
      "attention demand", "concentration demand",
      "information overload", "decision demand",
      "working memory load", "dual task",
      "NASA-TLX", "mental effort",
      "vigilance task", "monitoring task",
      "cognitive fatigue", "mental fatigue",
      "human error", "cognitive ergonomics",
      "situation awareness", "supervisory control"
    ),
    worker_exposure_terms = c(
      "worker", "operator", "air traffic controller"
    )
  ),

  # C05
  thermal_comfort = list(
    label_en  = "Moderate thermal environment (thermal comfort)",
    label_es  = "Ambiente termico moderado",
    block     = "C - Ergonomics",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "thermal comfort", "thermal environment", "PMV", "PPD",
      "predicted mean vote", "perceived temperature",
      "office temperature", "workplace temperature",
      "air temperature", "relative humidity workplace",
      "thermal neutrality", "ISO 7730",
      "HVAC comfort", "air conditioning comfort",
      "draught discomfort", "radiant asymmetry"
    ),
    worker_exposure_terms = c(
      "worker", "office worker", "employee"
    )
  ),

  # C06
  lighting = list(
    label_en  = "Lighting conditions",
    label_es  = "Iluminacion",
    block     = "C - Ergonomics",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "lighting", "illuminance", "lux", "luminance",
      "glare", "visual comfort", "workplace lighting",
      "task lighting", "natural light", "daylight",
      "artificial lighting", "LED lighting",
      "visual fatigue", "eye strain", "visual discomfort",
      "contrast ratio", "reflectance", "light flicker",
      "emergency lighting", "EN 12464"
    ),
    worker_exposure_terms = c(
      "worker", "office worker", "visual task worker"
    )
  ),

  # C07
  indoor_air_quality = list(
    label_en  = "Indoor air quality",
    label_es  = "Calidad del ambiente interior",
    block     = "C - Ergonomics",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "indoor air quality", "IAQ", "sick building syndrome",
      "building related illness", "CO2 indoor",
      "carbon dioxide indoor", "ventilation rate",
      "air change rate", "fresh air supply",
      "indoor pollutant", "formaldehyde indoor",
      "VOC indoor", "mould indoor", "humidity indoor",
      "radon indoor", "TVOC", "total VOC",
      "office air quality", "confined space air"
    ),
    worker_exposure_terms = c(
      "worker", "office worker", "building occupant"
    )
  ),

  # C08
  display_screen = list(
    label_en  = "Display screen equipment (DSE)",
    label_es  = "Riesgos ergonomicos por uso de pantallas de visualizacion",
    block     = "C - Ergonomics",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "display screen", "DSE", "VDU", "visual display unit",
      "computer workstation", "screen work",
      "prolonged screen use", "screen time",
      "digital eye strain", "computer vision syndrome",
      "monitor ergonomics", "keyboard ergonomics",
      "mouse ergonomics", "workstation setup",
      "remote work ergonomics", "home office ergonomics",
      "laptop use", "tablet use", "smartphone use occupational"
    ),
    worker_exposure_terms = c(
      "worker", "office worker", "computer user"
    )
  )
)


# =============================================================================
# BLOQUE D - PSICOSOCIOLOGIA (11 categorias - INSST)
# =============================================================================

.dict_d <- list(

  # D01
  work_content = list(
    label_en  = "Work content (repetitiveness, low content)",
    label_es  = "Contenido de trabajo",
    block     = "D - Psychosociology",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "work content", "job content", "task variety",
      "skill utilisation", "monotonous work", "repetitive work",
      "meaningful work", "job enrichment", "task identity",
      "boredom", "underutilisation", "understimulation",
      "job simplification", "Taylorism", "deskilling"
    ),
    worker_exposure_terms = c("worker", "employee")
  ),

  # D02
  workload_pace = list(
    label_en  = "Work overload and work pace",
    label_es  = "Carga de trabajo y ritmo de trabajo",
    block     = "D - Psychosociology",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "work overload", "workload", "work pace", "time pressure",
      "deadline pressure", "work intensity", "job demands",
      "quantitative demands", "qualitative demands",
      "effort reward imbalance", "ERI",
      "demand control model", "Karasek model",
      "job strain", "high strain job",
      "work underload", "work-life balance",
      "overtime", "excessive working hours"
    ),
    worker_exposure_terms = c("worker", "employee")
  ),

  # D03
  working_time = list(
    label_en  = "Working time (shift work, night work, irregular hours)",
    label_es  = "Tiempo de trabajo",
    block     = "D - Psychosociology",
    taxonomy  = "INSST/ISO45001/NIOSH",
    terms = c(
      "shift work", "night work", "night shift",
      "rotating shift", "extended shift", "12-hour shift",
      "irregular schedule", "unpredictable schedule",
      "on-call work", "flexible working time",
      "circadian rhythm", "sleep disorder occupational",
      "fatigue shift work", "alertness impairment",
      "work schedule", "rest period", "recovery time"
    ),
    worker_exposure_terms = c("shift worker", "worker", "nurse")
  ),

  # D04
  participation_control = list(
    label_en  = "Participation and control (job autonomy)",
    label_es  = "Participacion y control",
    block     = "D - Psychosociology",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "job control", "job autonomy", "decision latitude",
      "participation", "worker participation",
      "empowerment", "self-determination work",
      "lack of control", "micromanagement",
      "algorithmic management", "surveillance work",
      "monitoring workers", "electronic monitoring",
      "performance monitoring"
    ),
    worker_exposure_terms = c("worker", "employee")
  ),

  # D05
  role_performance = list(
    label_en  = "Role performance (ambiguity, conflict)",
    label_es  = "Desempeno de rol",
    block     = "D - Psychosociology",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "role ambiguity", "role conflict", "role overload",
      "role underload", "job description clarity",
      "unclear expectations", "conflicting demands",
      "boundary spanning", "role stress",
      "boundary role", "responsibility clarity"
    ),
    worker_exposure_terms = c("worker", "manager", "employee")
  ),

  # D06
  professional_development = list(
    label_en  = "Professional development and career",
    label_es  = "Desarrollo profesional",
    block     = "D - Psychosociology",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "career development", "professional development",
      "job insecurity", "employment insecurity",
      "precarious work", "training opportunity",
      "promotion", "recognition", "reward",
      "lack of recognition", "underpromotion",
      "overqualification", "glass ceiling",
      "deskilling", "obsolescence"
    ),
    worker_exposure_terms = c("worker", "employee")
  ),

  # D07
  interpersonal_relations = list(
    label_en  = "Interpersonal relations and social support",
    label_es  = "Relaciones interpersonales y apoyo social",
    block     = "D - Psychosociology",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "social support", "interpersonal relations",
      "team cohesion", "collegial support",
      "supervisor support", "management support",
      "isolation", "social isolation", "remote work isolation",
      "loneliness work", "poor communication",
      "workplace conflict", "interpersonal conflict",
      "organisational climate", "trust"
    ),
    worker_exposure_terms = c("worker", "employee")
  ),

  # D08
  team_equipment = list(
    label_en  = "Work teams and equipment exposure",
    label_es  = "Equipos de trabajo y exposicion a otros riesgos",
    block     = "D - Psychosociology",
    taxonomy  = "INSST",
    terms = c(
      "team work", "teamwork", "team dynamics",
      "group work", "collaborative work",
      "tool design", "equipment design",
      "human machine interface", "HMI",
      "control room design", "technology acceptance",
      "digital tool adoption", "automation impact"
    ),
    worker_exposure_terms = c("worker", "team member")
  ),

  # D09
  harassment = list(
    label_en  = "Harassment (sexual, discriminatory)",
    label_es  = "Acoso",
    block     = "D - Psychosociology",
    taxonomy  = "INSST/ISO45001/EU-OSHA",
    terms = c(
      "harassment", "sexual harassment", "discriminatory harassment",
      "workplace harassment", "moral harassment",
      "psychological harassment", "mobbing", "bullying",
      "cyber bullying work", "discriminacion",
      "racial harassment", "gender harassment",
      "age discrimination", "disability discrimination",
      "LGBTQ+ discrimination", "anti-harassment policy"
    ),
    worker_exposure_terms = c("worker", "victim", "employee")
  ),

  # D10
  external_violence = list(
    label_en  = "External violence",
    label_es  = "Violencia externa",
    block     = "D - Psychosociology",
    taxonomy  = "INSST/EU-OSHA",
    terms = c(
      "workplace violence", "external violence",
      "aggression", "physical assault work",
      "verbal aggression", "threat work",
      "customer violence", "patient aggression",
      "robbery workplace", "anti-social behaviour",
      "lone worker risk", "lone working",
      "conflict management", "de-escalation"
    ),
    worker_exposure_terms = c("worker", "healthcare worker", "retail worker")
  ),

  # D11
  emotional_demands = list(
    label_en  = "Emotional demands (client contact)",
    label_es  = "Demandas emocionales",
    block     = "D - Psychosociology",
    taxonomy  = "INSST/ISO45001",
    terms = c(
      "emotional demand", "emotional labour", "emotional work",
      "surface acting", "deep acting", "emotional exhaustion",
      "burnout", "compassion fatigue", "secondary trauma",
      "vicarious trauma", "emotional dissonance",
      "client contact stress", "patient contact stress",
      "customer service stress", "empathy fatigue"
    ),
    worker_exposure_terms = c(
      "worker", "healthcare worker", "social worker"
    )
  )
)


# =============================================================================
# BLOQUE E - RIESGOS BIOLOGICOS (5 categorias - EU-OSHA/NIOSH)
# =============================================================================

.dict_e <- list(

  # E01
  virus = list(
    label_en  = "Biological hazard - Viruses",
    label_es  = "Riesgo biologico - Virus",
    block     = "E - Biological",
    taxonomy  = "EU-OSHA/NIOSH/ISO45001",
    terms = c(
      "virus", "viral", "viral infection", "occupational viral",
      "airborne virus", "droplet transmission",
      "influenza occupational", "COVID-19 occupational",
      "SARS-CoV-2", "hepatitis B occupational",
      "hepatitis C occupational", "HIV occupational",
      "needlestick virus", "bloodborne virus",
      "respiratory virus", "norovirus occupational",
      "arbovirus", "zoonotic virus", "rabies occupational",
      "Ebola occupational", "hantavirus",
      "biological risk group", "containment level",
      "biosafety level", "BSL", "antiviral PPE"
    ),
    worker_exposure_terms = c(
      "healthcare worker", "laboratory worker", "worker"
    )
  ),

  # E02
  bacteria = list(
    label_en  = "Biological hazard - Bacteria",
    label_es  = "Riesgo biologico - Bacterias",
    block     = "E - Biological",
    taxonomy  = "EU-OSHA/NIOSH",
    terms = c(
      "bacteria", "bacterial", "bacterial infection occupational",
      "Legionella", "legionellosis", "Leptospira", "leptospirosis",
      "Brucella", "brucellosis", "Mycobacterium tuberculosis",
      "tuberculosis occupational", "anthrax occupational",
      "Clostridium", "MRSA", "methicillin resistant",
      "antibiotic resistant bacteria", "nosocomial infection",
      "healthcare associated infection", "wound infection",
      "zoonotic bacteria", "Q fever", "Coxiella",
      "biological aerosol", "bioaerosol bacteria",
      "endotoxin", "gram negative bacteria exposure"
    ),
    worker_exposure_terms = c(
      "healthcare worker", "agricultural worker", "worker"
    )
  ),

  # E03
  fungi = list(
    label_en  = "Biological hazard - Fungi and moulds",
    label_es  = "Riesgo biologico - Hongos",
    block     = "E - Biological",
    taxonomy  = "EU-OSHA/NIOSH",
    terms = c(
      "fungi", "fungal", "mould", "mold", "fungal spore",
      "mycotoxin", "Aspergillus occupational",
      "Aspergillosis", "Candida occupational",
      "dermatophyte", "ringworm occupational",
      "occupational asthma fungi", "farmer lung",
      "hypersensitivity pneumonitis fungi",
      "organic dust toxic syndrome", "ODTS",
      "bioaerosol fungi", "biofilm fungi",
      "compost fungi", "agricultural fungi",
      "construction mould"
    ),
    worker_exposure_terms = c(
      "farmer", "agricultural worker", "construction worker", "worker"
    )
  ),

  # E04
  parasites = list(
    label_en  = "Biological hazard - Parasites",
    label_es  = "Riesgo biologico - Parasitos",
    block     = "E - Biological",
    taxonomy  = "EU-OSHA/NIOSH",
    terms = c(
      "parasite", "parasitic", "parasitic infection occupational",
      "Toxoplasma", "toxoplasmosis occupational",
      "Echinococcus", "hydatid disease",
      "Leishmania", "leishmaniasis occupational",
      "malaria occupational", "tick-borne disease",
      "Lyme disease", "Borrelia occupational",
      "scabies occupational", "worm infection",
      "helminth occupational", "protozoa occupational",
      "vector-borne disease", "insect vector",
      "arthropod vector"
    ),
    worker_exposure_terms = c(
      "outdoor worker", "agricultural worker", "worker"
    )
  ),

  # E05
  prions = list(
    label_en  = "Biological hazard - Prions",
    label_es  = "Riesgo biologico - Priones",
    block     = "E - Biological",
    taxonomy  = "EU-OSHA",
    terms = c(
      "prion", "prion disease", "transmissible spongiform encephalopathy",
      "TSE", "Creutzfeldt-Jakob disease", "CJD",
      "variant CJD", "vCJD", "BSE occupational",
      "bovine spongiform encephalopathy",
      "scrapie", "occupational prion",
      "surgical instrument prion", "decontamination prion",
      "autopsy prion risk", "pathology prion",
      "neurodegenerative prion"
    ),
    worker_exposure_terms = c(
      "healthcare worker", "pathologist", "veterinary worker"
    )
  )
)


# =============================================================================
# BLOQUE F - TECNOLOGIAS EMERGENTES (8 categorias - EU-OSHA 2024-2026)
# =============================================================================

.dict_f <- list(

  # F01
  additive_manufacturing = list(
    label_en  = "Additive manufacturing and 3D printing",
    label_es  = "Fabricacion aditiva e impresion 3D",
    block     = "F - Emerging technologies",
    taxonomy  = "EU-OSHA/NIOSH/ORISMA",
    terms = c(
      "additive manufacturing", "3D printing", "AM metal",
      "metal additive manufacturing", "powder bed fusion", "PBF",
      "selective laser melting", "SLM", "selective laser sintering", "SLS",
      "electron beam melting", "EBM",
      "laser powder bed fusion", "LPBF",
      "directed energy deposition", "DED",
      "laser metal deposition", "LMD",
      "wire arc additive manufacturing", "WAAM",
      "binder jetting", "material jetting",
      "fused deposition modelling", "FDM", "FFF",
      "stereolithography", "SLA", "digital light processing", "DLP",
      "metal powder", "titanium powder", "aluminium powder",
      "nickel alloy powder", "cobalt chromium powder",
      "stainless steel powder", "tool steel powder",
      "build chamber", "inert atmosphere",
      "post-processing AM", "depowdering",
      "support removal AM", "heat treatment AM",
      "AM occupational health", "AM worker exposure",
      "3D printing emission", "AM emission",
      "layer manufacturing", "rapid prototyping",
      "Industry 4.0 manufacturing"
    ),
    worker_exposure_terms = c(
      "AM worker", "operator", "3D printing operator",
      "worker", "technician"
    )
  ),

  # F02
  nanomaterials = list(
    label_en  = "Nanomaterials and nanotechnology",
    label_es  = "Nanomateriales y nanotecnologia",
    block     = "F - Emerging technologies",
    taxonomy  = "EU-OSHA/NIOSH",
    terms = c(
      "nanomaterial", "nanotechnology", "engineered nanomaterial", "ENM",
      "nanoparticle", "nano-object", "nanofibre", "nanofiber",
      "nanotube", "carbon nanotube", "CNT",
      "fullerene", "graphene", "quantum dot",
      "nano-silver", "nano-gold", "nano-copper",
      "nano-titanium dioxide", "nano-zinc oxide",
      "nano-silica", "nano-alumina",
      "NOAA", "nano-object aggregate agglomerate",
      "nano-risk", "nano-safety", "nano-toxicology",
      "nano-regulation", "REACH nanomaterial"
    ),
    worker_exposure_terms = c(
      "worker", "nano-worker", "laboratory worker"
    )
  ),

  # F03
  robotics_automation = list(
    label_en  = "Robotics, cobots and automation",
    label_es  = "Robotica, cobots y automatizacion",
    block     = "F - Emerging technologies",
    taxonomy  = "EU-OSHA/ISO45001",
    terms = c(
      "robot", "robotics", "industrial robot",
      "collaborative robot", "cobot",
      "human robot collaboration", "HRC",
      "human robot interaction", "HRI",
      "robot safety", "robot hazard",
      "robot workspace", "robot cell",
      "autonomous robot", "mobile robot",
      "autonomous guided vehicle", "AGV",
      "autonomous mobile robot", "AMR",
      "drone", "unmanned aerial vehicle", "UAV",
      "automation", "automated system",
      "cyber-physical system", "CPS",
      "industry 4.0", "smart factory",
      "robot programming", "teach pendant",
      "collaborative workspace", "ISO 10218",
      "ISO/TS 15066", "robot speed force limiting"
    ),
    worker_exposure_terms = c(
      "worker", "robot operator", "programmer"
    )
  ),

  # F04
  artificial_intelligence = list(
    label_en  = "Artificial intelligence and autonomous systems",
    label_es  = "Inteligencia artificial y sistemas autonomos",
    block     = "F - Emerging technologies",
    taxonomy  = "EU-OSHA/NIOSH",
    terms = c(
      "artificial intelligence", "AI", "machine learning",
      "deep learning", "neural network",
      "algorithm management", "algorithmic management",
      "AI monitoring", "surveillance AI",
      "facial recognition workplace", "biometric monitoring",
      "automated decision making", "AI decision",
      "digital labour platform", "gig work",
      "platform economy", "app-based work",
      "agentic AI", "autonomous AI system",
      "large language model", "LLM workplace",
      "generative AI work", "ChatGPT workplace",
      "AI ethics work", "AI safety",
      "digital twin", "predictive maintenance AI",
      "AI fatigue", "technostress"
    ),
    worker_exposure_terms = c(
      "worker", "platform worker", "gig worker"
    )
  ),

  # F05
  exoskeletons = list(
    label_en  = "Exoskeletons and wearable technology",
    label_es  = "Exoesqueletos y tecnologia vestible",
    block     = "F - Emerging technologies",
    taxonomy  = "EU-OSHA/NIOSH",
    terms = c(
      "exoskeleton", "exosuit", "powered exoskeleton",
      "passive exoskeleton", "active exoskeleton",
      "upper limb exoskeleton", "lower limb exoskeleton",
      "back support exoskeleton", "wearable robot",
      "wearable technology", "wearable sensor",
      "smart PPE", "connected PPE",
      "IoT wearable", "health monitoring wearable",
      "fatigue monitoring wearable",
      "biometric wearable", "heart rate monitor work",
      "posture sensor", "motion capture occupational"
    ),
    worker_exposure_terms = c(
      "worker", "construction worker", "assembly worker"
    )
  ),

  # F06
  synthetic_biology = list(
    label_en  = "Synthetic biology and biotechnology",
    label_es  = "Biologia sintetica y biotecnologia",
    block     = "F - Emerging technologies",
    taxonomy  = "EU-OSHA/NIOSH",
    terms = c(
      "synthetic biology", "gene editing", "CRISPR",
      "genetically modified organism", "GMO",
      "gene therapy", "cell therapy",
      "bioengineering", "bioreactor",
      "fermentation occupational", "microbial exposure lab",
      "biosafety synthetic biology",
      "containment synthetic biology",
      "dual use research", "gain of function",
      "biosecurity", "bioterrorism prevention",
      "advanced therapy medicinal product", "ATMP"
    ),
    worker_exposure_terms = c(
      "laboratory worker", "researcher", "biotech worker"
    )
  ),

  # F07
  quantum_computing = list(
    label_en  = "Quantum technologies",
    label_es  = "Tecnologias cuanticas",
    block     = "F - Emerging technologies",
    taxonomy  = "EU-OSHA/ORISMA",
    terms = c(
      "quantum computing", "quantum technology",
      "quantum sensor", "quantum communication",
      "cryogenic quantum", "dilution refrigerator",
      "superconducting qubit", "cryogenic hazard",
      "liquid helium", "liquid nitrogen cryogenic",
      "electromagnetic shielding quantum",
      "laser quantum", "photon quantum",
      "quantum laboratory safety",
      "qubit manipulation"
    ),
    worker_exposure_terms = c(
      "researcher", "laboratory worker", "quantum engineer"
    )
  ),

  # F08
  green_circular_economy = list(
    label_en  = "Green jobs and circular economy",
    label_es  = "Empleos verdes y economia circular",
    block     = "F - Emerging technologies",
    taxonomy  = "EU-OSHA/NIOSH",
    terms = c(
      "green job", "green economy", "circular economy",
      "renewable energy worker", "solar panel installation",
      "wind turbine worker", "offshore wind",
      "recycling worker", "waste management worker",
      "e-waste", "electronic waste recycling",
      "battery recycling", "lithium battery",
      "electric vehicle maintenance",
      "energy storage worker", "bioenergy worker",
      "sustainable manufacturing", "eco-design",
      "remanufacturing", "upcycling",
      "critical raw material", "rare earth extraction",
      "cobalt mining", "lithium mining"
    ),
    worker_exposure_terms = c(
      "green worker", "renewable energy worker", "worker"
    )
  )
)


# =============================================================================
# MASTER DICTIONARY - combina los 6 bloques
# =============================================================================

.dict_iso45001_insst <- c(.dict_a, .dict_b, .dict_c, .dict_d, .dict_e, .dict_f)


# =============================================================================
# PUBLIC API
# =============================================================================

#' List available built-in dictionaries
#' @return Invisibly prints available dictionaries. Called for side effects.
#' @return A data frame with columns: \code{key}, \code{block}, \code{label}, \code{taxonomy}, \code{n_terms}.
#' @return Invisibly prints available dictionaries. Called for side effects.
#' @return A data frame with columns: \code{key}, \code{block}, \code{label}, \code{taxonomy}, \code{n_terms}.
#' @export
#' @details This function takes no arguments.
#' @return A character vector with the names of the built-in dictionaries available in ORISMA.
orm_dict_list <- function() {
  cat("Available built-in dictionaries:\n")
  cat("  'iso45001_insst' - Full 56-category dictionary (default)\n")
  cat("    Blocks: A) Safety (18) | B) Hygiene (7) | C) Ergonomics (8)\n")
  cat("            D) Psychosociology (11) | E) Biological (5) | F) Emerging (8)\n")
  cat("    Anchored in: INSST + ISO 45001 + NIOSH + EU-OSHA\n")
  invisible(c("iso45001_insst"))
}


#' Load a risk dictionary
#'
#' @param name Character. Dictionary name. Default `"iso45001_insst"`.
#' @return A named list (class `orisma_dict`).
#' @export
orm_dict <- function(name = "iso45001_insst") {
  d <- switch(name,
    iso45001_insst = .dict_iso45001_insst,
    stop(paste0("Unknown dictionary: '", name,
                "'. Use orm_dict_list() to see options."),
         call. = FALSE)
  )
  class(d) <- c("orisma_dict", "list")
  attr(d, "dict_name")    <- name
  attr(d, "dict_version") <- "2.0.0"
  attr(d, "dict_n_cats")  <- length(d)
  attr(d, "dict_created") <- Sys.time()
  d
}


#' List risk categories in a dictionary
#' @param dict An `orisma_dict` object.
#' @param lang Character. `"en"` or `"es"`.
#' @export
#' @return A data frame containing the available risk categories, including category keys, labels, blocks and dictionary metadata.
orm_dict_categories <- function(dict,
                                lang = getOption("orisma.lang", "en")) {
  cats <- lapply(names(dict), function(k) {
    entry <- dict[[k]]
    data.frame(
      key      = k,
      block    = entry$block,
      label    = if (lang == "es") entry$label_es else entry$label_en,
      taxonomy = entry$taxonomy,
      n_terms  = length(entry$terms),
      stringsAsFactors = FALSE
    )
  })
  dplyr::bind_rows(cats)
}


#' Add terms to an existing dictionary category
#' @param dict An `orisma_dict` object.
#' @param category Character. Category key.
#' @param terms Character vector. New terms to add.
#' @return Updated `orisma_dict`.
#' @export
orm_dict_add_terms <- function(dict, category, terms) {
  if (!category %in% names(dict)) {
    stop(paste0("Category '", category,
                "' not found. Use orm_dict_categories() to list categories."),
         call. = FALSE)
  }
  dict[[category]]$terms <- unique(c(dict[[category]]$terms, terms))
  dict
}


#' Add a new risk category to a dictionary
#' @param dict An `orisma_dict` object.
#' @param key Character. Short identifier (no spaces).
#' @param label_en Character. Category name in English.
#' @param label_es Character. Category name in Spanish.
#' @param terms Character vector. Search terms.
#' @param worker_exposure_terms Character vector. Worker exposure indicators.
#' @param taxonomy Character. Source taxonomy label.
#' @param block Character. Block label (e.g. "A - Safety").
#' @return Updated `orisma_dict`.
#' @export
orm_dict_add_category <- function(dict, key, label_en, label_es, terms,
                                   worker_exposure_terms = character(0),
                                   taxonomy = "user",
                                   block = "G - Custom") {
  if (key %in% names(dict)) {
    warning(paste0("Category '", key,
                   "' already exists. Use orm_dict_add_terms() instead."))
    return(dict)
  }
  dict[[key]] <- list(
    label_en              = label_en,
    label_es              = label_es,
    block                 = block,
    taxonomy              = taxonomy,
    terms                 = terms,
    worker_exposure_terms = worker_exposure_terms
  )
  dict
}


#' Print method for orisma_dict
#' @param x An `orisma_dict` object.
#' @param ... Further arguments (ignored).
#' @return Invisibly returns `x`.
#' @export
print.orisma_dict <- function(x, ...) {
  cat("\n-- ORISMA Risk Dictionary --\n")
  cat(" Name:       ", attr(x, "dict_name"), "\n")
  cat(" Version:    ", attr(x, "dict_version"), "\n")
  cat(" Categories: ", length(x), "\n\n")
  df <- orm_dict_categories(x)
  blocks <- unique(df$block)
  for (b in blocks) {
    sub <- df[df$block == b, ]
    cat(" ", b, "(", nrow(sub), "categories )\n")
  }
  cat("\nUse orm_dict_categories() for full detail.\n")
  invisible(x)
}
# orisma dict v2.0.0

library(testthat)
library(orisma)

# ── Helpers: minimal synthetic data ──────────────────────────────────────────

.make_refs <- function(n = 10) {
  refs <- data.frame(
    record_id     = paste0("ORM", sprintf("%05d", seq_len(n))),
    source_db     = rep("Test DB", n),
    source_file   = rep("test.ris", n),
    title = c(
      "Nanoparticle exposure in laser powder bed fusion",
      "Occupational health risks of metal additive manufacturing",
      "Laser radiation hazards in LPBF processes",
      "Worker exposure to ultrafine particles in 3D printing",
      "Dust explosion risk in titanium powder handling",
      "Ergonomic assessment of additive manufacturing operators",
      "Inert gas asphyxiation risk in EBM chambers",
      "Metal fume emissions in directed energy deposition",
      "Review of occupational hazards in metal AM",
      "Electrical safety in additive manufacturing equipment"
    ),
    abstract = c(
      "This study measured nanoparticle concentrations during metal AM operations.",
      "Worker exposure to occupational hazards was assessed across 15 facilities.",
      "Laser radiation levels were characterised during LPBF printing.",
      "Breathing zone measurements of ultrafine particles were performed.",
      "Titanium dust explosion risk was evaluated using minimum ignition energy.",
      "RULA assessment was conducted for 20 AM operators.",
      "Oxygen depletion in confined build chambers was monitored.",
      "Metal fume and VOC emissions were measured during DED operations.",
      "A systematic review of occupational exposure in metal printing.",
      "Lockout tagout procedures for electrical systems in AM were reviewed."
    ),
    keywords    = rep("additive manufacturing; occupational health", n),
    year        = rep(2022L, n),
    doi         = paste0("10.1234/test.", seq_len(n)),
    authors     = paste0("Author", seq_len(n), ", A"),
    journal     = rep("Test Journal", n),
    volume      = rep("1", n),
    issue       = rep("1", n),
    pages       = rep("1-10", n),
    document_type = rep("Article", n),
    stringsAsFactors = FALSE
  )
  class(refs) <- c("orisma_refs", "tbl_df", "tbl", "data.frame")
  attr(refs, "orisma_stage")   <- "loaded"
  attr(refs, "orisma_lang")    <- "en"
  attr(refs, "orisma_created") <- Sys.time()
  attr(refs, "n_sources")      <- 1L
  dplyr::as_tibble(refs)
}


# ── Tests: orm_dict ───────────────────────────────────────────────────────────

test_that("orm_dict returns correct class", {
  d <- orm_dict()
  expect_s3_class(d, "orisma_dict")
})

test_that("orm_dict has expected categories", {
  d <- orm_dict()
  expect_true("nanoparticles" %in% names(d))
  expect_true("laser_radiation" %in% names(d))
  expect_true("explosion" %in% names(d))
})

test_that("orm_dict_categories returns a data frame", {
  d  <- orm_dict()
  df <- orm_dict_categories(d)
  expect_s3_class(df, "data.frame")
  expect_true(all(c("category", "label", "n_terms") %in% names(df)))
})

test_that("orm_dict_add_terms adds new terms", {
  d  <- orm_dict()
  d2 <- orm_dict_add_terms(d, "nanoparticles", c("nano-cloud", "nano-fog"))
  expect_true("nano-cloud" %in% d2$nanoparticles$terms)
  expect_true("nano-fog"   %in% d2$nanoparticles$terms)
})

test_that("orm_dict_add_category creates new entry", {
  d  <- orm_dict()
  d2 <- orm_dict_add_category(
    d, "vibration",
    label_en = "Vibration exposure",
    label_es = "Exposición a vibraciones",
    terms    = c("hand-arm vibration", "whole-body vibration", "HAV")
  )
  expect_true("vibration" %in% names(d2))
  expect_equal(d2$vibration$label_en, "Vibration exposure")
})


# ── Tests: orm_extract ────────────────────────────────────────────────────────

test_that("orm_extract returns orisma_matrix", {
  refs <- .make_refs()
  mx   <- orm_extract(refs, verbose = FALSE)
  expect_s3_class(mx, "orisma_matrix")
})

test_that("orm_extract produces binary matrix with correct dimensions", {
  refs  <- .make_refs(10)
  dict  <- orm_dict()
  mx    <- orm_extract(refs, dict = dict, verbose = FALSE)
  n_cats <- length(dict)
  expect_equal(nrow(mx$matrix), 10)
  expect_equal(ncol(mx$matrix), n_cats)
  expect_true(all(mx$matrix %in% c(0L, 1L)))
})

test_that("orm_extract detects nanoparticle category correctly", {
  refs <- .make_refs()
  mx   <- orm_extract(refs, verbose = FALSE)
  # First record is about nanoparticles — should be flagged
  expect_equal(mx$matrix[1, "nanoparticles"], 1L)
})

test_that("orm_extract detects worker exposure data", {
  refs <- .make_refs()
  mx   <- orm_extract(refs, verbose = FALSE)
  # Records 2 and 4 explicitly mention "worker exposure" / "breathing zone"
  expect_true(any(mx$refs$has_worker_data))
})


# ── Tests: orm_analyse ────────────────────────────────────────────────────────

test_that("orm_analyse returns orisma_result", {
  refs   <- .make_refs()
  mx     <- orm_extract(refs, verbose = FALSE)
  result <- orm_analyse(mx, verbose = FALSE)
  expect_s3_class(result, "orisma_result")
})

test_that("WRDI is between 0 and 1", {
  refs   <- .make_refs()
  mx     <- orm_extract(refs, verbose = FALSE)
  result <- orm_analyse(mx, verbose = FALSE)
  expect_gte(result$WRDI_global, 0)
  expect_lte(result$WRDI_global, 1)
})

test_that("RCS values are positive", {
  refs   <- .make_refs()
  mx     <- orm_extract(refs, verbose = FALSE)
  result <- orm_analyse(mx, verbose = FALSE)
  expect_true(all(result$RCS[!is.na(result$RCS)] > 0))
})

test_that("indicators table has expected columns", {
  refs   <- .make_refs()
  mx     <- orm_extract(refs, verbose = FALSE)
  result <- orm_analyse(mx, verbose = FALSE)
  expect_true(all(c("category", "n_records", "WRDI", "RCS") %in%
                    names(result$indicators)))
})

test_that("cooccurrence matrix is symmetric", {
  refs   <- .make_refs()
  mx     <- orm_extract(refs, verbose = FALSE)
  result <- orm_analyse(mx, verbose = FALSE)
  expect_equal(result$cooccur_mat, t(result$cooccur_mat))
})


# ── Tests: orm_run ────────────────────────────────────────────────────────────

test_that("orm_run completes without error", {
  refs   <- .make_refs()
  result <- orm_run(refs, verbose = FALSE)
  expect_s3_class(result, "orisma_result")
})

test_that("orm_run result has expected structure", {
  refs   <- .make_refs()
  result <- orm_run(refs, verbose = FALSE)
  expect_true(!is.null(result$WRDI_global))
  expect_true(!is.null(result$indicators))
  expect_true(!is.null(result$cooccur_mat))
})


# ── Tests: orm_report ─────────────────────────────────────────────────────────

test_that("orm_report creates output directory and files", {
  refs    <- .make_refs()
  result  <- orm_run(refs, verbose = FALSE)
  tmp_dir <- tempfile("orisma_test_")

  orm_report(result,
             out_dir = tmp_dir,
             formats = c("csv", "certificate"),
             verbose = FALSE)

  expect_true(dir.exists(tmp_dir))
  expect_true(file.exists(file.path(tmp_dir, "orisma_indicators.csv")))
  expect_true(file.exists(file.path(tmp_dir, "analysis.orisma")))

  unlink(tmp_dir, recursive = TRUE)
})

test_that("reproducibility certificate contains required fields", {
  refs    <- .make_refs()
  result  <- orm_run(refs, verbose = FALSE)
  tmp_dir <- tempfile("orisma_cert_")

  orm_report(result,
             out_dir = tmp_dir,
             formats = "certificate",
             verbose = FALSE)

  cert <- jsonlite::read_json(file.path(tmp_dir, "analysis.orisma"))
  expect_true("orisma_version"  %in% names(cert))
  expect_true("WRDI_global"     %in% names(cert))
  expect_true("digest_matrix"   %in% names(cert))

  unlink(tmp_dir, recursive = TRUE)
})

#' Bridge Article Detection and Priority Ranking
#'
#' @description
#' `orm_bridge()` identifies **bridge articles** - studies that connect
#' technical science with real occupational prevention. These are the
#' highest-value articles for an occupational health practitioner because
#' they have already done the translation from laboratory to workplace.
#'
#' A bridge article simultaneously mentions:
#' 1. **Technology/process** (what was studied)
#' 2. **Hazardous agent** (what risk was characterised)
#' 3. **Workers** (real people in real workplaces)
#' 4. **Exposure measurement** (quantitative data)
#' 5. **Prevention/recommendation** (actionable output)
#'
#' Articles meeting 4 or 5 criteria are classified as **strong bridges**.
#' Articles meeting 3 criteria (must include workers + measurement) are
#' **partial bridges**. Others are technical studies.
#'
#' @param mx An `orisma_matrix` object, ideally after running [orm_ass()].
#' @param text_col Character. Text field to analyse. Default `"abstract"`.
#' @param lang Character. `"en"` or `"es"`.
#' @param verbose Logical.
#'
#' @return The `orisma_matrix` object with added columns:
#'   `bridge_score` (0-5), `bridge_type` (Strong/Partial/Technical),
#'   `bridge_criteria` (which criteria were met).
#'
#' @export
orm_bridge <- function(mx,
                        text_col = "abstract",
                        lang     = getOption("orisma.lang", "en"),
                        verbose  = getOption("orisma.verbose", TRUE)) {

  .check_lang(lang)
  if (!inherits(mx, "orisma_matrix"))
    stop("'mx' must be an orisma_matrix object from orm_extract().", call. = FALSE)

  is_es <- lang == "es"
  refs  <- mx$refs

  if (!text_col %in% names(refs) ||
      mean(is.na(refs[[text_col]]) | refs[[text_col]] == "") > 0.5) {
    text_col <- "title"
  }

  text_all <- tolower(as.character(refs[[text_col]]))
  text_all[is.na(text_all)] <- ""

  # ── Bridge criteria patterns ─────────────────────────────────────────────────

  # Criterion 1: Technology / process
  pat_c1 <- paste0("\\b(",
    "additive manufacturing|3D print|powder bed|laser melting|sintering|",
    "directed energy|electron beam|fused deposition|FDM|SLM|EBM|LPBF|DED|",
    "welding|machining|grinding|painting|coating|plating|casting|forging|",
    "nanotechnology|nanomaterial|semiconductor|pharmaceutical|",
    "construction|demolition|renovation|agriculture|mining|",
    "healthcare|hospital|laboratory|office work|transport|logistics",
    ")\\b")

  # Criterion 2: Hazardous agent
  pat_c2 <- paste0("\\b(",
    "nanoparticle|ultrafine|aerosol|fume|metal fume|dust|respirable|",
    "chemical|solvent|VOC|isocyanate|carcinogen|mutagen|toxic|",
    "noise|vibration|radiation|laser|UV|infrared|EMF|",
    "biological|pathogen|bacteria|virus|fungi|allergen|",
    "ergonomic|manual handling|repetitive|posture|",
    "psychosocial|stress|burnout|harassment",
    ")\\b")

  # Criterion 3: Workers (MANDATORY for bridge)
  pat_c3 <- paste0("\\b(",
    "worker|workers|employee|employees|operator|operators|",
    "occupational|workforce|personnel|staff|",
    "worker exposure|worker health|worker safety|",
    "exposed worker|occupational cohort|study population",
    ")\\b")

  # Criterion 4: Exposure measurement (MANDATORY for bridge)
  pat_c4 <- paste0("\\b(",
    "measured|measurement|monitored|sampling|concentration|",
    "breathing zone|personal air|area sampling|biological monitoring|",
    "TWA|STEL|OEL|TLV|mg.m|ug.m|ppm|ppb|dB|fiber|",
    "dose|dosimetry|quantif|detected|assessed exposure",
    ")\\b")

  # Criterion 5: Prevention / recommendation
  pat_c5 <- paste0("\\b(",
    "prevention|preventive|control measure|engineering control|",
    "administrative control|PPE|personal protective|ventilation|",
    "enclosure|substitution|elimination|protective measure|",
    "safety measure|intervention|recommendation|guideline|",
    "limit value|exposure limit|risk management|mitigation|",
    "reduce exposure|lower exposure|protective action|",
    "health surveillance|medical monitoring|awareness",
    ")\\b")

  # ── Score each record ─────────────────────────────────────────────────────────
  results <- lapply(seq_along(text_all), function(i) {
    txt <- text_all[i]
    if (nchar(txt) < 30) {
      return(list(score = 0L, criteria = character(0),
                  type = if(is_es) "Tecnico" else "Technical"))
    }

    c1 <- grepl(pat_c1, txt, perl = TRUE)
    c2 <- grepl(pat_c2, txt, perl = TRUE)
    c3 <- grepl(pat_c3, txt, perl = TRUE)
    c4 <- grepl(pat_c4, txt, perl = TRUE)
    c5 <- grepl(pat_c5, txt, perl = TRUE)

    met      <- c(c1, c2, c3, c4, c5)
    score    <- sum(met)
    criteria <- c("technology","agent","workers","measurement","prevention")[met]

    # Bridge requires BOTH workers (c3) AND measurement (c4)
    is_bridge <- c3 && c4

    type <- if (score >= 4 && is_bridge)
      if(is_es) "Puente fuerte" else "Strong bridge"
    else if (score >= 3 && is_bridge)
      if(is_es) "Puente parcial" else "Partial bridge"
    else
      if(is_es) "Tecnico" else "Technical"

    list(score = as.integer(score), criteria = criteria, type = type)
  })

  mx$refs$bridge_score    <- vapply(results, `[[`, integer(1), "score")
  mx$refs$bridge_type     <- vapply(results, `[[`, character(1), "type")
  mx$refs$bridge_criteria <- vapply(results, function(x)
    paste(x$criteria, collapse = "+"), character(1))

  if (verbose) {
    n_strong  <- sum(mx$refs$bridge_type %in% c("Strong bridge", "Puente fuerte"))
    n_partial <- sum(mx$refs$bridge_type %in% c("Partial bridge", "Puente parcial"))
    n_tech    <- nrow(mx$refs) - n_strong - n_partial

    cli::cli_alert_success(paste0(
      if(is_es) "Articulos puente detectados:" else "Bridge articles detected:"
    ))
    cli::cli_alert_info(paste0(
      "  ", if(is_es) "Puentes fuertes: " else "Strong bridges: ", n_strong,
      " (", round(100*n_strong/nrow(mx$refs), 1), "%)"
    ))
    cli::cli_alert_info(paste0(
      "  ", if(is_es) "Puentes parciales: " else "Partial bridges: ", n_partial,
      " (", round(100*n_partial/nrow(mx$refs), 1), "%)"
    ))
    cli::cli_alert_info(paste0(
      "  ", if(is_es) "Tecnicos: " else "Technical: ", n_tech,
      " (", round(100*n_tech/nrow(mx$refs), 1), "%)"
    ))
  }

  mx
}


#' Generate priority reading ranking
#'
#' @description
#' `orm_ranking()` produces a **priority reading list** for occupational
#' health practitioners, ranking articles by their combined relevance score
#' (bridge score + ASS score + number of risk categories detected).
#'
#' Articles at the top of the list are those most likely to contain
#' actionable preventive information and should be read first in full.
#'
#' @param mx An `orisma_matrix` object after running [orm_bridge()] and
#'   optionally [orm_ass()].
#' @param top_n Integer. Number of top articles to return. Default `20`.
#' @param out_dir Character or NULL. Directory to save the ranking CSV.
#' @param lang Character. `"en"` or `"es"`.
#'
#' @return A data frame with the top_n priority articles.
#' @export
orm_ranking <- function(mx,
                         top_n   = 20L,
                         out_dir = NULL,
                         lang    = getOption("orisma.lang", "en")) {

  .check_lang(lang)
  if (!inherits(mx, "orisma_matrix"))
    stop("'mx' must be an orisma_matrix object.", call. = FALSE)
  if (!"bridge_score" %in% names(mx$refs))
    stop("Run orm_bridge() first.", call. = FALSE)

  is_es <- lang == "es"
  refs  <- mx$refs

  # Combined priority score
  ass_score    <- if ("ass_score" %in% names(refs)) refs$ass_score else 0L
  bridge_score <- refs$bridge_score
  n_cats       <- rowSums(mx$matrix)

  # Weighted score: bridge (0-5) x2 + ASS (0-5) x1.5 + categories x0.5
  priority_score <- (bridge_score * 2) + (ass_score * 1.5) + (n_cats * 0.5)

  ranking <- refs %>%
    dplyr::mutate(
      priority_score = priority_score,
      ass_score_col  = ass_score,
      n_categories   = n_cats
    ) %>%
    dplyr::arrange(dplyr::desc(.data$priority_score)) %>%
    dplyr::slice_head(n = top_n) %>%
    dplyr::select(
      dplyr::any_of(c(
        "record_id", "title", "authors", "year", "doi", "source_db",
        "bridge_type", "bridge_score", "bridge_criteria",
        "ass_score_col", "n_categories", "priority_score"
      ))
    )

  names(ranking)[names(ranking) == "ass_score_col"] <- "ass_score"

  if (!is.null(out_dir)) {
    if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)
    readr::write_csv(ranking, file.path(out_dir, "orisma_priority_ranking.csv"))
  }

  class(ranking) <- c("orisma_ranking", "data.frame")
  ranking
}


#' Print method for orisma_ranking
#' @param x An `orisma_ranking` object.
#' @param ... Further arguments (ignored).
#' @return Invisibly returns `x`.
#' @export
print.orisma_ranking <- function(x, ...) {
  cat("\n-- ORISMA Priority Reading Ranking --\n")
  cat(" Top", nrow(x), "articles by preventive relevance\n\n")
  for (i in seq_len(min(nrow(x), 20))) {
    cat(sprintf(" %2d. [%s] score=%.1f  ASS=%s  cats=%s\n",
                i,
                x$bridge_type[i],
                x$priority_score[i],
                if("ass_score" %in% names(x)) x$ass_score[i] else "?",
                if("n_categories" %in% names(x)) x$n_categories[i] else "?"))
    cat(sprintf("     %s\n", substr(x$title[i], 1, 80)))
    if (!is.na(x$doi[i]) && x$doi[i] != "")
      cat(sprintf("     DOI: %s\n", x$doi[i]))
    cat("\n")
  }
  invisible(x)
}

#' Sample bibliographic records for ORISMA
#'
#' Twenty bibliographic records on occupational health in metal
#' additive manufacturing (2015-2026) from Web of Science and Scopus,
#' pre-processed with ORISMA to illustrate the full pipeline.
#'
#' @format A data frame with 20 rows and 9 variables:
#' \describe{
#'   \item{record_id}{Character. Unique record identifier.}
#'   \item{title}{Character. Article title.}
#'   \item{abstract}{Character. Abstract (max 800 characters).}
#'   \item{year}{Integer. Publication year.}
#'   \item{doi}{Character. Digital Object Identifier.}
#'   \item{source_db}{Character. Source database.}
#'   \item{bridge_type}{Character. Bridge classification.}
#'   \item{bridge_score}{Integer. Bridge score (0-5).}
#'   \item{ass_score}{Integer. Abstract Sufficiency Score (0-5).}
#' }
#' @source Web of Science and Scopus (2015-2026).
#' @usage data(orisma_sample)
#' @keywords datasets
"orisma_sample"

#' Run the complete ORISMA pipeline in one call
#'
#' @description
#' `orm_run()` is the **single-function entry point** for a complete ORISMA
#' analysis. It runs all pipeline steps automatically:
#'
#' 1. Deduplication (3-step: DOI + title + fuzzy)
#' 2. Risk category extraction (dictionary-based)
#' 3. Bibliometric analysis (WRDI, RCS, MGP indicators)
#' 4. Automatic dimension detection (normative blocks)
#' 5. Abstract Sufficiency Score (ASS, 0-5)
#' 6. Bridge article detection and priority ranking
#'
#' ## Minimal usage (3 lines)
#'
#' ```r
#' library(orisma)
#' refs   <- orm_load("my_references/")
#' result <- orm_run(refs)
#' orm_report(result, lang = "es")
#' ```
#'
#' All intermediate objects are stored in the result for downstream use
#' with [orm_report()], [orm_risk_sheet()], [orm_ranking()], and
#' [orm_extraction_matrix()].
#'
#' @param refs An `orisma_refs` object from [orm_load()].
#' @param dict An `orisma_dict` object. Default: [orm_dict()].
#' @param autodim_method Character. `"blocks"` (default) or `"text"`.
#' @param material_col Character or NULL. Column for MGP. Default NULL.
#' @param year_col Character. Year column. Default `"year"`.
#' @param fuzzy_threshold Numeric. Deduplication threshold. Default `0.90`.
#' @param fields Character vector. Text fields for extraction. Default
#'   `c("title", "abstract", "keywords")`.
#' @param lang Character. `"en"` or `"es"`.
#' @param verbose Logical. Default `TRUE`.
#' @param save_report Logical. Auto-call [orm_report()]? Default `FALSE`.
#' @param topic Character. Domain or technology being analysed (e.g.
#'   'Noise in construction', 'Metal AM'). Used in plot subtitles and
#'   report headers. If NULL, neutral generic text is used.
#' @param out_dir Character. Output directory if `save_report = TRUE`.
#'
#' @return An `orisma_result` object containing all indicators, analyses,
#'   dimensions (`result$dims`), extraction matrix (`result$mx`),
#'   ASS scores and bridge classification (in `result$mx$refs`),
#'   and priority ranking (`result$ranking`).
#'
#' @export
orm_run <- function(refs,
                    dict            = orm_dict(),
                    topic           = NULL,
                    autodim_method  = "blocks",
                    material_col    = NULL,
                    year_col        = "year",
                    fuzzy_threshold = 0.90,
                    fields          = c("title", "abstract", "keywords"),
                    lang            = getOption("orisma.lang", "en"),
                    verbose         = getOption("orisma.verbose", TRUE),
                    save_report     = FALSE,
                    out_dir         = getOption("orisma.out_dir", "orisma_output")) {

  .check_lang(lang)
  if (!inherits(refs, "orisma_refs"))
    stop("'refs' must be an orisma_refs object. Run orm_load() first.", call. = FALSE)

  t_start <- proc.time()

  # ── Step 1: Deduplication ───────────────────────────────────────────────────
  deduped <- orm_dedup(refs,
                       fuzzy_threshold = fuzzy_threshold,
                       lang = lang, verbose = verbose)

  # ── Step 2: Risk extraction ─────────────────────────────────────────────────
  mx <- orm_extract(deduped, dict = dict, fields = fields,
                    lang = lang, verbose = verbose)

  # ── Step 3: Bibliometric analysis ───────────────────────────────────────────
  result <- orm_analyse(mx, material_col = material_col,
                        year_col = year_col, lang = lang, verbose = verbose)

  # ── Step 4: Automatic dimension detection ───────────────────────────────────
  if (verbose) cli::cli_h2(
    if (lang == "es") "Deteccion automatica de dimensiones"
    else "Automatic dimension detection"
  )
  dims <- tryCatch(
    orm_autodim(mx, method = autodim_method, lang = lang, verbose = verbose),
    error = function(e) {
      cli::cli_alert_warning(paste0("orm_autodim: ", e$message))
      NULL
    }
  )

  # ── Step 5: Abstract Sufficiency Score ──────────────────────────────────────
  if (verbose) cli::cli_h2(
    if (lang == "es") "Abstract Sufficiency Score (ASS)"
    else "Abstract Sufficiency Score (ASS)"
  )
  mx <- tryCatch(
    orm_ass(mx, lang = lang, verbose = verbose),
    error = function(e) {
      cli::cli_alert_warning(paste0("orm_ass: ", e$message))
      mx
    }
  )

  # ── Step 6: Bridge article detection ────────────────────────────────────────
  if (verbose) cli::cli_h2(
    if (lang == "es") "Deteccion de articulos puente"
    else "Bridge article detection"
  )
  mx <- tryCatch(
    orm_bridge(mx, lang = lang, verbose = verbose),
    error = function(e) {
      cli::cli_alert_warning(paste0("orm_bridge: ", e$message))
      mx
    }
  )

  # ── Step 7: Priority ranking ─────────────────────────────────────────────────
  ranking <- tryCatch(
    orm_ranking(mx, top_n = 20L, lang = lang),
    error = function(e) {
      cli::cli_alert_warning(paste0("orm_ranking: ", e$message))
      NULL
    }
  )

  # ── Assemble result ──────────────────────────────────────────────────────────
  result$dims    <- dims
  result$topic   <- topic
  result$mx      <- mx
  result$ranking <- ranking

  # ── Pipeline summary ─────────────────────────────────────────────────────────
  elapsed <- round((proc.time() - t_start)["elapsed"], 1)

  attr(result, "pipeline_summary") <- list(
    n_loaded    = nrow(refs),
    n_deduped   = attr(deduped, "dedup_n_unique"),
    n_removed   = attr(deduped, "dedup_n_total"),
    n_analysed  = result$n_records,
    elapsed_sec = elapsed,
    lang        = lang,
    dict_name   = attr(dict, "dict_name")
  )

  if (verbose) {
    cat("\n")
    cli::cli_rule(
      left  = "ORISMA pipeline complete",
      right = paste0(elapsed, " sec")
    )

    # Summary stats
    n_strong <- if (!is.null(result$mx) && "bridge_type" %in% names(result$mx$refs))
      sum(result$mx$refs$bridge_type %in% c("Strong bridge", "Puente fuerte"))
    else 0L

    ass_mean <- if (!is.null(result$mx) && "ass_score" %in% names(result$mx$refs))
      round(mean(result$mx$refs$ass_score, na.rm = TRUE), 2)
    else NA

    cat(
      " Records loaded:     ", nrow(refs), "\n",
      "Records analysed:   ", result$n_records,
      paste0("(", attr(deduped, "dedup_n_total"), " duplicates removed)\n"),
      "WRDI (global):      ", result$WRDI_global, "\n",
      "Dimensions detected:", if(!is.null(dims)) dims$n_dims else 0, "\n",
      "Strong bridges:     ", n_strong, "\n",
      "Mean ASS score:     ", if(!is.na(ass_mean)) ass_mean else "N/A", "/5\n\n",
      sep = ""
    )
    cat("Run orm_report(result) for full report\n")
    cat("Run orm_risk_sheet(result) for practitioner risk sheet\n\n")
  }

  if (save_report) {
    orm_report(result, lang = lang, out_dir = out_dir, topic = topic, verbose = verbose)
  }

  result
}

# utils-global.R
# Resolves R CMD check NOTEs about global variables and missing imports
# This file is NOT exported - internal use only

# ── Suppress R CMD check NOTEs for ggplot2 aes() variables ───────────────────
# These are column names used inside aes() that R CMD check cannot resolve
# statically. Declaring them here suppresses the "no visible binding" NOTE.

utils::globalVariables(c(
  # ggplot2 aes variables used in orm_report.R plot functions
  "WRDI", "RCS", "label", "n_records", "pct_records",
  "year", "n", "category",
  # dplyr .data pronoun and dot
  ".data", ".",
  # Internal helper
  ".msg"
))

# ── importFrom declarations for functions used via :: in code ─────────────────
# These tell R CMD check that we are intentionally importing these functions

#' @importFrom dplyr mutate filter select group_by ungroup summarise
#'   bind_rows bind_cols across starts_with all_of as_tibble n
#'   row_number case_when
#' @importFrom tidyr pivot_longer
#' @importFrom readr read_csv write_csv
#' @importFrom stringr str_extract str_escape
#' @importFrom ggplot2 ggplot aes geom_col geom_point geom_line geom_vline
#'   geom_hline geom_text scale_fill_gradient scale_colour_manual
#'   scale_colour_gradient scale_size_continuous annotate labs
#'   theme_minimal theme element_blank guides guide_legend ggsave
#' @importFrom ggrepel geom_text_repel
#' @importFrom grDevices colorRampPalette dev.off png
#' @importFrom stats reorder na.omit
#' @importFrom utils packageVersion head
#' @importFrom tools file_ext
#' @importFrom jsonlite write_json read_json
#' @importFrom digest digest
#' @importFrom glue glue
#' @importFrom cli cli_h1 cli_h2 cli_alert_success cli_alert_warning
#'   cli_alert_danger cli_alert_info cli_progress_bar cli_progress_update
#'   cli_progress_done cli_rule
NULL

#' @importFrom magrittr %>%
NULL
# R/utils-msg.R
# Sistema de mensajes bilingue - funcion interna permanente del paquete
# Esto reemplaza el sistema de inst/i18n/messages.R

.orisma_messages <- list(
  en = list(
    phase_load     = "Phase 1 - Loading and ingestion",
    phase_dedup    = "Phase 2 - Deduplication",
    phase_audit    = "Phase 3 - Quality audit",
    phase_extract  = "Phase 4 - Risk category extraction",
    phase_analyse  = "Phase 5 - Bibliometric analysis",
    phase_report   = "Phase 6 - Report generation",
    load_start     = "Reading reference files from: {path}",
    load_files     = "Files found: {n} ({formats})",
    load_done      = "Records loaded: {n_total} from {n_sources} source(s)",
    load_no_files  = "No supported files found in: {path}",
    load_fmt_hint  = "Supported formats: .ris, .bib, .csv",
    dedup_start    = "Starting deduplication pipeline",
    dedup_doi      = "Step 1/3 - Exact DOI match: {n_removed} duplicates removed",
    dedup_title    = "Step 2/3 - Normalised title match: {n_removed} duplicates removed",
    dedup_fuzzy    = "Step 3/3 - Fuzzy match: {n_removed} duplicates removed",
    dedup_review   = "{n_ambiguous} ambiguous case(s) require manual review",
    dedup_done     = "Deduplication complete - {n_unique} unique records retained ({n_total_removed} removed)",
    audit_start    = "Running quality audit",
    audit_retract  = "Retracted records flagged: {n}",
    audit_pubpeer  = "PubPeer alerts flagged: {n}",
    audit_ok       = "Records passing audit: {n}",
    extract_start  = "Extracting risk categories using dictionary: {dict_name} (v{dict_version})",
    extract_done   = "Extraction complete - {n_records} records x {n_cats} risk categories",
    extract_empty  = "Warning: {n} records matched no risk category",
    analyse_start  = "Computing bibliometric indicators",
    analyse_wrdi   = "WRDI computed: {value} ({pct}% of studies lack direct worker exposure data)",
    analyse_rcs    = "RCS computed for {n_cats} risk categories",
    analyse_mgp    = "MGP computed for {n_mats} materials",
    analyse_done   = "Analysis complete",
    report_start   = "Generating reports in: {out_dir}",
    report_html    = "Executive report (HTML): {file}",
    report_pdf     = "Academic report (PDF): {file}",
    report_cert    = "Reproducibility certificate: {file}",
    report_done    = "All outputs saved to: {out_dir}",
    err_no_doi     = "Column 'doi' not found - DOI-based deduplication skipped",
    err_no_title   = "Column 'title' not found - cannot proceed",
    err_dict_miss  = "Dictionary file not found: {path}",
    err_lang       = "Language '{lang}' not supported. Use 'en' or 'es'."
  ),
  es = list(
    phase_load     = "Fase 1 - Carga e ingestion",
    phase_dedup    = "Fase 2 - Deduplicacion",
    phase_audit    = "Fase 3 - Auditoria de calidad",
    phase_extract  = "Fase 4 - Extraccion de categorias de riesgo",
    phase_analyse  = "Fase 5 - Analisis bibliometrico",
    phase_report   = "Fase 6 - Generacion de informes",
    load_start     = "Leyendo ficheros desde: {path}",
    load_files     = "Ficheros encontrados: {n} ({formats})",
    load_done      = "Registros cargados: {n_total} de {n_sources} fuente(s)",
    load_no_files  = "No se encontraron ficheros en: {path}",
    load_fmt_hint  = "Formatos admitidos: .ris, .bib, .csv",
    dedup_start    = "Iniciando pipeline de deduplicacion",
    dedup_doi      = "Paso 1/3 - DOI exacto: {n_removed} duplicados eliminados",
    dedup_title    = "Paso 2/3 - Titulo normalizado: {n_removed} duplicados eliminados",
    dedup_fuzzy    = "Paso 3/3 - Fuzzy matching: {n_removed} duplicados eliminados",
    dedup_review   = "{n_ambiguous} caso(s) ambiguo(s) requieren revision manual",
    dedup_done     = "Deduplicacion completada - {n_unique} registros unicos ({n_total_removed} eliminados)",
    audit_start    = "Ejecutando auditoria de calidad",
    audit_retract  = "Registros retractados: {n}",
    audit_pubpeer  = "Alertas PubPeer: {n}",
    audit_ok       = "Registros que superan la auditoria: {n}",
    extract_start  = "Extrayendo categorias con diccionario: {dict_name} (v{dict_version})",
    extract_done   = "Extraccion completada - {n_records} registros x {n_cats} categorias",
    extract_empty  = "Aviso: {n} registros sin categoria detectada",
    analyse_start  = "Calculando indicadores bibliometricos",
    analyse_wrdi   = "WRDI calculado: {value} ({pct}% sin datos de trabajadores)",
    analyse_rcs    = "RCS calculado para {n_cats} categorias",
    analyse_mgp    = "MGP calculado para {n_mats} materiales",
    analyse_done   = "Analisis completado",
    report_start   = "Generando informes en: {out_dir}",
    report_html    = "Informe ejecutivo (HTML): {file}",
    report_pdf     = "Informe academico (PDF): {file}",
    report_cert    = "Certificado de reproducibilidad: {file}",
    report_done    = "Todos los outputs guardados en: {out_dir}",
    err_no_doi     = "Columna 'doi' no encontrada - deduplicacion por DOI omitida",
    err_no_title   = "Columna 'title' no encontrada - no es posible continuar",
    err_dict_miss  = "Fichero de diccionario no encontrado: {path}",
    err_lang       = "Idioma '{lang}' no admitido. Use 'en' o 'es'."
  )
)

#' Internal message retrieval function
#' @noRd
orm_msg <- function(key, lang = getOption("orisma.lang", "en"), ...) {
  msgs <- .orisma_messages[[lang]]
  if (is.null(msgs)) {
    lang <- "en"
    msgs <- .orisma_messages[["en"]]
  }
  template <- msgs[[key]]
  if (is.null(template)) return(paste0("[missing message: ", key, "]"))
  # Use simple string interpolation instead of glue to avoid environment issues
  result <- template
  args <- list(...)
  for (nm in names(args)) {
    result <- gsub(paste0("\\{", nm, "\\}"), as.character(args[[nm]]), result)
  }
  result
}

#' @importFrom stats setNames
#' @importFrom dplyr everything
NULL

#' Automatic deduplication of bibliographic records
#'
#' @description
#' `orm_dedup()` removes duplicate records using a **three-step progressive
#' pipeline**:
#'
#' 1. **Exact DOI match** — most reliable signal; decisive for records with DOIs.
#' 2. **Normalised title match** — removes punctuation, accents, case, and extra
#'    spaces before comparing; catches the same article listed with minor
#'    typographic differences across databases.
#' 3. **Fuzzy match** — compares title + year + first author using
#'    Optimal String Alignment distance; catches near-identical records that
#'    escape exact matching (e.g. different journal abbreviations, truncated
#'    author lists).
#'
#' Only records that remain ambiguous after all three steps are flagged for
#' optional manual review. These are saved to `dedup_log.csv`.
#'
#' @param refs An `orisma_refs` object returned by [orm_load()].
#' @param fuzzy_threshold Numeric (0–1). Similarity threshold for fuzzy
#'   matching. Default `0.90` (90% similarity = duplicate). Increase for
#'   stricter matching, decrease for more aggressive deduplication.
#' @param lang Character. `"en"` or `"es"`. Overrides `orisma.lang` option.
#' @param verbose Logical. Print progress? Default `TRUE`.
#' @param save_log Logical. Save `dedup_log.csv` to working directory?
#'   Default `TRUE`.
#'
#' @return An `orisma_refs` tibble with duplicates removed. Attributes record
#'   deduplication statistics for inclusion in the PRISMA log.
#'
#' @examples
#' \dontrun{
#' refs    <- orm_load("my_references/")
#' deduped <- orm_dedup(refs)
#'
#' # More aggressive fuzzy matching
#' deduped <- orm_dedup(refs, fuzzy_threshold = 0.85)
#'
#' # Spanish messages, no log file
#' deduped <- orm_dedup(refs, lang = "es", save_log = FALSE)
#' }
#'
#' @export
orm_dedup <- function(refs,
                      fuzzy_threshold = 0.90,
                      lang    = getOption("orisma.lang", "en"),
                      verbose = getOption("orisma.verbose", TRUE),
                      save_log = TRUE) {

  .check_lang(lang)
  if (!inherits(refs, "orisma_refs")) {
    stop("'refs' must be an orisma_refs object. Run orm_load() first.",
         call. = FALSE)
  }

  if (verbose) cli::cli_h1(orm_msg("phase_dedup", lang))
  if (verbose) cli::cli_alert_info(orm_msg("dedup_start", lang))

  n_start  <- nrow(refs)
  log_rows <- list()

  # ── Step 1: Exact DOI match ─────────────────────────────────────────────────
  n_before <- nrow(refs)

  if (!"doi" %in% names(refs) || all(is.na(refs$doi))) {
    if (verbose) cli::cli_alert_warning(orm_msg("err_no_doi", lang))
    n_doi_removed <- 0L
  } else {
    refs <- refs %>%
      dplyr::mutate(
        doi_clean = tolower(trimws(.data$doi))
      ) %>%
      dplyr::group_by(.data$doi_clean) %>%
      dplyr::filter(is.na(.data$doi_clean) | .data$doi_clean == "" |
                      dplyr::row_number() == 1L) %>%
      dplyr::ungroup() %>%
      dplyr::select(-"doi_clean")

    n_doi_removed <- n_before - nrow(refs)
    log_rows[[1]] <- data.frame(
      step = "DOI exact match", removed = n_doi_removed, method = "exact"
    )
  }

  if (verbose) {
    cli::cli_alert_success(orm_msg("dedup_doi", lang, n_removed = n_doi_removed))
  }

  # ── Step 2: Normalised title match ──────────────────────────────────────────
  n_before <- nrow(refs)

  if (!"title" %in% names(refs) || all(is.na(refs$title))) {
    cli::cli_alert_danger(orm_msg("err_no_title", lang))
    stop("Cannot deduplicate without a title column.", call. = FALSE)
  }

  refs <- refs %>%
    dplyr::mutate(
      title_norm = .normalise_title(.data$title)
    ) %>%
    dplyr::group_by(.data$title_norm) %>%
    dplyr::filter(dplyr::row_number() == 1L) %>%
    dplyr::ungroup() %>%
    dplyr::select(-"title_norm")

  n_title_removed <- n_before - nrow(refs)
  log_rows[[2]] <- data.frame(
    step = "Normalised title match", removed = n_title_removed, method = "exact"
  )

  if (verbose) {
    cli::cli_alert_success(orm_msg("dedup_title", lang,
                                n_removed = n_title_removed))
  }

  # ── Step 3: Fuzzy match (title + year + first author) ───────────────────────
  n_before <- nrow(refs)

  # Build composite key for fuzzy matching
  refs <- refs %>%
    dplyr::mutate(
      first_author = .extract_first_author(.data$authors),
      fuzzy_key    = paste(.normalise_title(.data$title),
                           as.character(.data$year),
                           .data$first_author,
                           sep = "||")
    )

  # Compute pairwise distances (using stringdist OSA)
  keys     <- refs$fuzzy_key
  dist_mat <- stringdist::stringdistmatrix(keys, keys, method = "osa")

  # Normalise to [0, 1]: similarity = 1 - dist / max(nchar(a), nchar(b))
  n_chars  <- nchar(keys)
  max_mat  <- outer(n_chars, n_chars, pmax)
  sim_mat  <- 1 - dist_mat / pmax(max_mat, 1)

  # Flag pairs above threshold (lower triangle only)
  is_dup   <- which(lower.tri(sim_mat) & sim_mat >= fuzzy_threshold,
                    arr.ind = TRUE)

  ambiguous_pairs <- nrow(is_dup)

  # Keep first occurrence in each duplicate group
  to_remove <- unique(is_dup[, 1])   # rows identified as duplicates
  if (length(to_remove) > 0) {
    refs <- refs[-to_remove, ]
  }

  refs <- refs %>% dplyr::select(-"first_author", -"fuzzy_key")

  n_fuzzy_removed <- n_before - nrow(refs)
  log_rows[[3]] <- data.frame(
    step = "Fuzzy match (title+year+author)", removed = n_fuzzy_removed,
    method = "fuzzy"
  )

  if (verbose) {
    cli::cli_alert_success(orm_msg("dedup_fuzzy", lang,
                                n_removed = n_fuzzy_removed))
  }

  # ── Save log ─────────────────────────────────────────────────────────────────
  dedup_log <- dplyr::bind_rows(log_rows)
  dedup_log$total_before <- n_start
  dedup_log$total_after  <- nrow(refs)

  if (save_log) {
    readr::write_csv(dedup_log, "dedup_log.csv")
  }

  # ── Final summary ─────────────────────────────────────────────────────────────
  n_total_removed <- n_start - nrow(refs)

  if (verbose) {
    cli::cli_alert_success(
      orm_msg("dedup_done", lang,
           n_unique        = nrow(refs),
           n_total_removed = n_total_removed)
    )
  }

  # Update class and attributes
  class(refs) <- c("orisma_refs", "tbl_df", "tbl", "data.frame")
  attr(refs, "orisma_stage")       <- "deduped"
  attr(refs, "orisma_lang")        <- lang
  attr(refs, "dedup_n_start")      <- n_start
  attr(refs, "dedup_n_doi")        <- n_doi_removed
  attr(refs, "dedup_n_title")      <- n_title_removed
  attr(refs, "dedup_n_fuzzy")      <- n_fuzzy_removed
  attr(refs, "dedup_n_total")      <- n_total_removed
  attr(refs, "dedup_n_unique")     <- nrow(refs)
  attr(refs, "dedup_log")          <- dedup_log

  refs
}


# ── Deduplication helpers ─────────────────────────────────────────────────────

#' @noRd
.normalise_title <- function(x) {
  x <- tolower(x)
  x <- iconv(x, to = "ASCII//TRANSLIT")   # remove accents
  x <- gsub("[^a-z0-9 ]", "", x)          # keep only alphanumeric + spaces
  x <- trimws(gsub("\\s+", " ", x))       # collapse multiple spaces
  x
}

#' @noRd
.extract_first_author <- function(authors) {
  # Authors stored as "Last, First; Last2, First2" or "Last First, Last2 First2"
  first <- stringr::str_extract(authors, "^[^;,]+")
  tolower(trimws(first))
}

#' Manual validation assistant with Cohen's Kappa
#'
#' @description
#' `orm_validate()` supports methodological validation of ORISMA's automatic
#' risk extraction by presenting a random sample of classified records for
#' manual review. It then computes **Cohen's Kappa** to measure agreement
#' between automatic and manual classification.
#'
#' This addresses a key peer-review concern: distinguishing between
#' "category detected by dictionary" and "risk actually evaluated in study".
#'
#' The function saves a CSV file pre-filled with automatic classifications
#' that the researcher edits manually, then re-loads for Kappa computation.
#'
#' @param mx An `orisma_matrix` object from [orm_extract()].
#' @param n_sample Integer. Number of records to sample. Default `30`.
#' @param out_dir Character. Directory to save validation files.
#' @param validation_file Character or NULL. Path to a completed validation
#'   CSV (output of a previous `orm_validate()` call) for Kappa computation.
#'   If NULL, creates the file for manual review.
#' @param seed Integer. Random seed for reproducibility. Default `42`.
#' @param lang Character. `"en"` or `"es"`.
#' @param verbose Logical.
#'
#' @return If `validation_file` is NULL: invisibly returns the path to the
#'   validation CSV. If `validation_file` is provided: returns a data frame
#'   with Kappa statistics per category.
#'
#' @export
orm_validate <- function(mx,
                          n_sample        = 30L,
                          out_dir         = "orisma_validation",
                          validation_file = NULL,
                          seed            = 42L,
                          lang            = getOption("orisma.lang", "en"),
                          verbose         = getOption("orisma.verbose", TRUE)) {

  .check_lang(lang)
  if (!inherits(mx, "orisma_matrix")) {
    stop("'mx' must be an orisma_matrix object from orm_extract().", call. = FALSE)
  }

  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)

  # ── Mode 1: Create validation file ──────────────────────────────────────────
  if (is.null(validation_file)) {

    set.seed(seed)
    refs    <- mx$refs
    bin_mat <- mx$matrix
    n_total <- nrow(refs)
    n_samp  <- min(n_sample, n_total)

    # Sample records with at least one category detected (more informative)
    has_cat <- which(rowSums(bin_mat) > 0)
    no_cat  <- which(rowSums(bin_mat) == 0)

    # 80% from records with categories, 20% from those without
    n_with <- min(round(n_samp * 0.8), length(has_cat))
    n_with <- max(n_with, 0)
    n_without <- min(n_samp - n_with, length(no_cat))

    idx <- c(
      if (n_with > 0) sample(has_cat, n_with) else integer(0),
      if (n_without > 0) sample(no_cat, n_without) else integer(0)
    )
    idx <- sort(idx)

    # Build validation dataframe
    sample_refs <- refs[idx, ]
    sample_mat  <- bin_mat[idx, , drop = FALSE]
    cat_names   <- colnames(bin_mat)

    # Active categories only (those with at least 1 detection in sample)
    active_cats <- cat_names[colSums(sample_mat) > 0]

    val_df <- data.frame(
      record_id   = sample_refs$record_id,
      title       = sample_refs$title,
      abstract    = substr(as.character(sample_refs$abstract), 1, 500),
      stringsAsFactors = FALSE
    )

    # Add automatic classification columns
    for (cat in active_cats) {
      col_name <- paste0("auto_", cat)
      val_df[[col_name]] <- as.integer(sample_mat[, cat])
    }

    # Add manual classification columns (pre-filled with NA for researcher)
    for (cat in active_cats) {
      col_name <- paste0("manual_", cat)
      val_df[[col_name]] <- NA_integer_
    }

    # Add notes column
    val_df$notes <- NA_character_

    # Save
    val_path <- file.path(out_dir, "orisma_validation_sample.csv")
    readr::write_csv(val_df, val_path)

    if (verbose) {
      cli::cli_alert_success(paste0(
        if (lang == "es") "Fichero de validacion creado: " else "Validation file created: ",
        val_path
      ))
      cli::cli_alert_info(paste0(
        if (lang == "es")
          paste0("Rellene las columnas 'manual_*' con 0 o 1 para ", n_samp,
                 " articulos y ejecute orm_validate() con validation_file=")
        else
          paste0("Fill in the 'manual_*' columns with 0 or 1 for ", n_samp,
                 " articles and run orm_validate() with validation_file="),
        val_path
      ))
    }

    return(invisible(val_path))
  }

  # ── Mode 2: Compute Kappa from completed file ────────────────────────────────
  val_df <- readr::read_csv(validation_file, show_col_types = FALSE)

  auto_cols   <- names(val_df)[grepl("^auto_",   names(val_df))]
  manual_cols <- names(val_df)[grepl("^manual_", names(val_df))]
  cat_keys    <- gsub("^auto_", "", auto_cols)

  # Filter rows with manual annotations
  has_manual <- rowSums(!is.na(val_df[, manual_cols, drop = FALSE])) > 0
  val_df     <- val_df[has_manual, ]
  n_annotated <- nrow(val_df)

  if (n_annotated == 0) {
    cli::cli_alert_warning(
      if (lang == "es") "No hay anotaciones manuales en el fichero."
      else "No manual annotations found in the file."
    )
    return(invisible(NULL))
  }

  # Compute Kappa per category
  kappa_results <- lapply(seq_along(cat_keys), function(i) {
    cat_key    <- cat_keys[i]
    auto_col   <- paste0("auto_",   cat_key)
    manual_col <- paste0("manual_", cat_key)

    auto   <- as.integer(val_df[[auto_col]])
    manual <- as.integer(val_df[[manual_col]])

    # Only rows where manual annotation exists
    valid <- !is.na(manual)
    if (sum(valid) < 5) return(NULL)

    auto_v   <- auto[valid]
    manual_v <- manual[valid]

    # Cohen's Kappa
    n      <- length(auto_v)
    p_obs  <- sum(auto_v == manual_v) / n

    p_yes_auto   <- mean(auto_v)
    p_yes_manual <- mean(manual_v)
    p_exp <- p_yes_auto * p_yes_manual +
             (1 - p_yes_auto) * (1 - p_yes_manual)

    kappa <- if (p_exp == 1) 1 else (p_obs - p_exp) / (1 - p_exp)

    data.frame(
      category     = cat_key,
      n_annotated  = sum(valid),
      p_agreement  = round(p_obs, 3),
      kappa        = round(kappa, 3),
      kappa_interp = dplyr::case_when(
        kappa >= 0.8 ~ if(lang=="es") "Excelente" else "Excellent",
        kappa >= 0.6 ~ if(lang=="es") "Bueno" else "Good",
        kappa >= 0.4 ~ if(lang=="es") "Moderado" else "Moderate",
        kappa >= 0.2 ~ if(lang=="es") "Debil" else "Fair",
        TRUE         ~ if(lang=="es") "Pobre" else "Poor"
      ),
      n_auto_pos   = sum(auto_v),
      n_manual_pos = sum(manual_v),
      stringsAsFactors = FALSE
    )
  })

  kappa_df <- dplyr::bind_rows(Filter(Negate(is.null), kappa_results))
  kappa_df <- kappa_df[order(-kappa_df$kappa), ]

  # Global Kappa (pooled)
  global_kappa <- round(mean(kappa_df$kappa, na.rm = TRUE), 3)

  if (verbose) {
    cli::cli_alert_success(paste0(
      if (lang == "es") "Kappa global: " else "Global Kappa: ",
      global_kappa,
      " (", n_annotated, " ",
      if (lang == "es") "registros anotados)" else "annotated records)"
    ))
    cli::cli_alert_info(
      if (lang == "es")
        "Kappa >= 0.7 es aceptable para publicacion en revistas de alto impacto"
      else
        "Kappa >= 0.7 is acceptable for publication in high-impact journals"
    )
  }

  # Save kappa results
  kappa_path <- file.path(out_dir, "orisma_kappa_results.csv")
  readr::write_csv(kappa_df, kappa_path)

  attr(kappa_df, "global_kappa")  <- global_kappa
  attr(kappa_df, "n_annotated")   <- n_annotated
  class(kappa_df) <- c("orisma_kappa", "data.frame")
  kappa_df
}


#' Print method for orisma_kappa
#' @param x An `orisma_kappa` object.
#' @param ... Further arguments (ignored).
#' @return Invisibly returns `x`.
#' @export
print.orisma_kappa <- function(x, ...) {
  cat("\n-- ORISMA Validation . Cohen Kappa --\n")
  cat(" Global Kappa:     ", attr(x, "global_kappa"), "\n")
  cat(" Records annotated:", attr(x, "n_annotated"), "\n\n")
  cat(" Kappa by category:\n")
  print(x[, c("category", "n_annotated", "p_agreement", "kappa", "kappa_interp")],
        row.names = FALSE)
  invisible(x)
}

#' Abstract Sufficiency Score (ASS)
#'
#' @description
#' `orm_ass()` computes an **Abstract Sufficiency Score** (0-5) for each
#' record, measuring how much preventively useful information the abstract
#' contains for an occupational health practitioner.
#'
#' The score is **cumulative and hierarchical** - a record cannot reach
#' level N without satisfying all previous levels:
#'
#' - **0** Non-informative abstract for OHS purposes
#' - **1** Mentions a hazard or risk, but no occupational context
#' - **2** Mentions occupational/workplace context
#' - **3** Mentions exposure measurement or quantification
#' - **4** Mentions exposure in workers with some result
#' - **5** Mentions exposure, worker population, method AND control/prevention
#'
#' @param mx An `orisma_matrix` object from [orm_extract()].
#' @param text_col Character. Text field to score. Default `"abstract"`,
#'   falls back to `"title"` if abstract is mostly empty.
#' @param lang Character. `"en"` or `"es"`.
#' @param verbose Logical.
#'
#' @return The `orisma_matrix` object with added columns:
#'   `ass_score` (0-5), `ass_label` (descriptive label),
#'   `ass_level_reached` (highest level passed).
#'
#' @export
orm_ass <- function(mx,
                    text_col = "abstract",
                    lang     = getOption("orisma.lang", "en"),
                    verbose  = getOption("orisma.verbose", TRUE)) {

  .check_lang(lang)
  if (!inherits(mx, "orisma_matrix"))
    stop("'mx' must be an orisma_matrix object from orm_extract().", call. = FALSE)

  is_es <- lang == "es"
  refs  <- mx$refs

  # Select text column
  if (!text_col %in% names(refs) ||
      mean(is.na(refs[[text_col]]) | refs[[text_col]] == "") > 0.5) {
    text_col <- "title"
    if (verbose) cli::cli_alert_warning(
      if(is_es) "Abstract no disponible, usando titulo (scores seran mas bajos)"
      else "Abstract not available, using title (scores will be lower)"
    )
  }

  text_all <- tolower(as.character(refs[[text_col]]))
  text_all[is.na(text_all)] <- ""

  # ── Level definitions (cumulative) ───────────────────────────────────────────

  # Level 1: mentions a hazard/risk term
  pat_l1 <- paste0("\\b(",
    "hazard|risk|exposure|toxic|carcinogen|nanoparticle|fume|dust|noise|",
    "vibration|radiation|chemical|biological|ergonomic|psychosocial|",
    "emission|particle|contaminant|pollutant|agent|substance",
    ")\\b")

  # Level 2: occupational/workplace context
  pat_l2 <- paste0("\\b(",
    "occupational|workplace|worker|workers|employee|employees|operator|",
    "operators|workforce|personnel|staff|professional|industrial|",
    "work.related|work environment|manufacturing|factory|plant|",
    "laboratory worker|healthcare worker|construction worker",
    ")\\b")

  # Level 3: exposure measurement/quantification
  pat_l3 <- paste0("\\b(",
    "measured|measurement|concentration|level|levels|monitored|monitoring|",
    "sampling|sampl|personal exposure|area sampling|breathing zone|",
    "biomonitoring|biological monitoring|urinary|blood level|",
    "TWA|STEL|OEL|TLV|PEL|mg.m|ug.m|ppm|ppb|dB|fiber.cm|",
    "quantif|detected|assess|assessed|evaluated|characteriz",
    ")\\b")

  # Level 4: worker exposure with result
  pat_l4_worker <- paste0("\\b(",
    "worker exposure|workers were exposed|workers exposed|",
    "exposed workers|occupational exposure|personal exposure|",
    "operator exposure|employee exposure|workforce exposure|",
    "breathing zone|inhalation exposure|dermal exposure|",
    "biological monitoring of workers",
    ")\\b")
  pat_l4_result <- paste0("\\b(",
    "result|results|found|showed|demonstrated|indicated|revealed|",
    "exceeded|below|above|higher|lower|significant|associated|",
    "increased|decreased|elevated|risk assessment|health effect|",
    "adverse effect|symptom|disease|injury",
    ")\\b")

  # Level 5: exposure + population + method + control/prevention
  pat_l5_method <- paste0("\\b(",
    "cross.sectional|cohort|case.control|longitudinal|randomized|",
    "survey|questionnaire|interview|observational|experimental|",
    "study design|n=|sample size|participants|subjects|",
    "NIOSH|OSHA method|ISO method|EN standard|sampling method",
    ")\\b")
  pat_l5_prevention <- paste0("\\b(",
    "prevention|preventive|control measure|engineering control|",
    "administrative control|PPE|personal protective equipment|",
    "ventilation|enclosure|substitution|elimination|hierarchy of controls|",
    "protective measure|safety measure|intervention|recommendation|",
    "guideline|limit value|exposure limit|protective action|",
    "risk management|mitigation|reduce exposure|lower exposure",
    ")\\b")

  # ── Compute scores (cumulative) ───────────────────────────────────────────────
  scores <- vapply(seq_along(text_all), function(i) {
    txt <- text_all[i]
    if (nchar(txt) < 20) return(0L)

    l1 <- grepl(pat_l1, txt, perl = TRUE)
    if (!l1) return(0L)

    l2 <- grepl(pat_l2, txt, perl = TRUE)
    if (!l2) return(1L)

    l3 <- grepl(pat_l3, txt, perl = TRUE)
    if (!l3) return(2L)

    l4 <- grepl(pat_l4_worker, txt, perl = TRUE) &&
          grepl(pat_l4_result,  txt, perl = TRUE)
    if (!l4) return(3L)

    l5 <- grepl(pat_l5_method,     txt, perl = TRUE) &&
          grepl(pat_l5_prevention, txt, perl = TRUE)
    if (!l5) return(4L)

    return(5L)
  }, integer(1))

  # ── Labels ────────────────────────────────────────────────────────────────────
  labels <- dplyr::case_when(
    scores == 0 ~ if(is_es) "0 - No informativo para PRL"
                  else "0 - Non-informative for OHS",
    scores == 1 ~ if(is_es) "1 - Menciona riesgo sin contexto laboral"
                  else "1 - Mentions hazard, no occupational context",
    scores == 2 ~ if(is_es) "2 - Contexto laboral presente"
                  else "2 - Occupational context present",
    scores == 3 ~ if(is_es) "3 - Menciona exposicion o medicion"
                  else "3 - Mentions exposure or measurement",
    scores == 4 ~ if(is_es) "4 - Exposicion en trabajadores con resultado"
                  else "4 - Worker exposure with result",
    scores == 5 ~ if(is_es) "5 - Abstract completo: exposicion + poblacion + metodo + prevencion"
                  else "5 - Complete: exposure + population + method + prevention",
    TRUE        ~ "?"
  )

  # ── Attach to refs ────────────────────────────────────────────────────────────
  mx$refs$ass_score <- scores
  mx$refs$ass_label <- labels

  if (verbose) {
    cli::cli_alert_success(paste0(
      if(is_es) "ASS calculado para " else "ASS computed for ",
      nrow(mx$refs), if(is_es) " registros" else " records"
    ))
    dist_tbl <- table(scores)
    for (s in 0:5) {
      n <- if (as.character(s) %in% names(dist_tbl))
             dist_tbl[as.character(s)] else 0L
      pct <- round(100 * n / nrow(mx$refs), 1)
      cli::cli_alert_info(paste0(
        "  Level ", s, ": ", n, " (", pct, "%) - ",
        switch(as.character(s),
          "0" = if(is_es) "no informativo" else "non-informative",
          "1" = if(is_es) "riesgo sin contexto" else "hazard no context",
          "2" = if(is_es) "contexto laboral" else "occupational context",
          "3" = if(is_es) "medicion" else "measurement",
          "4" = if(is_es) "exposicion trabajadores" else "worker exposure",
          "5" = if(is_es) "completo" else "complete"
        )
      ))
    }
  }

  mx
}


#' Plot ASS distribution
#'
#' @description
#' Generates a bar chart showing the distribution of Abstract Sufficiency
#' Scores across the corpus.
#'
#' @param mx An `orisma_matrix` object after running [orm_ass()].
#' @param out_dir Character or NULL. Directory to save the plot.
#' @param lang Character. `"en"` or `"es"`.
#'
#' @return A ggplot2 object invisibly.
#' @export
orm_ass_plot <- function(mx,
                          out_dir = NULL,
                          lang    = getOption("orisma.lang", "en")) {

  if (!"ass_score" %in% names(mx$refs))
    stop("Run orm_ass() first.", call. = FALSE)

  is_es <- lang == "es"

  df <- mx$refs %>%
    dplyr::count(.data$ass_score) %>%
    dplyr::mutate(
      pct   = round(100 * .data$n / sum(.data$n), 1),
      label_s = dplyr::case_when(
        .data$ass_score == 0 ~ if(is_es) "0\nNo informativo" else "0\nNon-informative",
        .data$ass_score == 1 ~ if(is_es) "1\nRiesgo sin\ncontexto" else "1\nHazard no\ncontext",
        .data$ass_score == 2 ~ if(is_es) "2\nContexto\nlaboral" else "2\nOccupational\ncontext",
        .data$ass_score == 3 ~ if(is_es) "3\nMedicion\nexposicion" else "3\nExposure\nmeasurement",
        .data$ass_score == 4 ~ if(is_es) "4\nExposicion\ntrabajadores" else "4\nWorker\nexposure",
        .data$ass_score == 5 ~ if(is_es) "5\nCompleto" else "5\nComplete",
        TRUE ~ as.character(.data$ass_score)
      ),
      fill_col = dplyr::case_when(
        .data$ass_score <= 1 ~ "#D85A30",
        .data$ass_score == 2 ~ "#E8A838",
        .data$ass_score == 3 ~ "#F5C842",
        .data$ass_score == 4 ~ "#4DAF8D",
        .data$ass_score == 5 ~ "#0F6E56",
        TRUE ~ "#888"
      )
    )

  p <- ggplot2::ggplot(df, ggplot2::aes(
    x    = factor(.data$ass_score),
    y    = .data$n,
    fill = .data$fill_col
  )) +
    ggplot2::geom_col(show.legend = FALSE, width = 0.7) +
    ggplot2::geom_text(ggplot2::aes(
      label = paste0(.data$n, "\n(", .data$pct, "%)")
    ), vjust = -0.3, size = 3.5, colour = "grey30") +
    ggplot2::scale_fill_identity() +
    ggplot2::scale_x_discrete(labels = setNames(df$label_s, as.character(df$ass_score))) +
    ggplot2::labs(
      title    = if(is_es) "Distribucion del Abstract Sufficiency Score (ASS)"
                 else "Abstract Sufficiency Score (ASS) distribution",
      subtitle = if(is_es)
        paste0("N = ", sum(df$n), " registros . 0 = no informativo . 5 = completo para PRL")
      else
        paste0("N = ", sum(df$n), " records . 0 = non-informative . 5 = complete for OHS"),
      x = "ASS",
      y = if(is_es) "Numero de registros" else "Number of records"
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      panel.grid.major.x = ggplot2::element_blank(),
      plot.subtitle = ggplot2::element_text(size = 9, colour = "grey40")
    )

  if (!is.null(out_dir)) {
    if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)
    ggplot2::ggsave(file.path(out_dir, "ass_distribution.png"), p,
                    width = 10, height = 6, dpi = 300)
  }

  invisible(p)
}

#' Generate a guided extraction matrix for manual review
#'
#' @description
#' `orm_extraction_matrix()` generates a structured extraction template
#' pre-filled with automatically extracted information. The practitioner
#' completes the remaining fields using the full PDF.
#'
#' Articles are selected and ranked by combined bridge score + ASS score.
#' The matrix contains auto-filled bibliographic data, ORISMA scores,
#' detected technology and risk categories, and empty fields for manual
#' completion with full-text PDFs.
#'
#' @param mx An `orisma_matrix` object after [orm_bridge()] and [orm_ass()].
#' @param result An `orisma_result` object from [orm_run()].
#' @param top_n Integer. Max articles to include. Default `30`.
#' @param min_bridge_score Integer. Min bridge score. Default `2`.
#' @param out_dir Character. Output directory.
#' @param lang Character. `"en"` or `"es"`.
#' @param verbose Logical.
#'
#' @return Invisibly returns the path to the saved CSV.
#' @export
orm_extraction_matrix <- function(mx,
                                   result,
                                   top_n            = 30L,
                                   min_bridge_score = 2L,
                                   out_dir          = "orisma_output",
                                   lang             = getOption("orisma.lang", "en"),
                                   verbose          = getOption("orisma.verbose", TRUE)) {

  .check_lang(lang)
  if (!inherits(mx, "orisma_matrix"))
    stop("'mx' must be an orisma_matrix object.", call. = FALSE)
  if (!inherits(result, "orisma_result"))
    stop("'result' must be an orisma_result object.", call. = FALSE)
  if (!"bridge_score" %in% names(mx$refs))
    stop("Run orm_bridge() first.", call. = FALSE)

  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)

  is_es   <- lang == "es"
  refs    <- mx$refs
  bin_mat <- mx$matrix

  # Priority score
  ass_score      <- if ("ass_score" %in% names(refs)) refs$ass_score else 0L
  bridge_score   <- refs$bridge_score
  n_cats         <- rowSums(bin_mat)
  priority_score <- (bridge_score * 2) + (ass_score * 1.5) + (n_cats * 0.5)

  refs_ranked <- refs %>%
    dplyr::mutate(
      priority_score = priority_score,
      ass_score_col  = ass_score,
      n_categories   = n_cats
    ) %>%
    dplyr::filter(.data$bridge_score >= min_bridge_score) %>%
    dplyr::arrange(dplyr::desc(.data$priority_score)) %>%
    dplyr::slice_head(n = top_n)

  if (nrow(refs_ranked) == 0) {
    cli::cli_alert_warning(
      if(is_es) "Sin articulos con bridge_score >= min_bridge_score. Reduzca el umbral."
      else "No articles with bridge_score >= min_bridge_score. Reduce threshold."
    )
    return(invisible(NULL))
  }

  # Risk categories per article
  cat_labels <- result$indicators$label
  names(cat_labels) <- result$indicators$category
  bin_sub <- bin_mat[refs_ranked$record_id, , drop = FALSE]
  detected_risks <- apply(bin_sub, 1, function(row) {
    detected <- names(row)[row == 1L]
    labels   <- cat_labels[detected]
    paste(labels[!is.na(labels)], collapse = "; ")
  })

  # Auto-detect technology
  text_all <- tolower(as.character(refs_ranked$abstract))
  text_all[is.na(text_all)] <- ""

  tech_patterns <- list(
    "PBF/SLM/LPBF"  = "powder bed|selective laser|SLM|LPBF|laser powder",
    "DED"            = "directed energy|DED|WAAM|laser metal deposition",
    "EBM"            = "electron beam melting|EBM",
    "FDM/FFF"        = "fused deposition|FDM|FFF|fused filament",
    "Welding"        = "welding|arc welding|MIG|TIG",
    "Nanotechnology" = "nanotechnology|nanomaterial|nanoparticle",
    "Healthcare"     = "hospital|healthcare|clinical|medical",
    "Construction"   = "construction|demolition|renovation",
    "Mining"         = "mining|underground|quarry",
    "Agriculture"    = "agriculture|farm|pesticide"
  )

  auto_tech <- vapply(text_all, function(txt) {
    detected <- vapply(names(tech_patterns), function(nm)
      grepl(tech_patterns[[nm]], txt, perl = TRUE, ignore.case = TRUE),
      logical(1))
    if (any(detected)) paste(names(tech_patterns)[detected], collapse = "; ")
    else if(is_es) "No detectada" else "Not detected"
  }, character(1))

  # Clean authors (handle list columns from synthesisr)
  clean_authors <- vapply(refs_ranked$authors, function(a) {
    if (is.list(a)) paste(unlist(a), collapse = "; ")
    else as.character(a)
  }, character(1))

  # Complete placeholder
  ph <- if(is_es) "[COMPLETAR]" else "[COMPLETE]"
  ph_yn <- if(is_es) "[SI/NO/QUIZA]" else "[YES/NO/MAYBE]"

  # Build matrix — all character to avoid type conflicts
  em <- data.frame(
    record_id        = refs_ranked$record_id,
    title            = as.character(refs_ranked$title),
    authors          = clean_authors,
    year             = as.character(refs_ranked$year),
    doi              = as.character(refs_ranked$doi),
    source_db        = as.character(refs_ranked$source_db),
    bridge_type      = as.character(refs_ranked$bridge_type),
    bridge_score     = as.character(refs_ranked$bridge_score),
    bridge_criteria  = as.character(refs_ranked$bridge_criteria),
    ass_score        = as.character(refs_ranked$ass_score_col),
    priority_score   = as.character(round(refs_ranked$priority_score, 1)),
    auto_technology  = auto_tech,
    auto_risk_cats   = as.character(detected_risks[refs_ranked$record_id]),
    study_design     = ph,
    study_population = ph,
    n_participants   = ph,
    exposure_agent   = ph,
    exposure_level   = ph,
    exposure_units   = ph,
    exposure_method  = ph,
    main_result      = ph,
    health_effect    = ph,
    prevention_recs  = ph,
    limitations      = ph,
    quality_score    = ph,
    include_final    = ph_yn,
    reviewer_notes   = ph,
    stringsAsFactors = FALSE
  )

  mat_path <- file.path(out_dir, "orisma_extraction_matrix.csv")
  readr::write_csv(em, mat_path, na = "")

  if (verbose) {
    cli::cli_alert_success(paste0(
      if(is_es) "Matriz guardada: " else "Matrix saved: ", mat_path
    ))
    cli::cli_alert_info(paste0(
      nrow(em), if(is_es) " articulos incluidos" else " articles included"
    ))
    n_strong <- sum(em$bridge_type %in% c("Strong bridge", "Puente fuerte"))
    cli::cli_alert_info(paste0(
      "  ", if(is_es) "Puentes fuertes: " else "Strong bridges: ", n_strong,
      " | ", if(is_es) "Otros: " else "Others: ", nrow(em) - n_strong
    ))
    cli::cli_alert_info(
      if(is_es)
        "Complete las columnas [COMPLETAR] usando los PDFs completos."
      else
        "Complete the [COMPLETE] columns using full-text PDFs."
    )
  }

  invisible(mat_path)
}

---
title: "ORISMA: Mapping Occupational Risk Evidence in Metal Additive Manufacturing"
author: 
  - "PhD. Raúl Aguilar-Elena (GPRL, Universidad Internacional de Valencia)"
  - "Ana Delgado-García (Universidad de Salamanca)"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
    number_sections: true
vignette: >
  %\VignetteIndexEntry{ORISMA: Mapping Occupational Risk Evidence in Metal Additive Manufacturing}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  fig.width = 8,
  fig.height = 5,
  out.width = "100%",
  warning   = FALSE,
  message   = FALSE,
  eval      = FALSE
)
```

# Introduction

Metal additive manufacturing (AM) -- commonly known as 3D metal printing --
represents one of the most rapidly expanding advanced manufacturing technologies
of the 21st century. Its occupational health implications are still poorly
understood, partly because the scientific literature is growing faster than the
capacity of occupational health professionals to synthesise and apply it.

`orisma` (**O**ccupational **R**isk **I**ntegrated **S**ystematic **M**apping and
**A**nalysis) addresses this gap by automating the complete pipeline from
raw bibliographic exports to actionable risk intelligence -- in a single
function call.

This vignette demonstrates the full ORISMA workflow using a systematic corpus
of 184 records (114 unique after deduplication) retrieved from Web of Science
and Scopus on the topic of occupational health and safety in metal additive
manufacturing (2015-2026).

## What ORISMA does

ORISMA automates seven analytical steps:

1. **Ingestion** -- reads RIS/BibTeX/CSV files from multiple databases
2. **Deduplication** -- three-step pipeline (DOI + title + fuzzy matching)
3. **Risk extraction** -- dictionary-based classification into 58 normative categories
4. **Bibliometric analysis** -- computes WRDI, RCS, and MGP indicators
5. **Dimension detection** -- automatic discovery of normative blocks (A-F)
6. **Preventive intelligence** -- Abstract Sufficiency Score (ASS) and bridge article detection
7. **Report generation** -- bilingual HTML reports, risk sheets, extraction matrices

## Installation

```{r install, eval = FALSE}
# From CRAN
install.packages("orisma")

# Development version
remotes::install_github("Aguilar-Elena/orisma")
```

---

# The three-line workflow

The entire analysis pipeline runs in three lines of R:

```{r three_lines}
library(orisma)

refs   <- orm_load("path/to/ris/files/")
result <- orm_run(refs, topic = "Metal additive manufacturing - OHS")
orm_report(result, lang = "en", out_dir = "my_outputs/")
```

The remainder of this vignette unpacks what happens inside each step and
interprets the results from the metal AM corpus.

---

# Step 1: Loading and deduplication

## Loading references

`orm_load()` reads all RIS, BibTeX, and CSV files from a folder, detects the
source database automatically, and returns a standardised `orisma_refs` object.

```{r load}
library(orisma)

refs <- orm_load(
  path = "path/to/ris/files/",
  lang = "en"
)
```

In the metal AM corpus, two files were loaded:

- `Wos.ris` -- 73 records from Web of Science
- `scopusris.ris` -- 111 records from Scopus
- **Total: 184 records identified**

## Deduplication

`orm_dedup()` runs a three-step deduplication pipeline:

1. **Exact DOI match** -- records with identical DOIs are deduplicated
2. **Normalised title match** -- titles are lowercased, stripped of punctuation, and matched exactly
3. **Fuzzy match** -- remaining potential duplicates are identified using Levenshtein distance with a configurable threshold (default: 0.90 similarity)

```{r dedup}
# Deduplication runs automatically inside orm_run()
# but can be called independently:
deduped <- orm_dedup(refs, fuzzy_threshold = 0.90, verbose = TRUE)
```

**Result for the metal AM corpus:**

| Step | Duplicates removed | Records remaining |
|------|-------------------|-------------------|
| Exact DOI | 70 | 114 |
| Normalised title | 0 | 114 |
| Fuzzy match | 0 | 114 |

**70 duplicates (38.0%)** were removed, yielding **114 unique records**.
The high overlap between WoS and Scopus confirms the importance of
multi-database deduplication in systematic reviews of this topic.

---

# Step 2: Risk category extraction

## The ORISMA dictionary

Risk categories are extracted using a built-in normative dictionary of
**58 categories** organised in six blocks anchored in ISO 45001:2018,
INSST, NIOSH, and EU-OSHA:

```{r dict}
dict <- orm_dict()
orm_dict_categories(dict)
```

| Block | Categories | Normative anchors |
|-------|-----------|-------------------|
| A -- Safety at work | 18 | INSST / ISO 45001 |
| B -- Industrial hygiene | 8 | INSST / NIOSH |
| C -- Ergonomics | 8 | INSST / ISO 45001 |
| D -- Psychosociology | 11 | INSST / ISO 45001 |
| E -- Biological hazards | 5 | EU-OSHA / NIOSH |
| F -- Emerging technologies | 8 | EU-OSHA 2024-2026 |

The dictionary can be extended for any domain:

```{r dict_extend}
# Add terms to an existing category
dict <- orm_dict_add_terms(dict, "nanomaterials",
                           c("nano-aerosol", "NOAA particle"))

# Add a completely new category
dict <- orm_dict_add_category(dict,
  key      = "laser_safety",
  label_en = "Laser safety in AM processes",
  label_es = "Seguridad laser en procesos AM",
  terms    = c("laser safety", "laser hazard", "laser exposure")
)
```

## Extraction results

```{r extract}
mx <- orm_extract(deduped, dict = dict,
                  fields  = c("title", "abstract", "keywords"),
                  verbose = TRUE)
print(mx)
```

**22 of 58 categories** were detected in the metal AM corpus. The top five
by frequency:

| Category | N records | % |
|----------|-----------|---|
| Additive manufacturing and 3D printing | 113 | 99.1% |
| Exposure to hazardous chemical agents | 61 | 53.5% |
| Nanomaterials and nanotechnology | 28 | 24.6% |
| Exposure to carcinogens, mutagens, reprotoxics (CMR) | 15 | 13.2% |
| Exposure to ionising radiation | 10 | 8.8% |

Note that "Additive manufacturing and 3D printing" (Block F) dominates the
corpus -- this is the **technological context** of the corpus, not a risk
category per se. ORISMA's `orm_priority()` function automatically separates
context categories from risk categories.

---

# Step 3: The three original ORISMA indicators

## Worker-Risk Disconnection Index (WRDI)

The WRDI measures the proportion of studies characterising an occupational risk
**without** including direct worker exposure data. It is computed at both the
category level and globally.

$$WRDI_c = 1 - \frac{N_{workers,c}}{N_{total,c}}$$

where $N_{workers,c}$ is the number of studies in category $c$ that include
worker exposure terms (e.g. "worker exposure", "breathing zone", "personal
sampling", "field study") and $N_{total,c}$ is the total number of studies.

```{r wrdi}
result <- orm_run(refs, topic = "Metal additive manufacturing - OHS")
cat("Global WRDI:", result$WRDI_global, "\n")
```

**Global WRDI = 0.6053** for the metal AM corpus. This means that
**60.5% of studies** characterise occupational risks without including
direct worker exposure data. The literature is predominantly technical
-- characterising emissions, particles, and processes in controlled
settings -- without measuring what workers actually breathe or absorb.

WRDI by category reveals important heterogeneity:

| Category | WRDI | Interpretation |
|----------|------|---------------|
| Indoor air quality | 0.22 | Best connected to workers |
| Exposure to ionising radiation | 0.30 | Good worker data |
| Exposure to hazardous chemical agents | 0.34 | Moderate |
| Nanomaterials and nanotechnology | 0.39 | Moderate |
| Biological hazard - Bacteria | 1.00 | No worker data at all |
| Artificial intelligence and autonomous systems | 1.00 | No worker data |

## Risk Category Saturation Index (RCS)

The RCS measures the relative dominance of each risk category compared
to a hypothetical uniform distribution:

$$RCS_c = \frac{N_c \cdot K}{N_{total}}$$

where $K$ = 58 categories. RCS > 1 indicates over-representation;
RCS < 1 indicates under-representation.

```{r rcs}
ind <- result$indicators[result$indicators$n_records > 0, ]
ind[order(-ind$RCS), c("label", "n_records", "RCS")]
```

The metal AM corpus shows extreme saturation:

- **Additive manufacturing and 3D printing: RCS = 24.1** -- massively
  over-represented (the dominant topic of the corpus)
- **Exposure to hazardous chemical agents: RCS = 13.0** -- strongly
  over-represented
- **Most other categories: RCS < 1** -- under-represented or absent

This pattern confirms that the literature has concentrated on chemical
emissions while largely ignoring safety, ergonomic, psychosocial, and
biological risks.

## Material-Gap Profile (MGP)

The MGP identifies materials that are hazardous but understudied:

$$MGP_m = \frac{hazard\_proxy_m}{coverage_m}$$

```{r mgp}
print(result$MGP)
```

| Material | Coverage | Hazard proxy | MGP |
|----------|----------|--------------|-----|
| Titanium | 2.9% | 1.33 | **45.3** |
| Aluminium | 8.8% | 1.11 | 12.6 |
| Nickel/Inconel | 8.8% | 0.89 | 10.1 |
| Steel | 17.6% | 1.33 | 7.6 |
| Others/Mixed | 61.8% | 0.89 | 1.4 |

**Titanium has the highest MGP (45.3)**. Despite its known carcinogenic
and respiratory hazard potential, it appears in only 2.9% of corpus
studies. This represents the most critical material gap in the field.

---

# Step 4: Automatic dimension detection

`orm_autodim()` automatically discovers the normative blocks present in
the corpus without any user configuration. In "blocks" mode (default), it
uses the six normative blocks (A-F) as dimensions:

```{r autodim}
dims <- orm_autodim(result$mx, method = "blocks", verbose = TRUE)
print(dims)
```

For the metal AM corpus, all six blocks are present, with Block F
(Emerging technologies) dominating (n=113) followed by Block B
(Industrial hygiene, n=66).

`orm_dim_matrix()` then builds a risk category x normative block
cross-matrix, visualised as a hierarchical clustered heatmap:

```{r dim_matrix}
mat <- orm_dim_matrix(result, dims,
                      out_dir = "outputs/plots/",
                      lang    = "en")
```

The heatmap reveals that most studies addressing chemical agents (Block B)
also address the AM technology context (Block F), while safety risks
(Block A) are largely absent from the literature.

---

# Step 5: Preventive intelligence indicators

## Abstract Sufficiency Score (ASS)

The ASS is a cumulative 0-5 index of preventive informativeness:

```{r ass}
mx  <- orm_ass(result$mx, verbose = TRUE)
orm_ass_plot(mx, lang = "en")
```

**Distribution for the metal AM corpus (N=114):**

| Level | N | % | Meaning |
|-------|---|---|---------|
| 0 | 28 | 24.6% | Non-informative for OHS |
| 1 | 12 | 10.5% | Mentions hazard, no workplace context |
| 2 | 27 | 23.7% | Occupational context present |
| 3 | 24 | 21.1% | Exposure measurement reported |
| 4 | 20 | 17.5% | Worker exposure with result |
| 5 | 3 | 2.6% | Complete: exposure + population + method + prevention |

**Mean ASS = 2.04/5**. Only 3 abstracts (2.6%) contain complete
preventive information. Nearly one quarter (24.6%) contain no
OHS-useful information at all. This quantifies the practical gap
that practitioners face when consulting the primary literature.

## Bridge article detection

```{r bridge}
mx <- orm_bridge(result$mx, verbose = TRUE)
```

**Bridge classification:**

| Type | N | % |
|------|---|---|
| Strong bridge (score 4-5) | 22 | 19.3% |
| Partial bridge (score 3) | 4 | 3.5% |
| Technical study | 88 | 77.2% |

**77.2% of studies are purely technical** -- they do not simultaneously
address workers, exposure measurement, AND preventive recommendations.
Only 22 studies (19.3%) qualify as strong bridges between technical
science and applied OHS prevention.

## Priority reading ranking

```{r ranking}
ranking <- orm_ranking(result$mx, top_n = 10, lang = "en")
print(ranking)
```

The top-ranked article (*Additive Manufacturing for Occupational Hygiene*,
Stefaniak et al. 2021) achieves a perfect bridge score of 5/5 and ASS
of 5/5, with a combined priority score of 21. This is the single most
valuable article for a practitioner entering this field.

---

# Step 6: Practitioner outputs

## Risk sheet

The `orm_risk_sheet()` function generates a structured, actionable risk
sheet for OHS practitioners. It is regulation-neutral (applicable
globally) and includes the traffic-light priority classification:

```{r risk_sheet}
orm_risk_sheet(result,
  topic           = "Metal additive manufacturing",
  search_strategy = "Systematic search in WoS and Scopus (2015-2026).",
  out_dir         = "outputs/",
  lang            = "en"
)
```

The risk sheet:

- Separates **context categories** (RCS > 15, dominant topic) from
  actual **risk categories**
- Classifies each risk as **RED** (critical gap), **AMBER** (attention),
  **GREEN** (reasonable coverage), or **GREY** (insufficient evidence)
- Includes a full **methodological section** with WRDI definition,
  extraction method, deduplication statistics, and limitations

## Guided extraction matrix

For systematic reviewers, `orm_extraction_matrix()` generates a
pre-filled CSV template ready for full-text review:

```{r extraction}
orm_extraction_matrix(result$mx, result,
  top_n   = 20,
  out_dir = "outputs/",
  lang    = "en"
)
```

The matrix pre-fills bibliographic data, ORISMA scores, auto-detected
technology, and risk categories. Reviewers only need to complete the
PDF-dependent fields (study design, population, exposure level, main
result, preventive recommendations).

---

# Step 7: Validation

ORISMA's automatic classification should be validated against manual
review before publication. `orm_validate()` supports this:

```{r validate}
# Step 1: Generate validation sample
val_path <- orm_validate(result$mx,
  n_sample = 30,
  out_dir  = "outputs/validation/",
  lang     = "en"
)

# Step 2: Open the CSV, fill in manual_* columns (0 or 1)
# Step 3: Compute Cohen's Kappa
kappa_results <- orm_validate(result$mx,
  validation_file = val_path,
  out_dir         = "outputs/validation/",
  lang            = "en"
)
print(kappa_results)
```

A Kappa >= 0.7 is considered acceptable for publication in high-impact
OHS journals.

---

# Summary of results

The ORISMA analysis of 114 unique studies on occupational health in
metal additive manufacturing reveals:

1. **High technical-preventive disconnection** (WRDI = 0.61): 60.5% of
   studies lack direct worker exposure data. The evidence base is
   predominantly technical.

2. **Extreme category saturation**: Chemical agents (RCS = 13.0) and
   the AM technology context (RCS = 24.1) dominate the literature.
   Safety, ergonomic, and psychosocial risks are virtually absent.

3. **Critical material gap**: Titanium (MGP = 45.3) is the most
   hazardous and least-studied material. A priority target for future
   research.

4. **Low abstract informativeness** (mean ASS = 2.04/5): Most abstracts
   describe technical processes without conveying worker exposure data.
   Only 2.6% of abstracts are fully informative for OHS practitioners.

5. **Few bridge articles** (22/114, 19.3%): Only one in five studies
   connects technical characterisation with real worker data and
   preventive recommendations.

These findings demonstrate both the utility of ORISMA for rapid
evidence mapping and the significant methodological gap in the
occupational health literature on metal AM.

---

# Session information

```{r session}
sessionInfo()
```

---

# References

Aguilar-Elena, R. & Delgado-Garcia, A. (2025). *Mapping the Safety
Landscape of Emerging Technologies: A Bibliometric Analysis of
Occupational Risks in Metal Additive Manufacturing*. [Under review]

International Organization for Standardization. (2018). *ISO 45001:2018
Occupational health and safety management systems -- Requirements with
guidance for use*. ISO.

National Institute for Safety and Health at Work (INSST). (2023).
*Clasificacion de los accidentes de trabajo por forma/contacto*.
Ministerio de Trabajo y Economia Social, Spain.

National Institute for Occupational Safety and Health (NIOSH). (2023).
*NIOSH Pocket Guide to Chemical Hazards*. CDC/NIOSH.

European Agency for Safety and Health at Work (EU-OSHA). (2023).
*Foresight on new and emerging occupational safety and health risks*.
EU-OSHA.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_relevance_guard.R
\name{orm_relevance_guard}
\alias{orm_relevance_guard}
\title{Relevance guard for occupational risk evidence mapping}
\usage{
orm_relevance_guard(
  data,
  topic = NULL,
  topic_regex = NULL,
  occupational_regex = NULL,
  noise_regex = NULL,
  title_col = NULL,
  abstract_col = NULL,
  keywords_col = NULL,
  mode = c("conservative", "flag", "strict")
)
}
\arguments{
\item{data}{A data frame of bibliographic records.}

\item{topic}{Optional topic label used to derive a topic-specific regular expression.}

\item{topic_regex}{Optional regular expression defining the target technology/topic.}

\item{occupational_regex}{Optional regular expression defining occupational relevance.}

\item{noise_regex}{Optional regular expression defining likely off-topic biomedical/clinical noise.}

\item{title_col}{Optional title column name. If \code{NULL}, it is detected automatically.}

\item{abstract_col}{Optional abstract column name. If \code{NULL}, it is detected automatically.}

\item{keywords_col}{Optional keywords column name. If \code{NULL}, it is detected automatically.}

\item{mode}{Relevance filtering mode. \code{"flag"} excludes only records outside
the target topic and marks uncertain records for review. \code{"conservative"}
excludes off-topic and likely non-occupational biomedical/clinical records.
\code{"strict"} also excludes records with weak occupational context.}
}
\value{
The input data frame with additional relevance-control columns.
}
\description{
Adds a relevance-control layer before ORISMA analysis. The function identifies
whether each record is relevant to the target topic, whether it contains an
occupational context, whether it is likely to be biomedical or clinical noise,
and whether it should be excluded from the main occupational analysis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_dict.R
\name{orm_dict_add_category}
\alias{orm_dict_add_category}
\title{Add a new risk category to a dictionary}
\usage{
orm_dict_add_category(
  dict,
  key,
  label_en,
  label_es,
  terms,
  worker_exposure_terms = character(0),
  taxonomy = "user",
  block = "G - Custom"
)
}
\arguments{
\item{dict}{An \code{orisma_dict} object.}

\item{key}{Character. Short identifier (no spaces).}

\item{label_en}{Character. Category name in English.}

\item{label_es}{Character. Category name in Spanish.}

\item{terms}{Character vector. Search terms.}

\item{worker_exposure_terms}{Character vector. Worker exposure indicators.}

\item{taxonomy}{Character. Source taxonomy label.}

\item{block}{Character. Block label (e.g. "A - Safety").}
}
\value{
Updated \code{orisma_dict}.
}
\description{
Add a new risk category to a dictionary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_dict.R
\name{print.orisma_dict}
\alias{print.orisma_dict}
\title{Print method for orisma_dict}
\usage{
\method{print}{orisma_dict}(x, ...)
}
\arguments{
\item{x}{An \code{orisma_dict} object.}

\item{...}{Further arguments (ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print method for orisma_dict
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_priority.R
\name{orm_priority}
\alias{orm_priority}
\title{Compute risk priority scores and traffic light classification}
\usage{
orm_priority(
  result,
  min_records = 2L,
  wrdi_high = 0.7,
  wrdi_low = 0.3,
  context_rcs_threshold = 15,
  lang = getOption("orisma.lang", "en")
)
}
\arguments{
\item{result}{An \code{orisma_result} object.}

\item{min_records}{Integer. Min records for evaluation. Default \code{2}.}

\item{wrdi_high}{Numeric. WRDI threshold for high disconnection. Default \code{0.7}.}

\item{wrdi_low}{Numeric. WRDI threshold for low disconnection. Default \code{0.3}.}

\item{context_rcs_threshold}{Numeric. RCS above which a category is
considered a context category (dominant topic) rather than a risk.
Default \code{15}.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}
}
\value{
A list with two data frames: \verb{$risks} (priority-classified risk
categories) and \verb{$context} (dominant topic categories).
}
\description{
\code{orm_priority()} assigns a priority level to each detected risk category
using three criteria combined into a single priority score:
\itemize{
\item \strong{Frequency} (RCS): how saturated is this category in the literature
\item \strong{Disconnection} (WRDI): how far is the research from real worker data
\item \strong{Evidence volume}: number of records
}

Categories whose RCS exceeds \code{context_rcs_threshold} are flagged as
\strong{context categories} (the dominant topic of the corpus, not a risk per se)
and are reported separately rather than mixed with risk categories.

Priority levels for non-context categories:
\itemize{
\item \strong{RED}: WRDI >= wrdi_high AND RCS >= 1. Over-studied technically but
no worker data. Urgent preventive gap.
\item \strong{AMBER}: Moderate evidence OR partial worker data.
\item \strong{GREEN}: WRDI < wrdi_low. Good worker data connection.
\item \strong{GREY}: n_records < min_records. Insufficient evidence.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_analyse.R
\name{orm_analyse}
\alias{orm_analyse}
\title{Compute ORISMA bibliometric indicators and analyses}
\usage{
orm_analyse(
  mx,
  material_col = NULL,
  year_col = "year",
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{mx}{An \code{orisma_matrix} object returned by \code{\link[=orm_extract]{orm_extract()}}.}

\item{material_col}{Character. Name of the column containing material
information. If \code{NULL} (default), MGP is skipped with a warning.}

\item{year_col}{Character. Column name for publication year. Default \code{"year"}.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}

\item{verbose}{Logical. Print progress?}
}
\value{
A list (class \code{orisma_result}) with all indicators and analysis
objects ready for \code{\link[=orm_report]{orm_report()}} and visualisation functions.
}
\description{
\code{orm_analyse()} takes an extraction matrix and computes:
\itemize{
\item \strong{WRDI} - Worker-Risk Disconnection Index: the proportion of studies that
characterise a risk without measuring direct worker exposure. A WRDI of 1
means all studies are purely technical (no worker data); 0 means all
studies include direct worker exposure measurement.
\item \strong{RCS} - Risk Category Saturation Index: relative dominance of each
risk category compared to a uniform-distribution baseline. RCS > 1 means
the category is over-represented; RCS < 1 means it is under-represented.
\item \strong{MGP} - Material-Gap Profile: ratio of a material's known hazard
potential (from the literature consensus) to its proportional coverage in
the corpus. Detects hazardous materials that are academically under-studied.
}

It also computes co-occurrence matrices, temporal trends, and author networks
for visualisation.
}
\examples{
\dontrun{
refs    <- orm_load("my_references/")
deduped <- orm_dedup(refs)
mx      <- orm_extract(deduped)
result  <- orm_analyse(mx)

# View the three core indicators
result$indicators

# View WRDI
result$WRDI
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_indicators.R
\name{orisma_indicators}
\alias{orisma_indicators}
\alias{WRDI}
\alias{RCS}
\alias{MGP}
\alias{ASS}
\alias{bridge_score}
\title{ORISMA bibliometric indicators}
\description{
ORISMA implements five original bibliometric indicators designed
specifically for occupational health and safety (OHS) evidence mapping.
Three are corpus-level indicators (WRDI, RCS, MGP) and two are
record-level indicators (ASS, Bridge Score).
\subsection{1. Worker-Risk Disconnection Index (WRDI)}{
\subsection{Definition}{

The WRDI measures the proportion of studies in a corpus that characterise
an occupational risk \strong{without} including direct worker exposure data.
A study is considered to have worker exposure data if its abstract
contains terms indicating real measurement of exposure in actual workers
under real working conditions (e.g. "worker exposure", "occupational
exposure", "breathing zone", "personal sampling", "field study",
"workplace measurement").
}

\subsection{Formula}{

For a given risk category \eqn{c}:
\deqn{WRDI_c = 1 - \frac{N_{workers,c}}{N_{total,c}}}
where \eqn{N_{workers,c}} is the number of studies in category \eqn{c}
that include worker exposure data, and \eqn{N_{total,c}} is the total
number of studies in that category.

The \strong{global WRDI} is computed across all records:
\deqn{WRDI_{global} = 1 - \frac{N_{workers}}{N_{total}}}
}

\subsection{Interpretation}{
\itemize{
\item \strong{WRDI = 0}: All studies include direct worker exposure data.
The body of evidence is fully connected to real workplace conditions.
\item \strong{WRDI = 1}: No study includes worker exposure data. The entire
literature characterises the risk technically (e.g. in simulated
environments, chambers, or in vitro) without measuring real exposure
in workers.
\item \strong{WRDI >= 0.8}: Critical alert. The evidence has very low direct
preventive transferability. On-site risk assessment is essential.
\item \strong{WRDI 0.5-0.8}: Attention required. More than half the evidence
lacks worker data.
\item \strong{WRDI < 0.3}: Reasonable coverage. Most studies include worker data.
}
}

\subsection{Important limitation}{

WRDI detection is based on abstract text, not full text. Studies that
measured worker exposure but did not mention it in the abstract may
be misclassified. Manual validation via \code{\link[=orm_validate]{orm_validate()}} is recommended.
}

}

\subsection{2. Risk Category Saturation Index (RCS)}{
\subsection{Definition}{

The RCS measures the \strong{relative dominance} of a risk category in the
corpus compared to a hypothetical uniform distribution across all
categories. It identifies which categories are over-represented
(saturated) and which are under-represented (gaps) in the literature.
}

\subsection{Formula}{

\deqn{RCS_c = \frac{pct_c}{pct_{uniform}}}
where \eqn{pct_c} is the percentage of records assigned to category
\eqn{c}, and \eqn{pct_{uniform} = 100 / K} is the percentage each
category would have under a uniform distribution across all \eqn{K}
categories.

Equivalently:
\deqn{RCS_c = \frac{N_c \cdot K}{N_{total}}}
where \eqn{N_c} is the number of records in category \eqn{c},
\eqn{K} is the total number of categories, and \eqn{N_{total}} is
the total number of records.
}

\subsection{Interpretation}{
\itemize{
\item \strong{RCS > 1}: The category is over-represented relative to a uniform
baseline. The literature has concentrated disproportionately on this
risk type.
\item \strong{RCS = 1}: The category has exactly the representation expected
under a uniform distribution.
\item \strong{RCS < 1}: The category is under-represented. This risk type has
received less attention than a balanced literature would suggest.
\item \strong{RCS = 0}: No studies address this category. Complete evidence gap.
}
}

\subsection{Note}{

RCS is a relative measure. A category can have RCS > 1 with very few
absolute studies if the corpus is small or highly specialised. Always
interpret RCS together with the absolute number of records (N).
}

}

\subsection{3. Material-Gap Profile (MGP)}{
\subsection{Definition}{

The MGP is a domain-specific indicator designed for corpora where the
corpus can be stratified by material, substance, or agent. It measures
the ratio between a material's known hazard potential and its coverage
in the occupational health literature, identifying materials that are
dangerous but understudied.
}

\subsection{Formula}{

\deqn{MGP_m = \frac{hazard\_proxy_m}{coverage_m}}
where \eqn{hazard\_proxy_m} is an estimate of the material's hazard
potential (based on the number of distinct risk categories detected in
studies involving that material), and \eqn{coverage_m} is the proportion
of corpus records that address that material.
}

\subsection{Interpretation}{
\itemize{
\item \strong{High MGP}: The material is associated with multiple risk categories
but appears in few studies. Priority material for future research and
on-site risk assessment.
\item \strong{Low MGP}: The material is well-covered in the literature relative
to its known hazard profile.
\item \strong{MGP requires a material column}: The \code{material_col} parameter in
\code{\link[=orm_analyse]{orm_analyse()}} must point to a column classifying each record by
material or agent. If not available, MGP is not computed.
}
}

}

\subsection{4. Abstract Sufficiency Score (ASS)}{
\subsection{Definition}{

The ASS is a \strong{cumulative hierarchical index} (0-5) measuring how much
preventively useful information an abstract contains for an occupational
health practitioner. It is not a measure of study quality, but of
abstract informativeness for preventive purposes.

The score is strictly cumulative: a record cannot reach level N without
satisfying all previous levels.
}

\subsection{Levels}{

\describe{
\item{0 - Non-informative}{The abstract contains no hazard or risk
terms relevant to OHS. No useful preventive information.}
\item{1 - Hazard without context}{The abstract mentions a hazard or
risk agent (e.g. nanoparticles, noise, vibration) but provides no
occupational or workplace context. Could be an environmental or
laboratory study.}
\item{2 - Occupational context}{The abstract mentions workers,
employees, operators, or workplace/occupational setting. The study
is clearly situated in a work context.}
\item{3 - Exposure measurement}{The abstract reports quantitative
exposure data: concentrations, levels, measurements, or monitoring
results. Implies some form of exposure quantification.}
\item{4 - Worker exposure with result}{The abstract explicitly
reports exposure in workers (not just in the environment) with a
result (e.g. exceeded a limit, found significant association,
detected at breathing zone).}
\item{5 - Complete preventive abstract}{The abstract addresses all
four dimensions: worker population + exposure measurement + study
method/design + preventive recommendation or control measure. This
is the highest OHS informative level.}
}
}

\subsection{Computation}{

Each level is detected via regular expression patterns applied to the
abstract text. Detection is strictly cumulative: the algorithm tests
each level in sequence and stops at the first level not satisfied.
}

\subsection{Interpretation}{
\itemize{
\item \strong{Mean ASS < 2}: The corpus is predominantly technical with very
little preventive context. High priority for on-site investigation.
\item \strong{Mean ASS 2-3}: Mixed corpus. Some workplace context but limited
quantitative exposure data.
\item \strong{Mean ASS > 3}: Good preventive evidence base. Substantial
proportion of studies report actual worker exposure data.
\item \strong{ASS = 5 articles}: These are the most valuable abstracts for
practitioners and should be read in full first.
}
}

}

\subsection{5. Bridge Article Score}{
\subsection{Definition}{

A bridge article is a study that \strong{connects technical science with
applied OHS prevention}. It simultaneously addresses five dimensions
that are rarely all present in a single study:

\describe{
\item{Criterion 1 - Technology/process}{The study involves a specific
technology, industrial process, or work task (e.g. additive
manufacturing, welding, construction, healthcare).}
\item{Criterion 2 - Hazardous agent}{The study characterises a
specific hazardous agent (chemical, physical, biological, or
psychosocial).}
\item{Criterion 3 - Workers (MANDATORY)}{The study involves a real
worker population in a real workplace setting. This criterion is
mandatory for bridge classification.}
\item{Criterion 4 - Exposure measurement (MANDATORY)}{The study
quantitatively measures exposure (air sampling, biological
monitoring, dosimetry, etc.). This criterion is mandatory for
bridge classification.}
\item{Criterion 5 - Prevention/recommendation}{The study includes
preventive recommendations, control measures, or intervention
results.}
}
}

\subsection{Classification}{

\describe{
\item{Strong bridge (score 4-5)}{Meets criteria 3+4 (mandatory) plus
2 or 3 additional criteria. Highest priority for full-text reading.
These articles have already done the translation from laboratory
science to workplace prevention.}
\item{Partial bridge (score 3)}{Meets criteria 3+4 (mandatory) plus
1 additional criterion. Valuable but incomplete bridge.}
\item{Technical study (score 0-2, or missing C3/C4)}{Does not meet
the mandatory criteria. Contributes technical knowledge but lacks
direct preventive applicability.}
}
}

\subsection{Priority score}{

The overall \strong{priority reading score} used in \code{\link[=orm_ranking]{orm_ranking()}} combines
all record-level indicators:
\deqn{Priority = (Bridge \times 2) + (ASS \times 1.5) + (N_{cats} \times 0.5)}
where \eqn{N_{cats}} is the number of risk categories detected in the
record. Bridge score is weighted highest because it reflects the most
direct preventive relevance.
}

}

\subsection{References}{

The WRDI, RCS, and MGP indicators were first described in:

Aguilar-Elena, R. & Delgado-Garcia, A. (2025). \emph{Mapping the Safety
Landscape of Emerging Technologies: A Bibliometric Analysis of
Occupational Risks in Metal Additive Manufacturing}. (Under review)

The ORISMA methodological framework is described in:

Aguilar-Elena, R. & Delgado-Garcia, A. (2025). \emph{orisma: A Framework
for Occupational Risk Integrated Systematic Mapping and Analysis}.
R package version 0.1.0. Universidad Internacional de Valencia (VIU) &
Universidad de Salamanca (USAL).
}
}
\seealso{
\code{\link[=orm_analyse]{orm_analyse()}} to compute WRDI, RCS, and MGP.
\code{\link[=orm_ass]{orm_ass()}} to compute the Abstract Sufficiency Score.
\code{\link[=orm_bridge]{orm_bridge()}} to detect bridge articles.
\code{\link[=orm_ranking]{orm_ranking()}} to generate a priority reading list.
\code{\link[=orm_validate]{orm_validate()}} to validate automatic classification with Cohen's Kappa.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_dict.R
\name{orm_dict}
\alias{orm_dict}
\title{Load a risk dictionary}
\usage{
orm_dict(name = "iso45001_insst")
}
\arguments{
\item{name}{Character. Dictionary name. Default \code{"iso45001_insst"}.}
}
\value{
A named list (class \code{orisma_dict}).
}
\description{
Load a risk dictionary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_dict.R
\name{orm_dict_list}
\alias{orm_dict_list}
\title{List available built-in dictionaries}
\usage{
orm_dict_list()
}
\value{
Invisibly prints available dictionaries. Called for side effects.

A data frame with columns: \code{key}, \code{block}, \code{label}, \code{taxonomy}, \code{n_terms}.

Invisibly prints available dictionaries. Called for side effects.

A data frame with columns: \code{key}, \code{block}, \code{label}, \code{taxonomy}, \code{n_terms}.

A character vector with the names of the built-in dictionaries available in ORISMA.
}
\description{
List available built-in dictionaries
}
\details{
This function takes no arguments.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_ass.R
\name{orm_ass_plot}
\alias{orm_ass_plot}
\title{Plot ASS distribution}
\usage{
orm_ass_plot(mx, out_dir = NULL, lang = getOption("orisma.lang", "en"))
}
\arguments{
\item{mx}{An \code{orisma_matrix} object after running \code{\link[=orm_ass]{orm_ass()}}.}

\item{out_dir}{Character or NULL. Directory to save the plot.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}
}
\value{
A ggplot2 object invisibly.
}
\description{
Generates a bar chart showing the distribution of Abstract Sufficiency
Scores across the corpus.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_extraction_matrix.R
\name{orm_extraction_matrix}
\alias{orm_extraction_matrix}
\title{Generate a guided extraction matrix for manual review}
\usage{
orm_extraction_matrix(
  mx,
  result,
  top_n = 30L,
  min_bridge_score = 2L,
  out_dir = "orisma_output",
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{mx}{An \code{orisma_matrix} object after \code{\link[=orm_bridge]{orm_bridge()}} and \code{\link[=orm_ass]{orm_ass()}}.}

\item{result}{An \code{orisma_result} object from \code{\link[=orm_run]{orm_run()}}.}

\item{top_n}{Integer. Max articles to include. Default \code{30}.}

\item{min_bridge_score}{Integer. Min bridge score. Default \code{2}.}

\item{out_dir}{Character. Output directory.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}

\item{verbose}{Logical.}
}
\value{
Invisibly returns the path to the saved CSV.
}
\description{
\code{orm_extraction_matrix()} generates a structured extraction template
pre-filled with automatically extracted information. The practitioner
completes the remaining fields using the full PDF.

Articles are selected and ranked by combined bridge score + ASS score.
The matrix contains auto-filled bibliographic data, ORISMA scores,
detected technology and risk categories, and empty fields for manual
completion with full-text PDFs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_risk_sheet.R
\name{orm_risk_sheet}
\alias{orm_risk_sheet}
\title{Generate an occupational risk sheet}
\usage{
orm_risk_sheet(
  result,
  topic = "Occupational risk analysis",
  search_strategy = NULL,
  inclusion_criteria = NULL,
  out_dir = "orisma_output",
  lang = getOption("orisma.lang", "en"),
  min_records = 1L,
  context_rcs_threshold = 15,
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{result}{An \code{orisma_result} object.}

\item{topic}{Character. Technology or domain being assessed.}

\item{search_strategy}{Character or NULL. Description of the search strategy
used (databases, keywords, date range). If NULL, a placeholder is used.}

\item{inclusion_criteria}{Character or NULL. Description of inclusion/
exclusion criteria applied. If NULL, ORISMA defaults are described.}

\item{out_dir}{Character. Output directory.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}

\item{min_records}{Integer. Min records for a category to appear. Default \code{1}.}

\item{context_rcs_threshold}{Numeric. RCS threshold for context detection.
Default \code{15}.}

\item{verbose}{Logical.}
}
\value{
Invisibly returns the path to the generated HTML risk sheet.
}
\description{
\code{orm_risk_sheet()} generates a structured, actionable risk sheet for
occupational health practitioners. It synthesises ORISMA outputs into
a single HTML document that can be used as supporting evidence in a
workplace risk assessment.

The sheet is \strong{regulation-neutral}: it does not include country-specific
regulations or limit values, as these vary by jurisdiction. The practitioner
applies the relevant national/regional regulation based on the risk
categories identified.

Content:
\itemize{
\item Context analysis (dominant topic of the corpus)
\item Priority traffic light (RED/AMBER/GREEN/GREY) per risk category
\item Evidence summary with confidence level
\item Knowledge gap alerts
\item WRDI interpretation with confidence score
\item Methodological section (bases, deduplication, WRDI definition, limits)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_run_guarded.R
\name{orm_run_guarded}
\alias{orm_run_guarded}
\title{Run ORISMA with a relevance-control layer}
\usage{
orm_run_guarded(
  refs,
  topic = NULL,
  exclude_non_relevant = TRUE,
  min_records = 50,
  topic_regex = NULL,
  occupational_regex = NULL,
  noise_regex = NULL,
  mode = c("conservative", "flag", "strict"),
  ...
)
}
\arguments{
\item{refs}{A data frame of references, usually produced by \code{orm_load()}.}

\item{topic}{Topic label passed to \code{orm_relevance_guard()} and \code{orm_run()}.}

\item{exclude_non_relevant}{Logical. If \code{TRUE}, records flagged as non-relevant
are excluded before running the main ORISMA pipeline.}

\item{min_records}{Minimum number of records required after filtering. If the
filter leaves fewer records, the function stops to avoid accidental over-filtering.}

\item{topic_regex}{Optional topic regex.}

\item{occupational_regex}{Optional occupational relevance regex.}

\item{noise_regex}{Optional noise regex.}

\item{mode}{Relevance filtering mode. \code{"flag"} excludes only records outside
the target topic and marks uncertain records for review. \code{"conservative"}
excludes off-topic and likely non-occupational biomedical/clinical records.
\code{"strict"} also excludes records with weak occupational context.}

\item{...}{Additional arguments passed to \code{orm_run()}.}
}
\value{
An ORISMA result object with an added \code{relevance_guard} component.
}
\description{
Runs ORISMA after applying \code{orm_relevance_guard()}. This is useful for
real-world bibliographic searches where broad database queries may retrieve
technically related but non-occupational or off-topic records.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_autodim.R
\name{orm_dim_matrix}
\alias{orm_dim_matrix}
\title{Build a risk category x dimension cross-matrix}
\usage{
orm_dim_matrix(
  result,
  dims,
  min_records = 2L,
  out_dir = NULL,
  filename = "risk_dimension_heatmap.png",
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{result}{An \code{orisma_result} object from \code{\link[=orm_analyse]{orm_analyse()}} or \code{\link[=orm_run]{orm_run()}}.}

\item{dims}{An \code{orisma_dims} object from \code{\link[=orm_autodim]{orm_autodim()}}.}

\item{min_records}{Integer. Min records for a risk category row. Default \code{2}.}

\item{out_dir}{Character or NULL. Directory to save the heatmap PNG.}

\item{filename}{Character. Output filename. Default \code{"risk_dimension_heatmap.png"}.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}

\item{verbose}{Logical.}
}
\value{
Invisibly returns the cross-matrix (risk categories x dimensions).
}
\description{
Builds a risk category x dimension cross-matrix and saves a hierarchical
clustered heatmap with dendrograms and numeric values in each cell.

When \code{dims} was built with \code{method = "blocks"}, the matrix shows
risk categories x normative blocks (A-Safety, B-Hygiene, etc.).
When \code{dims} was built with \code{method = "text"}, the matrix shows
risk categories x discovered text dimensions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_autodim.R
\name{print.orisma_dims}
\alias{print.orisma_dims}
\title{Print method for orisma_dims}
\usage{
\method{print}{orisma_dims}(x, ...)
}
\arguments{
\item{x}{An \code{orisma_dims} object.}

\item{...}{Further arguments (ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print method for orisma_dims
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_bridge.R
\name{orm_bridge}
\alias{orm_bridge}
\title{Bridge Article Detection and Priority Ranking}
\usage{
orm_bridge(
  mx,
  text_col = "abstract",
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{mx}{An \code{orisma_matrix} object, ideally after running \code{\link[=orm_ass]{orm_ass()}}.}

\item{text_col}{Character. Text field to analyse. Default \code{"abstract"}.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}

\item{verbose}{Logical.}
}
\value{
The \code{orisma_matrix} object with added columns:
\code{bridge_score} (0-5), \code{bridge_type} (Strong/Partial/Technical),
\code{bridge_criteria} (which criteria were met).
}
\description{
\code{orm_bridge()} identifies \strong{bridge articles} - studies that connect
technical science with real occupational prevention. These are the
highest-value articles for an occupational health practitioner because
they have already done the translation from laboratory to workplace.

A bridge article simultaneously mentions:
\enumerate{
\item \strong{Technology/process} (what was studied)
\item \strong{Hazardous agent} (what risk was characterised)
\item \strong{Workers} (real people in real workplaces)
\item \strong{Exposure measurement} (quantitative data)
\item \strong{Prevention/recommendation} (actionable output)
}

Articles meeting 4 or 5 criteria are classified as \strong{strong bridges}.
Articles meeting 3 criteria (must include workers + measurement) are
\strong{partial bridges}. Others are technical studies.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_dict.R
\name{orisma_dict}
\alias{orisma_dict}
\title{Built-in risk dictionaries for ORISMA}
\description{
ORISMA ships with a comprehensive normative risk dictionary anchored in four
internationally recognised taxonomies:
\itemize{
\item \strong{INSST} Instituto Nacional de Seguridad y Salud en el Trabajo (Spain)
\item \strong{ISO 45001:2018} International OHS management system standard
\item \strong{NIOSH} National Institute for Occupational Safety and Health (USA)
\item \strong{EU-OSHA} European Agency for Safety and Health at Work
}

The dictionary covers 58 risk categories organised in 6 blocks:
A) Safety at work (18), B) Industrial hygiene (8), C) Ergonomics (8),
D) Psychosociology (11), E) Biological hazards (5),
F) Emerging technologies (8).
}
\examples{
# View available dictionaries
orm_dict_list()

# Load the default dictionary
dict <- orm_dict()

# View all categories
orm_dict_categories(dict)

# Add custom terms to a category
dict <- orm_dict_add_terms(dict, "nanomaterials",
                           c("metal powder", "powder bed"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_run.R
\name{orm_run}
\alias{orm_run}
\title{Run the complete ORISMA pipeline in one call}
\usage{
orm_run(
  refs,
  dict = orm_dict(),
  topic = NULL,
  autodim_method = "blocks",
  material_col = NULL,
  year_col = "year",
  fuzzy_threshold = 0.9,
  fields = c("title", "abstract", "keywords"),
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE),
  save_report = FALSE,
  out_dir = getOption("orisma.out_dir", "orisma_output")
)
}
\arguments{
\item{refs}{An \code{orisma_refs} object from \code{\link[=orm_load]{orm_load()}}.}

\item{dict}{An \code{orisma_dict} object. Default: \code{\link[=orm_dict]{orm_dict()}}.}

\item{topic}{Character. Domain or technology being analysed (e.g.
'Noise in construction', 'Metal AM'). Used in plot subtitles and
report headers. If NULL, neutral generic text is used.}

\item{autodim_method}{Character. \code{"blocks"} (default) or \code{"text"}.}

\item{material_col}{Character or NULL. Column for MGP. Default NULL.}

\item{year_col}{Character. Year column. Default \code{"year"}.}

\item{fuzzy_threshold}{Numeric. Deduplication threshold. Default \code{0.90}.}

\item{fields}{Character vector. Text fields for extraction. Default
\code{c("title", "abstract", "keywords")}.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}

\item{verbose}{Logical. Default \code{TRUE}.}

\item{save_report}{Logical. Auto-call \code{\link[=orm_report]{orm_report()}}? Default \code{FALSE}.}

\item{out_dir}{Character. Output directory if \code{save_report = TRUE}.}
}
\value{
An \code{orisma_result} object containing all indicators, analyses,
dimensions (\code{result$dims}), extraction matrix (\code{result$mx}),
ASS scores and bridge classification (in \code{result$mx$refs}),
and priority ranking (\code{result$ranking}).
}
\description{
\code{orm_run()} is the \strong{single-function entry point} for a complete ORISMA
analysis. It runs all pipeline steps automatically:
\enumerate{
\item Deduplication (3-step: DOI + title + fuzzy)
\item Risk category extraction (dictionary-based)
\item Bibliometric analysis (WRDI, RCS, MGP indicators)
\item Automatic dimension detection (normative blocks)
\item Abstract Sufficiency Score (ASS, 0-5)
\item Bridge article detection and priority ranking
}
\subsection{Minimal usage (3 lines)}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(orisma)
refs   <- orm_load("my_references/")
result <- orm_run(refs)
orm_report(result, lang = "es")
}\if{html}{\out{</div>}}

All intermediate objects are stored in the result for downstream use
with \code{\link[=orm_report]{orm_report()}}, \code{\link[=orm_risk_sheet]{orm_risk_sheet()}}, \code{\link[=orm_ranking]{orm_ranking()}}, and
\code{\link[=orm_extraction_matrix]{orm_extraction_matrix()}}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_autodim.R
\name{orm_autodim}
\alias{orm_autodim}
\title{Automatic dimension extraction and risk cross-matrix}
\usage{
orm_autodim(
  mx,
  method = "blocks",
  text_col = "abstract",
  n_dims = 12L,
  min_freq = 3L,
  max_doc_pct = 0.35,
  min_cooccur = 0.5,
  fuzzy_sim = 0.85,
  stopwords = NULL,
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{mx}{An \code{orisma_matrix} object from \code{\link[=orm_extract]{orm_extract()}}.}

\item{method}{Character. \code{"blocks"} (default) or \code{"text"}.}

\item{text_col}{Character. Text field for \code{method = "text"}. Default \code{"abstract"}.}

\item{n_dims}{Integer. Max dimensions for \code{method = "text"}. Default \code{12}.}

\item{min_freq}{Integer. Min document frequency for \code{method = "text"}. Default \code{3}.}

\item{max_doc_pct}{Numeric (0-1). Max document proportion for \code{method = "text"}.
Terms above this are too generic. Default \code{0.35}.}

\item{min_cooccur}{Numeric (0-1). Min co-occurrence with a risk. Default \code{0.5}.}

\item{fuzzy_sim}{Numeric (0-1). Fuzzy grouping threshold. Default \code{0.85}.}

\item{stopwords}{Character vector. Extra stopwords for \code{method = "text"}.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}

\item{verbose}{Logical.}
}
\value{
A list (class \code{orisma_dims}) ready for \code{\link[=orm_dim_matrix]{orm_dim_matrix()}}.
}
\description{
\code{orm_autodim()} automatically discovers the most relevant contextual
dimensions of a corpus using two complementary modes:

\strong{Mode 1: Dictionary blocks} (default, \code{method = "blocks"})
Uses the normative blocks of the ORISMA dictionary (A-Safety, B-Hygiene,
C-Ergonomics, D-Psychosociology, E-Biological, F-Emerging) as dimensions.
Computes a block x block co-occurrence matrix showing how many studies
address combinations of risk blocks simultaneously. Works for any corpus
without any configuration.

\strong{Mode 2: Free text} (\code{method = "text"})
Extracts discriminant terms from abstracts using TF-IDF-like filtering.
Useful for discovering domain-specific dimensions not covered by the
dictionary (e.g. specific materials, sectors, tasks).
}
\seealso{
\code{\link[=orm_dim_matrix]{orm_dim_matrix()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_dict.R
\name{orm_dict_categories}
\alias{orm_dict_categories}
\title{List risk categories in a dictionary}
\usage{
orm_dict_categories(dict, lang = getOption("orisma.lang", "en"))
}
\arguments{
\item{dict}{An \code{orisma_dict} object.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}
}
\value{
A data frame containing the available risk categories, including category keys, labels, blocks and dictionary metadata.
}
\description{
List risk categories in a dictionary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_validate.R
\name{orm_validate}
\alias{orm_validate}
\title{Manual validation assistant with Cohen's Kappa}
\usage{
orm_validate(
  mx,
  n_sample = 30L,
  out_dir = "orisma_validation",
  validation_file = NULL,
  seed = 42L,
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{mx}{An \code{orisma_matrix} object from \code{\link[=orm_extract]{orm_extract()}}.}

\item{n_sample}{Integer. Number of records to sample. Default \code{30}.}

\item{out_dir}{Character. Directory to save validation files.}

\item{validation_file}{Character or NULL. Path to a completed validation
CSV (output of a previous \code{orm_validate()} call) for Kappa computation.
If NULL, creates the file for manual review.}

\item{seed}{Integer. Random seed for reproducibility. Default \code{42}.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}

\item{verbose}{Logical.}
}
\value{
If \code{validation_file} is NULL: invisibly returns the path to the
validation CSV. If \code{validation_file} is provided: returns a data frame
with Kappa statistics per category.
}
\description{
\code{orm_validate()} supports methodological validation of ORISMA's automatic
risk extraction by presenting a random sample of classified records for
manual review. It then computes \strong{Cohen's Kappa} to measure agreement
between automatic and manual classification.

This addresses a key peer-review concern: distinguishing between
"category detected by dictionary" and "risk actually evaluated in study".

The function saves a CSV file pre-filled with automatic classifications
that the researcher edits manually, then re-loads for Kappa computation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_analyse.R
\name{print.orisma_result}
\alias{print.orisma_result}
\title{Print method for orisma_result}
\usage{
\method{print}{orisma_result}(x, ...)
}
\arguments{
\item{x}{An object to print.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns the input \code{orisma_result} object. Called primarily for its console-printing side effect.
}
\description{
Print method for orisma_result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_report.R
\name{orm_report}
\alias{orm_report}
\title{Generate all ORISMA outputs and reports}
\usage{
orm_report(
  result,
  topic = NULL,
  lang = getOption("orisma.lang", "en"),
  out_dir = getOption("orisma.out_dir", "orisma_output"),
  formats = c("html", "csv", "plots", "certificate"),
  min_records = 1L,
  top_n = 8L,
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{result}{An \code{orisma_result} object from \code{\link[=orm_analyse]{orm_analyse()}} or \code{\link[=orm_run]{orm_run()}}.}

\item{topic}{Character. Domain or technology being analysed. Used in plot
subtitles and report headers. If NULL, neutral generic text is used.}

\item{lang}{Character. \code{"en"} or \code{"es"}. Report language.}

\item{out_dir}{Character. Output directory. Created if it does not exist.}

\item{formats}{Character vector. Which outputs to generate.
Options: \code{"html"}, \code{"csv"}, \code{"plots"}, \code{"certificate"}. Default: all.}

\item{min_records}{Integer. Minimum records for a category to appear in
plots. Default \code{1}.}

\item{top_n}{Integer. Number of top categories to show in temporal plot.
Default \code{8}.}

\item{verbose}{Logical. Print progress?}
}
\value{
Invisibly returns the output directory path.
}
\description{
\code{orm_report()} takes a completed \code{orisma_result} object and generates
the full set of outputs including improved visualisations and a rich
bilingual HTML executive report.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_extract.R
\name{orm_extract}
\alias{orm_extract}
\title{Extract risk categories from bibliographic records}
\usage{
orm_extract(
  refs,
  dict = orm_dict(),
  fields = c("title", "abstract", "keywords"),
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{refs}{An \code{orisma_refs} object (output of \code{\link[=orm_load]{orm_load()}} or \code{\link[=orm_dedup]{orm_dedup()}}).}

\item{dict}{An \code{orisma_dict} object. Default: \code{\link[=orm_dict]{orm_dict()}} (ISO 45001 /
INSST / NIOSH).}

\item{fields}{Character vector. Which text fields to search. Default
\code{c("title", "abstract", "keywords")}.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}

\item{verbose}{Logical. Print progress?}
}
\value{
A list (class \code{orisma_matrix}) containing:
\describe{
\item{\code{refs}}{Original \code{orisma_refs} tibble with added columns:
one binary column per risk category (\verb{cat_*}), \code{n_categories} (total
categories matched), and \code{has_worker_data} (logical).}
\item{\code{matrix}}{Pure binary matrix (records x categories) for
downstream analysis.}
\item{\code{dict}}{The dictionary used.}
\item{\code{categories}}{Category metadata tibble.}
}
}
\description{
\code{orm_extract()} scans the \strong{title}, \strong{abstract}, and \strong{keywords} of each
record against the active risk dictionary and builds a \strong{binary presence
matrix} (record x risk category). It also detects whether each study
contains direct worker exposure data - the key signal for computing the
\strong{WRDI} indicator.

Matching is case-insensitive and uses whole-word boundary detection to
avoid false positives (e.g. "laser" does not match "eyelaser").
}
\examples{
\dontrun{
refs   <- orm_load("my_references/")
deduped <- orm_dedup(refs)

# Use default dictionary
mx <- orm_extract(deduped)

# Use a customised dictionary
dict <- orm_dict()
dict <- orm_dict_add_terms(dict, "nanoparticles", c("nano-dust", "UFP"))
mx   <- orm_extract(deduped, dict = dict)

# Restrict to title + abstract only
mx <- orm_extract(deduped, fields = c("title", "abstract"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orisma-package.R
\docType{package}
\name{orisma-package}
\alias{orisma-package}
\alias{orisma}
\title{orisma: Occupational Risk Integrated Systematic Mapping and Analysis}
\description{
\code{orisma} is an R package for systematic bibliometric mapping of
occupational risk evidence. It is designed to help researchers and
occupational safety and health practitioners analyse whether the scientific
literature on a given topic is connected to workers, workplaces, exposure
conditions and preventive decision-making.
}
\details{
ORISMA provides a complete workflow for occupational risk evidence mapping:

\itemize{
\item multi-source bibliographic ingestion;
\item deduplication;
\item relevance filtering through \code{orm_relevance_guard()} and \code{orm_run_guarded()};
\item risk category extraction using a 58-category occupational risk dictionary;
\item preventive bibliometric indicators such as WRDI, RCS and MGP;
\item Abstract Sufficiency Score (ASS);
\item Bridge Article Score;
\item bilingual academic reports and practitioner-oriented risk sheets.
}

Typical workflow:

\preformatted{
library(orisma)

refs <- orm_load("my_references/")

result <- orm_run_guarded(
  refs,
  topic = "Collaborative robotics and occupational health and safety",
  mode = "conservative"
)

orm_report(result)
orm_risk_sheet(result)
}
}
\section{Original indicators}{

\itemize{
\item WRDI: Worker-Risk Disconnection Index.
\item RCS: Risk Category Saturation Index.
\item MGP: Material-Gap Profile.
\item ASS: Abstract Sufficiency Score.
\item Bridge Article Score.
}
}

\section{Citation}{

Aguilar-Elena, R., & Delgado-Garcia, A. (2026). \code{orisma}:
Occupational Risk Integrated Systematic Mapping and Analysis.
R package version 0.1.0. Universidad Internacional de Valencia (VIU)
& Universidad de Salamanca (USAL).
https://github.com/Aguilar-Elena/orisma
}

\section{Author(s)}{

Raul Aguilar-Elena \href{mailto:raguilar@universidadviu.com}{raguilar@universidadviu.com} \cr
Occupational Risk Prevention and Occupational Health Research Group (GPRL),
Universidad Internacional de Valencia (VIU), Valencia, Spain. \cr

Ana Delgado-Garcia \href{mailto:a.delgado@usal.es}{a.delgado@usal.es} \cr
Universidad de Salamanca (USAL), Salamanca, Spain.
}

\seealso{
Useful links:
\itemize{
\item \url{https://github.com/Aguilar-Elena/orisma}
\item Report bugs at \url{https://github.com/Aguilar-Elena/orisma/issues}
}
}
\author{
\strong{Maintainer}: Raúl Aguilar Elena \email{raguilar@universidadviu.com} (\href{https://orcid.org/0000-0001-8656-8155}{ORCID})

Authors:
\itemize{
  \item Ana Delgado-Garcia \email{a.delgado@usal.es}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_extract.R
\name{print.orisma_matrix}
\alias{print.orisma_matrix}
\title{Print method for orisma_matrix}
\usage{
\method{print}{orisma_matrix}(x, ...)
}
\arguments{
\item{x}{An object to print.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns the input \code{orisma_matrix} object. Called primarily for its console-printing side effect.
}
\description{
Print method for orisma_matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_normativa.R
\name{orm_normativa}
\alias{orm_normativa}
\title{Cross-reference detected risks with applicable European regulation}
\usage{
orm_normativa(result, min_records = 1L, lang = getOption("orisma.lang", "en"))
}
\arguments{
\item{result}{An \code{orisma_result} object.}

\item{min_records}{Integer. Min records for a category to be included.
Default \code{1}.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}
}
\value{
A data frame with detected categories and their applicable
regulations.
}
\description{
\code{orm_normativa()} crosses the risk categories detected by ORISMA with
the main applicable European directives and ISO standards, providing
the occupational health practitioner with a direct regulatory anchor
for each identified risk.

The regulatory database is built into ORISMA and covers EU directives,
Spanish INSST technical notes (NTP), and key ISO standards. It is
updated with each major package release.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_bridge.R
\name{orm_ranking}
\alias{orm_ranking}
\title{Generate priority reading ranking}
\usage{
orm_ranking(
  mx,
  top_n = 20L,
  out_dir = NULL,
  lang = getOption("orisma.lang", "en")
)
}
\arguments{
\item{mx}{An \code{orisma_matrix} object after running \code{\link[=orm_bridge]{orm_bridge()}} and
optionally \code{\link[=orm_ass]{orm_ass()}}.}

\item{top_n}{Integer. Number of top articles to return. Default \code{20}.}

\item{out_dir}{Character or NULL. Directory to save the ranking CSV.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}
}
\value{
A data frame with the top_n priority articles.
}
\description{
\code{orm_ranking()} produces a \strong{priority reading list} for occupational
health practitioners, ranking articles by their combined relevance score
(bridge score + ASS score + number of risk categories detected).

Articles at the top of the list are those most likely to contain
actionable preventive information and should be read first in full.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_dict.R
\name{orm_dict_add_terms}
\alias{orm_dict_add_terms}
\title{Add terms to an existing dictionary category}
\usage{
orm_dict_add_terms(dict, category, terms)
}
\arguments{
\item{dict}{An \code{orisma_dict} object.}

\item{category}{Character. Category key.}

\item{terms}{Character vector. New terms to add.}
}
\value{
Updated \code{orisma_dict}.
}
\description{
Add terms to an existing dictionary category
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_dedup.R
\name{orm_dedup}
\alias{orm_dedup}
\title{Automatic deduplication of bibliographic records}
\usage{
orm_dedup(
  refs,
  fuzzy_threshold = 0.9,
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE),
  save_log = TRUE
)
}
\arguments{
\item{refs}{An \code{orisma_refs} object returned by \code{\link[=orm_load]{orm_load()}}.}

\item{fuzzy_threshold}{Numeric (0–1). Similarity threshold for fuzzy
matching. Default \code{0.90} (90\% similarity = duplicate). Increase for
stricter matching, decrease for more aggressive deduplication.}

\item{lang}{Character. \code{"en"} or \code{"es"}. Overrides \code{orisma.lang} option.}

\item{verbose}{Logical. Print progress? Default \code{TRUE}.}

\item{save_log}{Logical. Save \code{dedup_log.csv} to working directory?
Default \code{TRUE}.}
}
\value{
An \code{orisma_refs} tibble with duplicates removed. Attributes record
deduplication statistics for inclusion in the PRISMA log.
}
\description{
\code{orm_dedup()} removes duplicate records using a \strong{three-step progressive
pipeline}:
\enumerate{
\item \strong{Exact DOI match} — most reliable signal; decisive for records with DOIs.
\item \strong{Normalised title match} — removes punctuation, accents, case, and extra
spaces before comparing; catches the same article listed with minor
typographic differences across databases.
\item \strong{Fuzzy match} — compares title + year + first author using
Optimal String Alignment distance; catches near-identical records that
escape exact matching (e.g. different journal abbreviations, truncated
author lists).
}

Only records that remain ambiguous after all three steps are flagged for
optional manual review. These are saved to \code{dedup_log.csv}.
}
\examples{
\dontrun{
refs    <- orm_load("my_references/")
deduped <- orm_dedup(refs)

# More aggressive fuzzy matching
deduped <- orm_dedup(refs, fuzzy_threshold = 0.85)

# Spanish messages, no log file
deduped <- orm_dedup(refs, lang = "es", save_log = FALSE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{orisma_sample}
\alias{orisma_sample}
\title{Sample bibliographic records for ORISMA}
\format{
A data frame with 20 rows and 9 variables:
\describe{
\item{record_id}{Character. Unique record identifier.}
\item{title}{Character. Article title.}
\item{abstract}{Character. Abstract (max 800 characters).}
\item{year}{Integer. Publication year.}
\item{doi}{Character. Digital Object Identifier.}
\item{source_db}{Character. Source database.}
\item{bridge_type}{Character. Bridge classification.}
\item{bridge_score}{Integer. Bridge score (0-5).}
\item{ass_score}{Integer. Abstract Sufficiency Score (0-5).}
}
}
\source{
Web of Science and Scopus (2015-2026).
}
\usage{
data(orisma_sample)
}
\description{
Twenty bibliographic records on occupational health in metal
additive manufacturing (2015-2026) from Web of Science and Scopus,
pre-processed with ORISMA to illustrate the full pipeline.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_load.R
\name{orm_load}
\alias{orm_load}
\title{Load bibliographic references from one or multiple files / folders}
\usage{
orm_load(
  path,
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{path}{Character. Path to a \strong{folder} containing reference files, or a
\strong{character vector} of individual file paths.}

\item{lang}{Character. Language for console messages: \code{"en"} (default) or
\code{"es"}. Overrides \code{getOption("orisma.lang")}.}

\item{verbose}{Logical. Print progress messages? Default \code{TRUE}.}
}
\value{
A tibble (class \code{orisma_refs}) with standardised columns:
\describe{
\item{\code{record_id}}{Internal unique identifier assigned by ORISMA}
\item{\code{source_file}}{Name of the original file}
\item{\code{source_db}}{Database inferred from file name or format}
\item{\code{title}}{Article title}
\item{\code{authors}}{Authors (semicolon-separated)}
\item{\code{year}}{Publication year}
\item{\code{doi}}{Digital Object Identifier (if available)}
\item{\code{abstract}}{Abstract text}
\item{\code{keywords}}{Author keywords}
\item{\code{journal}}{Journal name}
\item{\code{volume}, \code{issue}, \code{pages}}{Bibliographic location}
\item{\code{document_type}}{Article, review, conference paper, etc.}
}
}
\description{
\code{orm_load()} is the entry point of every ORISMA analysis. It reads
bibliographic files in \strong{RIS}, \strong{BibTeX}, or \strong{CSV} format from a folder
(or a vector of individual file paths), detects the format of each file
automatically, combines all records into a single tidy data frame, and
records the source database for each record.

All major bibliographic databases export to at least one supported format:\tabular{lll}{
   Database \tab Recommended format \tab Notes \cr
   Web of Science \tab RIS / Plain text \tab Max 1 000 records per batch \cr
   Scopus \tab RIS or CSV \tab Max 2 000 records per batch \cr
   PubMed \tab RIS \tab No limit \cr
   Dimensions \tab CSV or RIS \tab Max 2 500 per batch \cr
   EBSCO (CINAHL, BSC) \tab RIS \tab Up to 25 000 \cr
   ProQuest \tab RIS or BibTeX \tab Max 100 per batch \cr
   Cochrane Library \tab RIS \tab No limit \cr
   Ovid / MEDLINE \tab RIS \tab Max 1 000 per batch \cr
   ScienceDirect \tab RIS \tab No limit \cr
   The Lens (free) \tab RIS or CSV \tab No limit \cr
}
}
\examples{
\dontrun{
# Load all .ris and .bib files from a folder
refs <- orm_load("my_references/")

# Load specific files
refs <- orm_load(c("wos_results.ris", "scopus_results.csv"))

# Spanish messages
refs <- orm_load("mis_referencias/", lang = "es")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orm_ass.R
\name{orm_ass}
\alias{orm_ass}
\title{Abstract Sufficiency Score (ASS)}
\usage{
orm_ass(
  mx,
  text_col = "abstract",
  lang = getOption("orisma.lang", "en"),
  verbose = getOption("orisma.verbose", TRUE)
)
}
\arguments{
\item{mx}{An \code{orisma_matrix} object from \code{\link[=orm_extract]{orm_extract()}}.}

\item{text_col}{Character. Text field to score. Default \code{"abstract"},
falls back to \code{"title"} if abstract is mostly empty.}

\item{lang}{Character. \code{"en"} or \code{"es"}.}

\item{verbose}{Logical.}
}
\value{
The \code{orisma_matrix} object with added columns:
\code{ass_score} (0-5), \code{ass_label} (descriptive label),
\code{ass_level_reached} (highest level passed).
}
\description{
\code{orm_ass()} computes an \strong{Abstract Sufficiency Score} (0-5) for each
record, measuring how much preventively useful information the abstract
contains for an occupational health practitioner.

The score is \strong{cumulative and hierarchical} - a record cannot reach
level N without satisfying all previous levels:
\itemize{
\item \strong{0} Non-informative abstract for OHS purposes
\item \strong{1} Mentions a hazard or risk, but no occupational context
\item \strong{2} Mentions occupational/workplace context
\item \strong{3} Mentions exposure measurement or quantification
\item \strong{4} Mentions exposure in workers with some result
\item \strong{5} Mentions exposure, worker population, method AND control/prevention
}
}
